# Spica
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==
# == Copyright (C) 2005 Spica Project Kei/Fuco/Takkyun ==

# ======================================
package Spica;
use Spica::Config;
use Spica::Language;
use Spica::Driver;
use Spica::Interface;
use Spica::App;
use Spica::Object;
use base qw( Spica::RequestObject );
use vars qw( $VERSION $WEBPAGE $PRODUCT $LIBRARY $MODPERL );
# 1.01 [2006/07/04] changed Spica::Data::Object
# 1.00 [2006/03/17] changed bracket rule
# 1.00 [2006/03/16] changed BEGIN, added $MODPERL to run on mod_perl2
# 1.00 [2006/03/06] added pid to get unique id per process
# 1.00 [2006/03/06] changed structure, now Spica is subclass of Spica::RequestObject
# 1.00 [2006/03/03] changed new to be able boot witouh driver.
# 1.00 [2005/10/09] refactored
# --------------------------------------
# // defines / static variables
$PRODUCT = 'Spica';
$WEBPAGE = 'http://serennz.sakura.ne.jp/toybox/spica/';
$VERSION = '1.01';
$LIBRARY = $VERSION;
$MODPERL = undef;
sub DEFAULT_MODULE (){ 'Main' };
# --------------------------------------
# // initializer
BEGIN
{
	if ($ENV{'MOD_PERL'})
	{ # -- mod_perl
		eval{ require mod_perl2; };
		if (!$@)
		{ # Apache2 + mod_perl2
			require Apache2::RequestUtil;
			$MODPERL = 2;
		}
		else
		{ # Apache + mod_perl
			eval{ require mod_perl; };
			die('mod_perl or mod_perl2 is required to run:' . $@) if ($@);
			require Apache;
			$MODPERL = 1;
		}
	}
}
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	my $name = shift;
	my %param = (
		'without_db' => undef,
		@_
	);
	$self->SUPER::initialize(
		'name'   => $name || DEFAULT_MODULE,
		'conf'   => Spica::Config->new(),
		'lang'   => undef,
		'driver' => undef,
	);
	$self->generate_accessor('name','driver','conf','lang');
	$self->lang(Spica::Language->new($self->conf->value('language')));
	$self->lang->load($self->conf->lang_resource($self->lang->language));
	if (!$param{'without_db'})
	{
		$self->driver(Spica::Driver->new($self->conf->value('driver'),%{$self->conf->value('db_param')}));
	}
	return $self;
}
sub DESTROY
{
	my $self = shift;
	$self->driver->finish if ($self->driver);
	  ($MODPERL == 2) ? ModPerl::Util::exit(0)
	: ($MODPERL == 1) ? Apache::exit(0)
	: CORE::exit(0);
}
# --------------------------------------
# // public functions - class method
sub run
{ # executer (includes construction)
	my $class = shift;
	my $spica = $class->new(@_);
	my $req = 
		  ($MODPERL == 2) ? Apache2::RequestUtil->request
		: ($MODPERL == 1) ? Apache->request
		: undef;
	my $cgi = Spica::Interface->new(
		'req'=>$req,
		'parser'=>{
			'max_data' => $spica->conf->value('max_data'),
		},
	);
	my $app_class = 'Spica::App::' . $spica->module($cgi);
	$app_class = 'Spica::App' if (!&Spica::Object::load_module($app_class));
	$app_class->new(
		'conf'   => $spica->conf,
		'lang'   => $spica->lang,
		'cgi'    => $cgi,
		'apache' => $req,
	)->run($@);
}
# --------------------------------------
# // public functions
sub pid
{ # get unique id
	my $self = shift;
	my $id = undef;
	eval
	{
		require Apache2::MPM;
		require APR::OS;
		$id = "$$." . ${ APR::OS::current_thread_id() } if (Apache2::MPM->is_threaded);
	};
	return (!$@ and defined($id)) ? $id : $$;
}
sub module
{ # check module and return module name to load
	my $self = shift;
	my $cgi  = shift;
	$self->name;
}
1;
__END__

=head1 NAME

B<Spica> - Simple Primary Interface and Core Application

=head1 SYNOPSIS

 use Spica;
 Spica->run('Main');

=head1 DESCRIPTION

B<Spica> is a simple core framework especially for CGI script.  
Spica consists of some fundamental classes.  
Spica.pm basically works as a system loader.

=head2 Spica->run( module, [without_db=>1] );

I<run> method provides booting module which subclass of Spica::App.  
I<run> method is just called usually in a CGI application.  
I<run> method includes whole application process between construction and destruction.

=head2 Spica->new( module, [without_db=>1] );

I<new> is a cunstructor of Spica.
When Spica is constructed, Spica::Config, Spica::Language are loaded.

"without_db" option indicates booting option for Spica.  
When "without_db" is set, Spica is booted without Spica::Driver.

=head1 AUTHOR

Takuya Otani <takuya.otani@gmail.com> 

=head1 COPYRIGHT & LICENSE

Copyright (C) 2005 Takuya Otani@SimpleBoxes / SerendipityNZ Ltd.

Copyright (C) 2005-2006 Kei / Fuco / Takkyun I<http://serennz.sakura.ne.jp/toybox/spica/>

Spica is distributed under the Perl Artistic Licence.
I<http://www.perl.com/pub/a/language/misc/Artistic.html>

=cut

