# Spica::App
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==
# == Copyright (C) 2005 Spica Project Kei/Fuco/Takkyun ==

# ======================================
package Spica::App;
use strict;
use Carp;
use Spica::TemplateManager;
use Spica::File;
use Spica::Time;
use Spica::Object;
use base qw( Spica::RequestObject );
use vars qw( $VERSION );
$VERSION = '0.01';
# 0.01 [2006/03/17] changed bracket rule
# 0.00 [2005/10/09] created
# --------------------------------------
# // defines / static variables
sub ERROR_TITLE          (){ 'error' };
sub DEFAULT_TEMPLATE_DIR (){ 'lib/resource/' };
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	my %param = (
		'time'   => Spica::Time->new,
		'conf'   => undef,
		'lang'   => undef,
		'cgi'    => undef,
		'apache' => undef,
		@_
	);
	$self->SUPER::initialize(%param);
	$self->generate_accessor('time','conf','lang','cgi','apache');
	return $self->_setup_application;
}
# --------------------------------------
# // public functions
sub run
{
	my $self = shift;
	$self->error(@_);
}
sub output
{
	my $self = shift;
	my $content = shift;
	$self->header($self->cgi->head);
	$self->content($content);
}
sub content
{
	my $self = shift;
	($self->apache) 
		? $self->apache->print(@_) 
		: CORE::print(@_);
}
sub header
{
	my $self = shift;
	my $header = shift;
	($self->apache) 
		? $self->apache->send_cgi_header($header)
		: CORE::print($header);
}
sub common_parts
{
	my $self = shift;
	my $cms  = shift;
	return if (!$cms or !$self);
	$cms->num(0);
	$cms->tag('site_css'=>$self->conf->value('site_css'));
	$cms->tag('site_encoding'=>$self->lang->charset);
	$cms->tag('site_lang'=>$self->lang->language);
	$cms->tag('product_version'=>$Spica::VERSION);
	$cms->tag('product_name'=>$Spica::PRODUCT);
	$cms->tag('product_webpage'=>$Spica::WEBPAGE);
	return;
}
sub error
{
	my $self = shift;
	my $message = shift;
	my $cms = Spica::TemplateManager->new( &_error_template() );
	$cms->num(0);
	$cms->tag('site_title'=>$Spica::PRODUCT . ' | ' . ERROR_TITLE);
	$cms->tag('error_title'=>ERROR_TITLE);
	$cms->tag('error_message'=>$self->lang->string($message) . join(',',caller));
	if ($@)
	{ # tracing stack
		my (@stack,$trace);
		eval { confess() };
		@stack = split("\n",$@);
		$trace = '<ul class="trace">';
		for (my $i=0;$i<@stack;$i++)
		{
			next if ($i < 2);
			my $line = $stack[$i];
			$line =~ s/called at (.+) line (\d+)/<em class="called">called at $1<\/em> <em class="line">line $2<\/em>/g;
			$trace .= '<li>' . $line . '</li>';
		}
		$trace .= '</ul>';
		$cms->tag('error_trace'=>$trace);
	}
	$self->common_parts($cms);
	$self->output($cms->output);
}
sub load_template
{
	my $self = shift;
	my %param = (
		'dir'  => $self->conf->value('resource_dir') || DEFAULT_TEMPLATE_DIR,
		'file' => undef,
		@_
	);
	my $file = Spica::File->new('path'=>$param{'dir'},'name'=>$param{'file'});
	return $file->get_text;
}
sub save_template
{
	my $self = shift;
	my %param = (
		'dir'  => $self->conf->value('resource_dir') || DEFAULT_TEMPLATE_DIR,
		'file' => undef,
		'body' => undef,
		@_
	);
	my $file = Spica::File->new('path'=>$param{'dir'},'name'=>$param{'file'});
	return $file->set_data($param{'body'});
}
# --------------------------------------
# // public functions - parsing path
sub end_host
{
	my $self = shift;
	return (split('\.',$self->cgi->host))[0];
}
sub path_info
{
	my $self = shift;
	my $path = $self->cgi->path_info;
	$path =~ s!^/!!; # remove the first letter '/' from the path
	$path;
}
sub base_path
{
	my $self = shift;
	my $path = $self->cgi->current_path;
	$path =~ s!(.*/)(.*)$!$1!;
	$path;
}
# --------------------------------------
# // private functions
sub _setup_application
{
	my $self = shift;
	$self->cgi(Spica::Interface->new('req'=>$self->apache)) if (!$self->cgi);
	$self->conf(Spica::Config->get) if (!$self->conf);
	$self->lang(Spica::Language->get) if (!$self->lang);
	return $self;
}
sub _error_template
{ # default error template
	return <<'__ERROR_TEMPLATE__';
<?xml version="1.0" encoding="{site_encoding}"?>
<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">
<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="{site_lang}">
<head>
<meta http-equiv="Content-Type" content="text/html; charset={site_encoding}" />
<meta name="generator" content="{product_name}" />
<link rel="stylesheet" type="text/css" href="{site_css}" title="default" media="screen,tv" />
<title>{site_title}</title>
</head>
<body>
<h1>{site_title}</h1>
<h2 class="error">{error_title}</h2>
<p class="msg">{error_message}</p>{error_trace}
<address>Powered by <a href="{product_webpage}">{product_name}</a> {product_version}</address>
</body>
</html>
__ERROR_TEMPLATE__
}
1;
__END__
