# Spica::App::XmlrpcBase
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==
# == Copyright (C) 2005 Spica Project Kei/Fuco/Takkyun ==

# ======================================
package Spica::App::XmlrpcBase;
use strict;
use Spica::Data;
use Spica::Text;
use Spica::Time;
use Spica::TemplateManager;
use Spica::App;
use base qw( Spica::App );
use vars qw( $VERSION );
$VERSION = '0.01';
# 0.01 [2006/03/17] changed bracket rule
# 0.00 [2006/03/04] refactored
# --------------------------------------
# // defines / static variables
sub OUTPUT_CHARSET (){ 'utf-8' };
sub OUTPUT_CODE    (){ 'utf8' };
sub CONTENT_TYPE   (){ 'text/xml' };
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	return $self->SUPER::initialize(
		'mt.supportedMethods' => \&_mt_supportedmethods,
		@_
	);
}
# --------------------------------------
# // override base class - Spica::App
sub run
{
	my $self = shift;
	$self->SUPER::run(@_) if (!$self->cgi->xmlrpc);
	my $error = undef;
	my $method = $self->{$self->_method};
	if (!defined($method) or ref($method) ne 'CODE')
	{
		$self->response(1,Spica::Text->new($self->_method)->entitized);
	}
	else
	{
		eval{ $self->response($self->$method($self->_params)); };
		$self->response(99,Spica::Text->new($@)->entitized) if ($@);
	}
}
# --------------------------------------
# // public functions
sub response
{
	my $self = shift;
	my ($err,$msg) = @_;
	my $cms = Spica::TemplateManager->new($self->_template($err));
	$msg = $self->error_code($err,$msg) if ($err);
	$cms->num(0);
	$cms->tag('site_encoding'=>OUTPUT_CHARSET);
	$cms->tag('xmlrpc_error'=>$err);
	$cms->tag('xmlrpc_message'=>$msg);
	my $output = $cms->output;
	$self->header($self->cgi->head(
		'type'    => CONTENT_TYPE,
		'charset' => OUTPUT_CHARSET,
		'length'  => length($output),
	));
	$self->content($output);
}
sub error_code
{
	my $self = shift;
	my ($err,$msg) = @_;
	my %errs = (
		0  => 'No error.',                  # works fine
		1  => 'Unknown method. [%s]',       # it's not supported
		2  => 'Too many parameters.',       # parameters more than requested
		3  => 'Failed to authorize.',       # could be wrong password
		4  => 'The object does not exist.', # no target data
		5  => 'Failed to lock a file.',     # a file could be still locked
		6  => 'Failed to upload a file.',   # uploading does not succeed
		7  => 'Permission denied.',         # it's not permitted
		8  => 'Lack of parameters.',        # parameters less than requested
		99 => 'Unknown error. %s',          # something wrong
	);
	return sprintf($errs{$err},$msg);
}
sub value
{
	my $self = shift;
	my $value = shift;
	my $out = {
		'type'  => 'string',
		'value' => undef,
	};
	$value =~ s/<value>(.*)<\/value>/$1/sg; # extract value
	$value =~ s/^[\s|\n]*(.*)[\s|\n]*$/$1/s; # get rid of spaces
	TYPE_SWITCH: {
		$_ = $value;
		/^<int/ && do {
			$out->{'type'} = 'int';
			$out->{'value'} = int($1) if ($value =~ /<int>(.*)<\/int>/s);
			last TYPE_SWITCH;
		};
		/^<i4/ && do {
			$out->{'type'} = 'int';
			$out->{'value'} = int($1); if ($value =~ /<i4>(.*)<\/i4>/s);
			last TYPE_SWITCH;
		};
		/^<boolean/ && do {
			$out->{'type'} = 'bool';
			$out->{'value'} = int($1); if ($value =~ /<boolean>(.*)<\/boolean>/s);
			last TYPE_SWITCH;
		};
		/^<double/ && do {
			$out->{'type'} = 'float';
			$out->{'value'} = $1 if ($value =~ /<double>(.*)<\/double>/s);
			last TYPE_SWITCH;
		};
		/^<dateTime\.iso8601/ && do {
			my $str = $1 if ($value =~ /<dateTime\.iso8601>(.*)<\/dateTime\.iso8601>/s);
			$out->{'type'} = 'time';
			if ($str =~ /(\d\d\d\d)-?(\d\d)-?(\d\d)T(\d\d):(\d\d):(\d\d)/)
			{
				$out->{'value'} = Spica::Time->new(
					'year'=>$1,'mon'=>$2,'day'=>$3,'hour'=>$4,'min'=>$5,'sec'=>$6,
				);
			}
			last TYPE_SWITCH;
		};
		/^<base64/ && do {
			$out->{'type'} = 'binary';
			if ($value =~ /<base64>(.*)<\/base64>/s)
			{
				require 'mimeutil.pl';
				my $buf = $1;
				$buf =~ s/\s//g; # by KIKUCHI Kouji
				$buf =~ s/(.{76})/$1\n/g;
				$out->{'value'}  = &mimeutil::bodydecode($buf,'b64');
				$out->{'value'} .= &mimeutil::bdeflush('b64');
			}
			last TYPE_SWITCH;
		};
		/^<struct/ && do {
			$out->{'type'} = 'hash';
			$out->{'value'} = &_struct($1) if ($value =~ /<struct>(.*)<\/struct>/s);
			last TYPE_SWITCH;
		};
		/^<array/ && do {
			$out->{'type'} = 'array';
			$out->{'value'} = &_array($1) if ($value =~ /<data>(.*)<\/data>/s);
			last TYPE_SWITCH;
		};
		/^<string/ && do {
			$out->{'type'} = 'string';
			$out->{'value'} = $1 if ($value =~ /<string>(.*)<\/string>/s);
			last TYPE_SWITCH;
		};
	}
	$out->{'value'} = $value if ( !defined($out->{'value'}) );
	return $out;
}
# --------------------------------------
# // private functions - xmlrpc methods
sub _mt_supportedmethods
{
	my $self = shift;
	return(2,undef) if (@_);
	my $output = '<array><data>' . "\n";
	foreach my $name ( keys(%{$self}) )
	{
		next if (ref($self->{$name}) ne 'CODE');
		next if ($name eq 'mt.supportedMethods');
		$output .= '<value><string>' . $name . '</string></value>' . "\n";
	}
	$output .= '</data></array>';
	return(0,$output);
}
# --------------------------------------
# // private functions
sub _template
{
	my $self = shift;
	my $err = shift;
	return <<'__ERROR_TEMPLATE__' if ($err);
<?xml version="1.0" encoding="{site_encoding}"?>
<methodResponse>
<fault><value><struct>
<member><name>faultCode</name><value><int>{xmlrpc_error}</int></value></member>
<member><name>faultString</name><value><string>{xmlrpc_message}</string></value></member>
</struct></value></fault>
</methodResponse>
__ERROR_TEMPLATE__
	return <<'__NORMAL_TEMPLATE__';
<?xml version="1.0" encoding="{site_encoding}"?>
<methodResponse>
<params><param><value>
{xmlrpc_message}
</value></param></params>
</methodResponse>
__NORMAL_TEMPLATE__
}
sub _method
{
	my $self = shift;
	return $self->cgi->value('methodName');
}
sub _params
{
	my $self = shift;
	return ($self->cgi->value('params') =~ m!<param>(.*)</param>!sg);
}
# --------------------------------------
# // private functions - for values
sub _struct
{
	my $input = shift;
	my $output;
	my @stack = ($input =~ /<member>(.*?)<\/member>/sg); # imperfect
	foreach my $member (@stack)
	{
		if ($member =~ /<name>(.*?)<\/name>.*<value>(.*?)<\/value>/s)
		{
			my $name = $1;
			my $value = $2;
			$output->{$name} = &_value($value);
		}
	}
	return($output);
}
sub _array
{
	my $input = shift;
	my $output;
	my $flag = 0;
	if ($input =~ /<value>[\s|\n]*<struct>/)
	{ # struct in array ?
		my @stack = ($input =~ /<value>[\s|\n]*<struct>(.*?)<\/struct>[\s|\n]*<\/value>/sg); # imperfect
		foreach my $value (@stack)
		{
			push(@{$output},&_struct($value));
		}
	}
	else
	{ # probably just an array
		my @stack = ($input =~ /<value>(.*?)<\/value>/sg); # imperfect
		foreach my $value (@stack)
		{
			push(@{$output},&_value($value));
		}
	}
	return($output);
}
1;
__END__
