# Spica::Config
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==
# == Copyright (C) 2005 Spica Project Kei/Fuco/Takkyun ==

# ======================================
package Spica::Config;
use strict;
use Spica::InitLoader;
use Spica::Object;
use base qw( Spica::SingleObject );
use vars qw( $VERSION );
$VERSION = '0.06';
# 0.06 [2006/07/04] got rid of "return( undef )" and "return undef"
# 0.05 [2006/04/06] changed load to use Spica::InitLoader
# 0.04 [2006/03/17] changed bracket rule
# 0.03 [2006/03/17] renamed lang_rsrc to lang_resource
# 0.02 [2006/03/04] changed procedure of initialization to have flexibility
# 0.01 [2005/11/14] refactored
# --------------------------------------
# // defines / static variables
my %mEnv = ();
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	eval{ require 'config.pl'; };
	$self->clear;
	$self->load;
	return $self->SUPER::initialize();
}
# --------------------------------------
# // public functions
sub value
{
	my $self = shift;
	my $key  = shift;
	return if ( !defined($key) );
	$mEnv{$key} = shift if (@_);
	return $mEnv{$key};
}
sub lang_resource
{
	my $self = shift;
	my $lang = shift || 'en';
	return $mEnv{'resource_dir'} . $lang . $mEnv{'language_ext'};
}
sub clear
{
	my $self = shift;
	%mEnv = ();
}
sub load
{
	my $self = shift;
	%mEnv = ( $self->_default_set, $self->setting );
	my $loader = Spica::InitLoader->new(
		'file' => $self->value('init_file'),
		'rule' => \&_check_key,
	);
	return $loader->load(\%mEnv);
	sub _check_key
	{
		my $key = shift;
		return if ($key =~ /\W/);
		return if (!defined($mEnv{$key}));
		return lc($key);
	}
}
# --------------------------------------
# // private functions
sub _default_set
{
	return (
		'max_data'     => 15360000,
		'driver'       => 'Text',
		'db_param'     => {},
		'language'     => 'ja',
		'local'        => 'jp',
		'language_ext' => '.txt',
		'resource_dir' => 'lib/resource/',
		'site_css'     => 'lib/resource/style.css',
		'server_path'  => 'http://127.0.0.1/',
		'init_file'    => 'init.cgi',
	);
}
# --------------------------------------
# // public functions - declaration
sub setting;
1; # end of package
__END__
