# Spica::Driver
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==
# == Copyright (C) 2005 Spica Project Kei/Fuco/Takkyun ==

# ======================================
package Spica::Driver;
use strict;
use Spica::Object;
use base qw( Spica::Object );
use vars qw( $VERSION );
$VERSION = '0.07';
# 0.07 [2006/03/31] added driver_param and table_param
# 0.06 [2006/03/29] added count
# 0.05 [2006/03/17] changed bracket rule
# 0.04 [2006/03/02] added name & create_table, changed save,load,new_id
# 0.03 [2006/02/27] added convert_type and changed finish
# 0.02 [2006/02/20] changed setup/matched_number
# 0.01 [2005/11/15] changed constructor to generate an instance correctly
# 0.00 [2005/10/25] porting from Serene Bach
# --------------------------------------
# // override base class
sub new
{
	my $class = shift;
	my $type = shift;
	die("need to set the first argument for driver type.") if ($type eq '');
	my $driver = $class->module_name($type);
	return ($class->load_module($type)) ? $driver->construct(@_) : undef;
}
# --------------------------------------
# // public functions - destructor
sub finish
{
	my $self = shift;
	undef;
}
# ----------------------------------------
# // public functions - constructor
sub construct
{
	my $class = shift;
	my $self = bless {},$class;
	return $self->setup(@_);
}
sub setup
{
	my $self = shift;
	return $self->SUPER::initialize(
		'num'    => 0,  # number of the last loaded objects
		'option' => {}, # options - declaration
		@_
	);
}
# ----------------------------------------
# // public functions - class method
sub convert_type
{
	my $class = shift;
	my $name  = shift;
	my %table = $class->table_param;
	my $type  = '';
	foreach my $key ( keys %table )
	{
		next if (index($name,$key) == -1);
		$type = $table{$key};
		$type .= $1 if ($name =~ /(\(\d+\))$/);
		last;
	}
	$type = 'UNKNOWN' if (!$type);
	return $type;
}
sub table_param
{
	my $class = shift;
	return (
		'id'   => 'ID',
		'str'  => 'STRING',
		'text' => 'TEXT',
		'date' => 'DATE',
		'zone' => 'ZONE',
		'int'  => 'INTEGER',
		'list' => 'ARRAY',
		'file' => 'FILE',
	);
}
sub driver_param
{
	my $class = shift;
	return();
}
# ----------------------------------------
# // public functions - instance method
sub matched_number
{
	my $self = shift;
	$self->{'num'};
}
sub name
{
	my $self = shift;
	return lc( (split('::',$_[0]))[-1] );
}
# --------------------------------------
# // public functions - declaration
sub load($$;@)
{
	my $self  = shift;
	my $class = shift; # get sub-class name of Spica::Data::Object
	my %param = ( # input paramaeters
		'cond'  => {},    # searching option
		'id'    => undef, # inidicates id to load directly
		'logic' => 'and', # logic factor for searching option
		'bgn'   => 0,     # offset of objects to load
		'num'   => -1,    # maximum of objects to load
		'sort'  => 'id',  # sorting option
		'order' => 0,     # sorting order 0:descent 1:ascent
		@_
	);
	undef;
}
sub count($$;@)
{
	my $self  = shift;
	my $class = shift; # get sub-class name of Spica::Data::Object
	my %param = ( # input paramaeters
		'cond'  => {},    # searching option
		'id'    => undef, # inidicates id to load directly
		'logic' => 'and', # logic factor for searching option
		'bgn'   => 0,     # offset of objects to load
		'num'   => -1,    # maximum of objects to load
		'sort'  => 'id',  # sorting option
		'order' => 0,     # sorting order 0:descent 1:ascent
		@_
	);
	undef;
}
sub save($@)
{
	my $self = shift;
	my @objs = @_; # input parameter => set objects to save
	undef;
}
sub new_id($$)
{
	my $self = shift;
	my $class = shift; # set class name of Spica::Data::Object
	undef;
}
sub create_table($$)
{
	my $self = shift;
	my $class = shift; # set class name of Spica::Data::Object
	undef;
}
1; # end of package
__END__
