# Spica::Driver::Mysql
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==
# == Copyright (C) 2005 Spica Project Kei/Fuco/Takkyun ==

# ======================================
# [note] Spica::Driver::Mysql requires DBD::mysql
package Spica::Driver::Mysql;
use strict;
use Spica::Language;
use Spica::Driver;
use Spica::Driver::SqlBase; # included 'use DBI'
use base qw( Spica::Driver::SqlBase );
use vars qw( $VERSION );
$VERSION = '0.04';
# 0.04 [2006/03/31] added driver_param and table_param
# 0.03 [2006/03/15] changed _connect to set character code correctly
# 0.02 [2006/03/04] moved basic implementations into SqlBase.pm
# 0.01 [2006/03/03] the first implementation
# 0.00 [2006/02/01] generated
# --------------------------------------
# // defines / static variables
sub DEFALUT_NAME     (){ 'Spica' };
sub DEFAULT_HOST     (){ '127.0.0.1' };
sub DEFAULT_PORT     (){ 3306 };
sub DEAFULT_SOCKET   (){ '' };
sub DEFALUT_USER     (){ 'root' };
sub DEFALUT_PASSWORD (){ '' };
# --------------------------------------
# // override base class - param
sub driver_param
{
	my $class = shift;
	return(
		'name' => DEFALUT_NAME,
		'host' => DEFAULT_HOST,
		'port' => DEFAULT_PORT,
		'sock' => DEAFULT_SOCKET,
		'user' => DEFALUT_USER,
		'pass' => DEFALUT_PASSWORD,
	);
}
sub table_param
{
	my $class = shift;
	return (
		'id'   => 'INTEGER',
		'str'  => 'VARCHAR',
		'text' => 'TEXT',
		'date' => 'INTEGER',
		'zone' => 'VARCHAR(7)',
		'int'  => 'INTEGER', 
		'list' => 'TEXT',
		'file' => 'VARCHAR(64)',
	);
}
# --------------------------------------
# // override base class
sub setup
{
	my $self = shift;
	$self->SUPER::setup(
		$self->driver_param,
		'option' => { # options - use for DBI
			'PrintError' => 0,
			'RaiseError' => 0,
		},
		@_
	);
	$self->dbh($self->_connect);
	return $self;
}
# --------------------------------------
# // override base class - implementation
sub fetch_id
{
	my $self = shift;
	my $sth = shift;
	return $sth->{'mysql_insertid'} || $sth->{'insertid'};
}
# --------------------------------------
# // public functions - class method
sub convert_type
{
	my $class = shift;
	my $name = shift;
	my $type = $class->SUPER::convert_type($name,$class->table_param);
	$type = 'VARCHAR(255)' if ($type eq 'UNKNOWN');
	$type .= ' NOT NULL AUTO_INCREMENT PRIMARY KEY' if ($name =~ /^UNI/i);
	return $type;
}
# --------------------------------------
# // private functions - instance method
sub _connect
{
	my $self = shift;
	my $lang = Spica::Language->get;
	my $handle = undef;
	my $source = 'DBI:mysql:database=' . $self->{'name'};
	$source .= ';hostname=' . $self->{'host'} if ($self->{'host'});
	$source .= ';port=' . $self->{'port'} if ($self->{'port'});
	$source .= ';mysql_socket=' . $self->{'sock'} if ($self->{'sock'});
	$handle = DBI->connect($source,$self->{'user'},$self->{'pass'},$self->{'option'});
	die($lang->string('Failed to connect database.') . ' ' . $DBI::errstr) if (!$handle);
	my $code = $lang->charcode;
	$code = ($code eq 'euc')  ? 'ujis' # euc-jp must indicate as 'ujis'
	      : ($code eq 'sjis') ? 'sjis' # shift-jis
	      : 'utf8'; # otherwise, set as utf8
	$handle->do('SET NAMES ' . $code);
	return $handle;
}
1;
__END__
