# Spica::Driver::SqlBase
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==
# == Copyright (C) 2005 Spica Project Kei/Fuco/Takkyun ==

# ======================================
package Spica::Driver::SqlBase;
use strict;
use DBI;
use Spica::Driver;
use base qw( Spica::Driver );
use vars qw( $VERSION );
$VERSION = '0.04';
# 0.04 [2006/07/04] got rid of "return( undef )"
# 0.03 [2006/03/29] added count adn count_list
# 0.02 [2006/03/17] changed bracket rule
# 0.01 [2006/03/06] changed generate_where to use SQL92 syntax
# 0.00 [2006/03/04] generated
# --------------------------------------
# // defines / static variables
sub DEFAULT_HEAD (){ 'spica_' };
# --------------------------------------
# // override base class
sub setup
{
	my $self = shift;
	return $self->SUPER::setup(
		'dbh' => undef, # database handler
		@_
	);
}
sub finish
{
	my $self = shift;
	$self->dbh->disconnect if ($self->dbh);
	undef;
}
# --------------------------------------
# // override base class - instance method
sub name
{
	my $self = shift;
	my ($class,$elem) = @_;
	return $self->SUPER::name($class) . '_' . $elem;
}
# --------------------------------------
# // override base class - implementation
sub load($$;@)
{
	my $self  = shift;
	my $class = shift; # get sub-class name of Spica::Data::Object
	my %param = ( # input paramaeters
		'cond'  => {},    # searching option
		'id'    => undef, # inidicates id to load directly
		'logic' => 'and', # logic factor for searching option
		'bgn'   => 0,     # offset of objects to load
		'num'   => -1,    # maximum of objects to load
		'sort'  => 'id',  # sorting option
		'order' => 0,     # sorting order 0:descent 1:ascent
		@_
	);
	return if ( !$self->dbh );
	return $self->load_list($class,%param);
}
sub count($$;@)
{
	my $self  = shift;
	my $class = shift; # get sub-class name of Spica::Data::Object
	my %param = ( # input paramaeters
		'cond'  => {},    # searching option
		'id'    => undef, # inidicates id to load directly
		'logic' => 'and', # logic factor for searching option
		'bgn'   => 0,     # offset of objects to load
		'num'   => -1,    # maximum of objects to load
		'sort'  => 'id',  # sorting option
		'order' => 0,     # sorting order 0:descent 1:ascent
		@_
	);
	return if ( !$self->dbh );
	return $self->count_list($class,%param);
}
sub save($@)
{
	my $self = shift;
	my @objs = @_; # input parameter => set objects to save
	return if ( $#objs == -1 );
	return if ( !$self->dbh );
	my $class = ref($objs[0]);
	return $self->save_list($class,@objs);
}
sub new_id($$)
{
	my $self = shift;
	my $class = shift; # set class name of Spica::Data::Object
	return if ( !$self->dbh );
	my $id = $self->prepare_id;
	my @target = ();
	my @values = ();
	foreach my $elem ( $class->elements )
	{
		next if ( $elem eq 'id' and !defined($id) );
		push(@target,$self->name($class,$elem));
		push(@values,($elem ne 'id') ? 'NULL' : $id);
	}
	my $sql = 'INSERT INTO ' .  $self->table_name($class);
	$sql .= ' (' . join(', ',@target) . ') ';
	$sql .= 'VALUES (' . join(', ', @values) . ')';
	my $sth = $self->dbh->prepare($sql);
	return if (!$sth);
	$sth->execute() or die($self->dbh->errstr);
	$sth->finish;
	$id = $self->fetch_id($sth) if ( !defined($id) );
	return $id;
}
sub create_table($$)
{
	my $self = shift;
	my $class = shift; # set class name of Spica::Data::Object
	return if ( !&Spica::Object::load_module($class) );
	return if ( !$self->dbh );
	my @sql = ();
	my %table = $class->table;
	foreach my $elem ( $class->elements )
	{
		my $type = $self->convert_type($table{$elem});
		push(@sql, $self->name($class,$elem) . ' ' . $type);
	}
	my $state = 'CREATE TABLE ' . $self->table_name($class);
	$state .= ' (' . join(', ',@sql) . ')';
	$self->dbh->do($state) or return;
}
# --------------------------------------
# // public functions - declaration
sub prepare_id
{ # prepare id before inserting
	my $self = shift;
	undef;
}
sub fetch_id
{ # fetch id after insering
	my $self = shift;
	undef;
}
# --------------------------------------
# // public functions - accessor
sub table_name
{
	my $self = shift;
	return DEFAULT_HEAD . $self->SUPER::name($_[0]);
}
sub dbh
{
	my $self = shift;
	$self->{'dbh'} = shift if (@_);
	$self->{'dbh'};
}
sub count_list
{
	my $self  = shift;
	my $class = shift;
	my %param = @_;
	# initialize local variables
	$param{'bgn'} = 0 if ($param{'num'} <= 0);
	my $sth = undef; # statment handler
	my $label = $self->name($class,'');
	my $logic = $self->check_logic($param{'logic'});
	my $order = $self->check_order($label,$param{'sort'},$param{'order'});
	my ($where,$place) = $self->generate_where($label,$param{'id'},$param{'cond'});
	my $sql = '';
	my $cnt = 0;
	# generate sql statement
	$sql .= 'SELECT COUNT(*) FROM ' . $self->table_name($class);
	$sql .= ' WHERE ' . join($logic,@{$where}) if (@{$where});
	$sql .= ' ORDER BY ' . $order;
	# get statement handler
	$sth = $self->dbh->prepare($sql) or die($self->dbh->errstr);
	return if (!$sth);
	$sth->execute(@{$place}) or die($self->dbh->errstr);
	$sth->bind_columns(undef,\$cnt); # the first "undef" is for backward compatibility
	$sth->fetch or return;
	$sth->finish;
	return $cnt;
}
sub load_list
{
	my $self  = shift;
	my $class = shift;
	my %param = @_;
	my @array = ();
	# initialize local variables
	$param{'bgn'} = 0 if ($param{'num'} <= 0);
	my $sth = undef; # statment handler
	my $label = $self->name($class,'');
	my $logic = $self->check_logic($param{'logic'});
	my $order = $self->check_order($label,$param{'sort'},$param{'order'});
	my ($where,$place) = $self->generate_where($label,$param{'id'},$param{'cond'});
	my $sql = '';
	my $num = -1;
	my $cnt = 0;
	# prepare binds
	my (%dat,@binds);
	foreach my $val ( $class->elements )
	{
		push(@binds,\$dat{$val});
	}
	# generate sql statement
	$sql .= 'SELECT * FROM ' . $self->table_name($class);
	$sql .= ' WHERE ' . join($logic,@{$where}) if (@{$where});
	$sql .= ' ORDER BY ' . $order;
	# get statement handler
	$sth = $self->dbh->prepare($sql) or die($self->dbh->errstr);
	return() if (!$sth);
	$sth->execute(@{$place}) or die($self->dbh->errstr);
	$sth->bind_columns(undef,@binds); # the first "undef" is for backward compatibility
	# get and count result
	while ($sth->fetch)
	{
		$num++;
		next if ($param{'bgn'} > $num);
		next if ($param{'num'} > 0 and $cnt >= $param{'num'});
		my $obj = $class->create(%dat);
		$cnt++;
		push(@array,$obj);
	}
	$sth->finish;
	$self->{'num'} = $num;
	return( @array );
}
sub save_list
{
	my $self  = shift;
	my $class = shift;
	my @objs  = @_;
	my $sth = undef; # statment handler
	my $table = $self->table_name($class);
	my @sets = grep { $_ ne 'id' } $objs[0]->elements;
	my $set  = join(', ',map { $self->name($class,$_) . '=?' } @sets);
	my $whph = ' WHERE ' . $self->name($class,'id') . '=';
	foreach my $obj ( @objs )
	{
		next if ( ref($obj) ne $class ); # ignore different object
		my $sql = '';
		my $where = $whph . $obj->id;
		if ($obj->erased)
		{
			$sql = 'DELETE FROM ' . $table . $where;
		}
		else
		{
			$sql = 'UPDATE ' . $table . ' SET ' . $set . $where;
		}
		$sth = $self->dbh->prepare($sql) or die($self->dbh->errstr);
		if (!$obj->erased)
		{
			for (my $i=0;$i<@sets;$i++)
			{
				my $elem = &_linefeed($obj->{$sets[$i]});
				$sth->bind_param($i+1,$elem);
			}
		}
		$sth->execute or die($self->dbh->errstr);
		$sth->finish;
	}
	return( 1 );
}
# --------------------------------------
# // public functions - sql tools
sub quote
{
	my $self = shift;
	return $self->dbh->quote(@_);
}
sub generate_where
{
	my $self = shift;
	my ($name,$id,$cond) = @_;
	my (@where,@place);
	if ($id ne '')
	{
		push(@where,$name . 'id=?');
		push(@place,int($id));
	}
	else
	{
		# [search string condition] ===
		my @like = ();
		foreach my $key ( keys(%{$cond}) )
		{
			next if ( $key eq '__range' or $key eq '__combo' );
			if ( $cond->{$key} =~ /^\/(.*)\/$/ )
			{
				push(@like,$name . $key . ' LIKE ?'); # almost same as /\Q$word\E/i
				push(@place,'%' . $1 . '%');
				$cond->{$key} = undef; # clear condition
			}
		}
		push(@where,'(' . join(' OR ',@like) . ')') if (@like);
		# [other conditions] ===
		foreach my $key ( keys(%{$cond}) )
		{
			next if ( $key eq '__range' or $key eq '__combo' );
			next if ( !defined($cond->{$key}) ); # skip if $cond->{$key} is undef
			if ( ref($cond->{$key}) eq 'ARRAY' )
			{ # array or range
				my $phrase = '(';
				my $label = $name . $key;
				if ( defined($cond->{'__range'}->{$key}) )
				{
					# range ( between the first value and the second value )
					if ( $cond->{'__range'}->{$key} eq 'zone' )
					{
						# timezone range, zone value should be offset of value
						$phrase .= $label . '-' . $name . 'zone>=? AND ' . $label . '-' . $name . 'zone<?';
					}
					else
					{
						$phrase .= $label . '>=? AND ' . $label . '<?';
					}
					push(@place,$cond->{$key}[0],$cond->{$key}[1]);
				}
				else
				{
					# array ( check matching one of list values )
					$phrase .= join(' OR ',map { $label . '=?' } @{$cond->{$key}});
					push(@place,@{$cond->{$key}});
				}
				$phrase .= ')';
				push(@where,$phrase);
			}
			else
			{ # normal or combo
				my $phrase = $name . $key . '=?';
				push(@place,$cond->{$key});
				if ( defined($cond->{'__combo'}->{$key}) )
				{
					my %combo = %{$cond->{'__combo'}};
					$phrase = '(' . $phrase . ' OR ';
					$phrase .= $name . $combo{$key} . ' LIKE ?';
					push(@place,'%' . $combo{$combo{$key}} . '%');
				}
				push(@where,$phrase);
			}
		} # end of [other conditions]
	}
	return(\@where,\@place);
}
sub check_logic
{
	my $self = shift;
	my $logic = shift;
	return ' OR ' if ($logic eq 'or');
	return ' AND ';
}
sub check_order
{
	my $self = shift;
	my $label = shift;
	my $check = shift;
	my $order = shift;
	my $out = $label . 'id ';
	$out = $label . $check . ',' . $out if ($check ne 'id');
	$out .= ($order) ? 'ASC' : 'DESC';
	$out;
}
# --------------------------------------
# // private functions - tools
sub _linefeed
{ # unify linefeed code
	$_[0] =~ s/\x0D\x0A/\n/g;
	$_[0] =~ tr/\x0D\x0A/\n\n/;
	while ($_[0] =~ /\n$/)
	{
		$_[0] =~ s/\n$//g;
	}
	return($_[0]);
}
1;
__END__
