# Spica::Driver::Sqlite
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==
# == Copyright (C) 2005 Spica Project Kei/Fuco/Takkyun ==

# ======================================
# [note] Spica::Driver::Sqlite requires DBD::SQLite and installation of SQLite
package Spica::Driver::Sqlite;
use strict;
use Spica::Language;
use Spica::Driver;
use Spica::Driver::SqlBase; # included 'use DBI'
use base qw( Spica::Driver::SqlBase );
use vars qw( $VERSION );
$VERSION = '0.03';
# 0.03 [2006/07/04] got rid of "return( undef )"
# 0.02 [2006/03/31] added driver_param and table_param
# 0.01 [2006/03/17] changed bracket rule
# 0.00 [2006/03/04] generated
# --------------------------------------
# // defines / static variables
sub DEFAULT_DBNAME (){ './sqlite/data.db' };
# --------------------------------------
# // override base class - param
sub driver_param
{
	my $class = shift;
	return(
		'name'    => DEFAULT_DBNAME,
		'sqlite2' => undef,
	);
}
sub table_param
{
	my $class = shift;
	return (
		'id'   => 'INTEGER',
		'str'  => 'VARCHAR',
		'text' => 'TEXT',
		'date' => 'INTEGER',
		'zone' => 'VARCHAR(7)',
		'int'  => 'INTEGER', 
		'list' => 'TEXT',
		'file' => 'VARCHAR(64)',
	);
}
# --------------------------------------
# // override base class
sub setup
{
	my $self = shift;
	$self->SUPER::setup(
		$self->driver_param,
		'option' => { # options - use for DBI
			'PrintError' => 0,
			'RaiseError' => 0,
		},
		@_
	);
	$self->dbh($self->_connect);
	return $self;
}
sub save($@)
{
	my $self = shift;
	my @objs = @_; # input parameter => set objects to save
	return if ( $#objs == -1 );
	return if ( !$self->dbh );
	my $class = ref($objs[0]);
	$self->dbh->do('BEGIN;');
	$self->save_list($class,@objs);
	$self->dbh->do('COMMIT;');
	return 1;
}
# --------------------------------------
# // override base class - implementation
sub fetch_id
{
	my $self = shift;
	return $self->dbh->func('last_insert_rowid');
}
# --------------------------------------
# // public functions - class method
sub convert_type
{
	my $class = shift;
	my $name = shift;
	my $type = $class->SUPER::convert_type($name,$class->table_param);
	$type = 'VARCHAR(255)' if ($type eq 'UNKNOWN');
	$type .= ' PRIMARY KEY NOT NULL' if ($name =~ /^UNI/i);
	return $type;
}
# --------------------------------------
# // private functions - instance method
sub _connect
{
	my $self = shift;
	my $lang = Spica::Language->get;
	my $handle = undef;
	my $source = ($self->{'sqlite2'}) ? 'DBI:SQLite2:' : 'DBI:SQLite:';
	if (!-e $self->{'name'})
	{
		require Spica::File;
		Spica::File->new('fullpath'=>$self->{'name'})->touch;
	}
	$source .= 'dbname=' . $self->{'name'};
	$handle = DBI->connect($source,'','',$self->{'option'});
	die($lang->string('Failed to connect database. ') . $DBI::errstr) if (!$handle);
	return $handle;
}
1;
__END__
