# Spica::Driver::Text
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==
# == Copyright (C) 2005 Spica Project Kei/Fuco/Takkyun ==

# ======================================
package Spica::Driver::Text;
use strict;
use Spica::Lock;
use Spica::Language;
use Spica::Driver;
use base qw( Spica::Driver );
use vars qw( $VERSION );
$VERSION = '0.09';
# 0.09 [2006/03/31] added driver_param
# 0.08 [2006/03/29] added count
# 0.07 [2006/03/17] changed bracket rule
# 0.06 [2006/03/02] replaced _name into Driver, added create_table
# 0.05 [2006/02/26] removed matched_number to move base class
# 0.04 [2005/11/24] changed _save_ids to save index file correctly
# 0.03 [2005/10/28] refactored
# 0.02 [2005/08/23] changed _load_detail to separate loading an object
# 0.01 [2005/03/01] changed how to lock a file
# 0.00 [2005/02/02] generated
# --------------------------------------
# // defines / static variables
sub DEFAULT_DIRECTORY  (){ './data/' };
sub DEFAULT_SUFFIX     (){ '.cgi' };
sub DEFAULT_INDEX      (){ 'id' };
sub DEFAULT_PERMISSION (){ 0666 };
sub DEFAULT_LOCKFILE   (){ 'save' };
sub TEMPORARY_FILE     (){ '_tmp' };
# --------------------------------------
# // override base class - param
sub driver_param
{
	my $class = shift;
	return(
		'name'       => DEFAULT_DIRECTORY,
		'suffix'     => DEFAULT_SUFFIX,
		'index'      => DEFAULT_INDEX,
		'permission' => DEFAULT_PERMISSION,
		'lock_file'  => DEFAULT_LOCKFILE,
		'tmp_file'   => TEMPORARY_FILE,
	);
}
# --------------------------------------
# // override base class
sub setup
{
	my $self = shift;
	$self->SUPER::setup(
		$self->driver_param,
		@_,
		'max' => {}, # buffer for id index
	);
	$self->_load_ids;
	return $self;
}
# --------------------------------------
# // override base class - public method
sub load($$;@)
{
	my $self  = shift;
	my $class = shift; # get sub-class name of Spica::Data::Object
	my %param = ( # input paramaeters
		'cond'  => {},    # searching option
		'id'    => undef, # inidicates id to load directly
		'logic' => 'and', # logic factor for searching option
		'bgn'   => 0,     # offset of objects to load
		'num'   => -1,    # maximum of objects to load
		'sort'  => 'id',  # sorting option
		'order' => 0,     # sorting order 0:descent 1:ascent
		@_
	);
	my $func = &_generate_condition($param{'logic'},%{$param{'cond'}});
	my @array = $self->_load_list($class,$func,%param);
	$self->{'num'} = $#array + 1;
	if ( $self->{'num'} > 1 )
	{ # there are more than one objects
		@array = &_sort_object($param{'sort'},$param{'order'},@array);
		@array = splice(@array,$param{'bgn'},$param{'num'}) if ( $param{'num'} > 0 and @array > $param{'num'} );
	}
	return( @array );
}
sub count($$;@)
{
	my $self = shift;
	$self->load(@_);
	return $self->matched_number;
}
sub save($@)
{
	my $self = shift;
	my @objs = @_; # input parameter => set objects to save
	return(0) if ( $#objs == -1 );
	my $class = ref($objs[0]);
	my $lock = Spica::Lock->basedir . $self->{'lock_file'};
	@objs = sort { $a->id <=> $b->id } @objs;
	my $lfh = Spica::Lock->locked_open($lock) or die(Spica::Language->get->string('Failed to lock.'));
	$self->_save_list($class,@objs);
	$self->_save_ids;
	close($lfh);
	return(1);
}
sub new_id($$)
{
	my $self = shift;
	my $class = shift; # set class name of Spica::Data::Object
	return $self->{'max'}->{$self->name($class)}++;
}
sub create_table($$)
{
	my $self  = shift;
	my $class = shift; # set class name of Spica::Data::Object
	my $lock = Spica::Lock->basedir . $self->{'lock_file'};
	my $file = $self->{'name'} . $self->name($class) . $self->{'suffix'};
	require Spica::File;
	Spica::File->new('fullpath'=>$file)->touch($self->{'permission'});
}
# --------------------------------------
# // private functions - instance method
sub _load_ids
{
	my $self = shift;
	my $file = $self->{'name'} . $self->{'index'} . $self->{'suffix'};
	if (!-e $file)
	{
		require Spica::File;
		Spica::File->new('fullpath'=>$file)->touch($self->{'permission'});
	}
	else
	{
		open(IDFILE_IN,"<$file") or die(Spica::Language->get->string('Failed to open the file') . ' : ' . $file);
		while (my $line = <IDFILE_IN>)
		{
			$line =~ tr/\x0D\x0A//d;
			my ($key,$val) = split("\t",$line,2);
			next if ($key eq '');
			$self->{'max'}->{$key} = int($val);
		}
		close(IDFILE_IN);
	}
}
sub _save_ids
{
	my $self = shift;
	my $file = $self->{'name'} . $self->{'index'} . $self->{'suffix'};
	open(IDFILE_OUT,">$file") or die(Spica::Language->get->string('Failed to open the file') . ' : ' . $file);
	binmode(IDFILE_OUT);
	foreach my $key ( keys(%{$self->{'max'}}) )
	{
		print IDFILE_OUT $key,"\t",$self->{'max'}->{$key},"\n";
	}
	close(IDFILE_OUT);
}
sub _load_list
{
	my $self  = shift;
	my $class = shift;
	my $func  = shift;
	my %param = @_;
	my @array = ();
	my $name  = $self->name($class);
	my $file  = $self->{'name'} . $name . $self->{'suffix'};
	return(@array) if (!-e $file);
	my @buffer = ();
	open(DATAIN,"<$file") or die(Spica::Language->get->string('Failed to open the file') . ' : ' . $file);
	while (my $line = <DATAIN>)
	{ # load each object to allocate
		my @data = &_decode($line);
		my $id = shift(@data);
		next if (defined($param{'id'}) and $param{'id'} != $id);
		my $obj = $class->create_with_array($id,@data);
		next if ( &{$func}($obj) );
		push(@array,$obj);
		last if (defined($param{'id'}));
	}
	close(DATAIN);
	return(@array);
}
sub _save_list
{
	my $self  = shift;
	my $class = shift;
	my @objs  = @_;
	my $name  = $self->name($class);
	my $file  = $self->{'name'} . $name . $self->{'suffix'};
	my $temp  = $self->{'name'} . $name . $self->{'tmp_file'};
	open(LISTOUT,">$temp") or die(Spica::Language->get->string('Failed to open the file') . ' : ' . $temp);
	open(LISTIN,"<$file")  or die(Spica::Language->get->string('Failed to open the file') . ' : ' . $file);
	binmode(LISTOUT);
	while (my $line = <LISTIN>)
	{ # load each object to check updating
		my @data = &_decode($line);
		if (@objs and $data[0] == $objs[0]->id)
		{
			print LISTOUT &_encode($objs[0],$name) if (!$objs[0]->erased and ref($objs[0]) eq $class);
			shift(@objs);
		}
		else
		{
			print LISTOUT $line;
		}
	}
	close(LISTIN);
	foreach my $obj ( @objs )
	{
		next if ($obj->erased or ref($obj) ne $class);
		print LISTOUT &_encode($obj,$name);
	}
	close(LISTOUT);
	rename($temp,$file);
	chmod($self->{'permission'},$file);
}
# --------------------------------------
# // private functions - tools
sub _generate_condition
{
	my $logic = shift;
	my %cond  = @_;
	return( sub { return(0) } ) if ( !keys(%cond) );
	my %reg = ();
	foreach my $key ( keys(%cond) )
	{
		next if ( $key eq '__range' or $key eq '__combo' );
		if ( $cond{$key} =~ /^\/(.*)\/$/ )
		{
			$reg{$key} = $1;
			$cond{$key} = undef;
		}
	}
	my $re = ( keys(%reg) ) ? 1 : 0;
	my $func  = sub {
		my $flag = 0;
		foreach my $key ( keys(%reg) )
		{
			my $word = $reg{$key};
			$flag = ( $_[0]->{$key} =~ /\Q$word\E/i );
			last if ($flag);
		}
		return( !$flag ) if ( ($flag and $logic eq 'or' and $re) or (!$flag and $logic eq 'and' and $re) );
		foreach my $key ( keys(%cond) )
		{
			next if ( $key eq '__range' or $key eq '__combo' );
			next if ( !defined($cond{$key}) or !defined($_[0]->{$key}) );
			if ( ref($cond{$key}) eq 'ARRAY' )
			{
				my $check = $_[0]->{$key};
				$check += sb::Time->diff_timezone($_[0]->{'tz'}, 0) if ( $cond{'__range'}->{$key} eq 'tz' );
				$flag = ( defined( $cond{'__range'}->{$key} ) ) 
				      ? ( $check >= $cond{$key}[0] and $check < $cond{$key}[1] )
				      : ( grep( /^\Q$check\E$/, @{$cond{$key}} ) );
			}
			else
			{
				$flag = ( $_[0]->{$key} eq $cond{$key} );
			}
			if ( defined( $cond{'__combo'}->{$key} ) and !$flag )
			{
				my %combo = %{$cond{'__combo'}};
				$flag = ( index($_[0]->{$combo{$key}},$combo{$combo{$key}}) > -1 );
			}
			last if ( ($flag and $logic eq 'or') or (!$flag and $logic eq 'and') );
		}
		return( !$flag );
	};
	return( $func );
}
sub _sort_object
{
	my $elem  = shift;
	my $order = shift;
	my @array = @_;
	my (@data,@ids);
	foreach (@array)
	{
		push(@data, $_->{$elem});
		push(@ids,  $_->id);
	}
	@array = @array[ sort {
		$data[$b] <=> $data[$a] or $data[$a] cmp $data[$b] or $ids[$b] <=> $ids[$a]
	} 0 .. $#ids ];
	@array = reverse(@array) if ($order);
	return @array;
}
sub _decode
{
	my $line = shift;
	$line =~ tr/\x0D\x0A//d;
	return map { s/\\(.)/$1 eq 't' and "\t" or $1 eq 'n' and "\n" or "$1"/eg; $_; } split("\t",$line);
}
sub _encode
{
	my ($obj,$name) = @_;
	my @data = ();
	foreach my $elem ( $obj->elements )
	{
		push(@data,&_linefeed($obj->{$elem}));
	}
	my @fields = map { s/\\/\\\\/g; s/\t/\\t/g; s/\n/\\n/g; $_; } @data;
	my $line = join("\t",@fields);
	return $line . "\t\n";
}
sub _linefeed
{ # unify linefeed code
	$_[0] =~ s/\x0D\x0A/\n/g;
	$_[0] =~ tr/\x0D\x0A/\n\n/;
	while ($_[0] =~ /\n$/)
	{
		$_[0] =~ s/\n$//g;
	}
	return($_[0]);
}
1;
__END__
