# Spica::File::Dir
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==
# == Copyright (C) 2005 Spica Project Kei/Fuco/Takkyun ==

# ======================================
package Spica::File::Dir;
use strict;
use Spica::File::Object;
use Spica::File;
use base qw( Spica::File );
use vars qw( $VERSION );
$VERSION = '0.03';
# 0.03 [2006/07/04] got rid of "return( undef )"
# 0.02 [2006/03/30] moved is_dir into Spica::File
# 0.01 [2006/03/17] changed bracket rule
# 0.00 [2005/10/27] generated
# --------------------------------------
# // defines / static variables
sub DEFAULT_PERMISSION (){ 0777 };
# --------------------------------------
# // override base class
sub content_type
{
	my $self = shift;
	return( 'directory' ) if ($self->is_dir);
	return $self->SUPER::content_type;
}
sub touch
{
	my $self = shift;
	return $self->SUPER::touch(@_) if ($self->name ne '');
	my $permission = shift;
	$permission = DEFAULT_PERMISSION if (!$permission);
	eval
	{
		die('already exists') if ($self->exists);
		eval { umask(0) };
		mkdir($self->fullpath,$permission) or die('failed to create a directory');
	};
	return ($@) ? undef : 1;
}
sub remove
{
	my $self = shift;
	return $self->SUPER::remove if ( !$self->is_dir );
	return if (!$self->exists or !$self->is_writable);
	rmdir($self->fullpath);
}
# --------------------------------------
# // public functions
sub get_files
{
	my $self = shift;
	my %param = (
		'recursive' => undef,
		'readable'  => 1,
		'writable'  => 1,
		'directory' => undef,
		'object'    => undef,
		@_
	);
	return() if (!$self->is_dir);
	return &_recursive_search($self->fullpath,%param);
}
# --------------------------------------
# // private functions
sub _recursive_search
{
	my $dir = shift;
	my %param = @_;
	my @out = ();
	opendir(TARGET_DIR,$dir);
	my @files = readdir(TARGET_DIR);
	close(TARGET_DIR);
	foreach my $file ( sort {$a cmp $b} @files)
	{
		my $obj = undef;
		my $check = $dir . $file;
		next if ($file =~ /^\./);
		next if ($file =~ /^_/);
		next if ($param{'directory'} and !-d $check);
		next if ($param{'writable'}  and !-w $check);
		next if ($param{'readable'}  and !-r $check);
		if (-d $check)
		{
			$obj = Spica::File::Object->new('path'=>$dir . $file . '/');
			push(@out,$obj) if (!$param{'object'});
			push(@out,$obj->get_files(%param)) if ($param{'recursive'});
		}
		else
		{
			$obj = Spica::File::Object->new('path'=>$dir,'name'=>$file);
			push(@out,$obj);
		}
	}
	return(@out);
}
1;
__END__
