# Spica::File::MimeType
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==
# == Copyright (C) 2005 Spica Project Kei/Fuco/Takkyun ==

# ======================================
package Spica::File::MimeType;
use strict;
use vars qw( $VERSION );
$VERSION = '0.01';
# 0.01 [2006/07/04] got rid of "return( undef )"
# 0.00 [2005/10/28] generated
# --------------------------------------
# // public functions
sub content_type
{ # file MIME types, if it's unknown, this function returns as "application/octet-stream"
	my $class = shift;
	my $name = shift;
	return('text/html')                     if ($name =~ /\.html?$/i);
	return('text/css')                      if ($name =~ /\.css$/i);
	return('text/plain')                    if ($name =~ /\.te?xt$/i);
	return('image/gif')                     if ($name =~ /\.gif$/i);
	return('image/jpeg')                    if ($name =~ /\.jpe?g$/i);
	return('image/png')                     if ($name =~ /\.png$/i);
	return('image/bmp')                     if ($name =~ /\.bmp$/i);
	return('image/tiff')                    if ($name =~ /\.tiff?$/i);
	return('audio/mpeg')                    if ($name =~ /\.mp3$/i);
	return('audio/aiff')                    if ($name =~ /\.aiff?$/i);
	return('audio/x-m4a')                   if ($name =~ /\.m4a$/i);
	return('audio/x-m4p')                   if ($name =~ /\.m4p$/i);
	return('audio/midi')                    if ($name =~ /\.midi?$/i);
	return('audio/wav')                     if ($name =~ /\.wav$/i);
	return('audio/x-pn-realaudio')          if ($name =~ /\.rm$/i);
	return('video/mpeg')                    if ($name =~ /\.mpe?g$/i);
	return('video/mp4')                     if ($name =~ /\.mp4$/i);
	return('video/x-msvideo')               if ($name =~ /\.avi$/i);
	return('video/quicktime')               if ($name =~ /\.mov$/i);
	return('application/xml')               if ($name =~ /\.xml$/i);
	return('application/x-shockwave-flash') if ($name =~ /\.swf$/i);
	return('application/x-stuffit')         if ($name =~ /\.sit$/i);
	return('application/x-compress')        if ($name =~ /\.z$/i);
	return('application/gzip')              if ($name =~ /\.gz$/i);
	return('application/zip')               if ($name =~ /\.zip$/i);
	return('application/lha')               if ($name =~ /\.lha$|\.lzh$/i);
	return('application/msword')            if ($name =~ /\.doc$/);
	return('application/mspowerpoint')      if ($name =~ /\.ppt$/);
	return('application/x-excel')           if ($name =~ /\.xls$/);
	return('application/pdf')               if ($name =~ /\.pdf$/);
	return('application/postscript')        if ($name =~ /\.e?ps$/);
	return('application/octet-stream');
}
sub extension
{ # file extensions, the following files can be handled on the framework
	my $class = shift;
	my $content_type = shift;
	return('.html') if ($content_type eq 'text/html');
	return('.css')  if ($content_type eq 'text/css');
	return('.txt')  if ($content_type eq 'text/plain');
	return('.xml')  if ($content_type eq 'application/xml');
	return('.jpg')  if ($content_type eq 'image/jpeg' or $content_type eq 'image/pjpeg');
	return('.png')  if ($content_type eq 'image/png' or $content_type eq 'image/x-png');
	return('.gif')  if ($content_type eq 'image/gif');
	return('.bmp')  if ($content_type eq 'image/bmp');
	return('.tif')  if ($content_type eq 'image/tiff');
	return('.mov')  if ($content_type eq 'video/quicktime');
	return('.mpg')  if ($content_type eq 'video/mpeg' or $content_type eq 'video/x-mpeg');
	return('.avi')  if ($content_type eq 'video/x-msvideo');
	return('.mp4')  if ($content_type eq 'video/mp4');
	return('.rm')   if ($content_type eq 'audio/x-pn-realaudio');
	return('.mp3')  if ($content_type eq 'audio/mpeg' or $content_type eq 'audio/x-mpeg');
	return('.aiff') if ($content_type eq 'audio/aiff');
	return('.m4a')  if ($content_type eq 'audio/x-m4a');
	return('.m4p')  if ($content_type eq 'audio/x-m4p');
	return('.midi') if ($content_type eq 'audio/midi' or $content_type eq 'audio/x-midi');
	return('.wav')  if ($content_type eq 'audio/wav' or $content_type eq 'audio/x-wav');
	return('.swf')  if ($content_type eq 'application/x-shockwave-flash');
	return('.gz')   if ($content_type eq 'application/gzip' or $content_type eq 'application/x-gzip');
	return('.zip')  if ($content_type eq 'application/zip' or $content_type eq 'application/x-zip-compressed');
	return('.lzh')  if ($content_type eq 'application/lha' or $content_type eq 'application/x-lha');
	return('.sit')  if ($content_type eq 'application/x-stuffit');
	return('.z')    if ($content_type eq 'application/x-compress');
	return('.doc')  if ($content_type eq 'application/msword');
	return('.ppt')  if ($content_type eq 'application/mspowerpoint');
	return('.xls')  if ($content_type eq 'application/x-excel');
	return('.pdf')  if ($content_type eq 'application/pdf');
	return('.ps')   if ($content_type eq 'application/postscript');
	return;
}
1;
__END__
