# Spica::Interface
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==
# == Copyright (C) 2005 Spica Project Kei/Fuco/Takkyun ==

# ======================================
package Spica::Interface;
use strict;
use Spica;
use Spica::Language;
use Spica::Time;
use Spica::Object;
use base qw( Spica::RequestObject );
use vars qw( $VERSION );
$VERSION = '0.11';
# 0.11 [2006/07/04] got rid of "return( undef )"
# 0.10 [2006/03/19] added _read_std, changed _error and _parse_multipart to work under mod_perl
# 0.09 [2006/03/17] changed bracket rule
# 0.08 [2006/03/16] use $Spica::MODPERL instead of $MOD_PERL
# 0.07 [2006/03/11] changed set_cookie to handle domain value
# 0.06 [2006/03/09] changed _parse_multipart to fix a bug
# 0.05 [2005/11/18] changed _parse_non_multipart to set posted raw data
# 0.04 [2005/10/26] implemented mod_perl functionality, added some accessors
# 0.03 [2005/08/08] changed head to output http header correctly
# 0.02 [2005/07/14] changed _parse_data to parse remote_host more precisely
# 0.01 [2005/06/08] changed set_cookie not to set domain field for cookie
# 0.00 [2004/xx/xx] generated
# --------------------------------------
# // defines / static variables
sub LF            (){ "\x0D\x0A" }
sub TIME_FORMAT   (){ '%Week%, %DayShort%-%MonShort%-%YearLong% %Hour24%:%Min%:%Sec% GMT' }
sub HOURS_PER_DAY (){ 24 };
sub SECS_PER_HOUR (){ 3600 };
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	my %param = (
		'params' => {},
		'keys'   => [],
		'xmlrpc' => undef,
		'cookie' => undef,
		'req'    => undef,
		'parser' => {},
		@_
	);
	$self->SUPER::initialize(%param);
	$self->req(Apache->request) if ($Spica::MODPERL and !$self->req);
	$self->_parse_data;
	$self;
}
# --------------------------------------
# // public functions - accessors
sub value
{
	my $self = shift;
	my $key  = shift;
	return ( $key and defined($self->{'params'}->{$key}) ) ? $self->{'params'}->{$key} : undef;
}
sub names
{
	my $self = shift;
	return( keys( %{$self->{'params'}} ) );
}
sub xmlrpc
{
	my $self = shift;
	return $self->{'xmlrpc'};
}
sub content_list
{
	my $self = shift;
	return @{$self->{'keys'}};
}
sub req
{ # from CGI.pm 3.05 / Copyright (C) 1995-1998 Lincoln D. Stein.
	my $self = shift;
	my $req  = $self->{'req'};
	$self->{'req'} = shift if (@_);
	return $req;
}
# --------------------------------------
# // public functions - cookies
sub cookie
{ # [accessor] get cookie parameters
	my $self = shift;
	my %cookie = ();
	my %param = ( # input parameters
		'name' => undef, # [required][CHAR] cookie name
		@_,
	);
	if ( defined($param{'name'}) )
	{
		my (@cookies,%cookie_base);
		@cookies = split(/;/,$ENV{'HTTP_COOKIE'});
		foreach (@cookies)
		{
			my ($cook_key,$cook_val) = split(/=/,$_);
			$cook_key =~ s/ //g;
			$cookie_base{$cook_key} = $cook_val;
		}
		@cookies = split(/,/,$cookie_base{$param{'name'}});
		foreach (@cookies)
		{
			my ($cook_key,$cook_val) = split('<>',$_);
			$cookie{$cook_key} = &_decode_uri($cook_val);
		}
	}
	return( \%cookie );
}
sub clear_cookie
{ # clear cookie parameters
	my $self = shift;
	my $cookie = $self->{'cookie'};
	$self->{'cookie'} = undef;
	return $cookie;
}
sub set_cookie
{ # set cookie parameters
	my $self = shift;
	my %param = ( # input parameters
		'time'   => undef, # [required][NUM.] current time
		'name'   => undef, # [required][CHAR] cookie name
		'expire' => 10,    # [optional][NUM.] expires day
		'path'   => '',    # [optional][URI.] path
		'domain' => '',    # [optional][URI.] domain
		'data'   => undef, # [optional][HASH] stored data
		'secure' => undef, # [optional][SEL.] secure flag / if set as 1, cookie will be issued as secure
		@_,
	);
	if ( defined($param{'time'}) and defined($param{'name'}) )
	{
		my ($cookie,$name,$date,$path,$domain);
		$param{'expire'} *= (HOURS_PER_DAY * SECS_PER_HOUR);
		$date = Spica::Time->new($param{'time'} + $param{'expire'});
		$date = $date->format('form' => TIME_FORMAT);
		$name = $param{'name'};
		if ( defined($param{'data'}) )
		{
			foreach my $key ( keys(%{$param{'data'}}) )
			{
				$cookie .= $key . '<>' . &_encode_uri($param{'data'}->{$key}) . ',';
			}
		}
		else
		{
			$cookie = $name . '<>on,';
		}
		$domain = $param{'domain'};
		if ($param{'path'} =~ m!https?://(.*?)/(.*)/!)
		{
			$domain = $1 if (!$domain);
			$path = '/' . $2 . '/';
		}
		elsif ($param{'path'} =~ m!^/!)
		{
			$path = $param{'path'};
		}
		if ( $domain eq 'localhost' 
		  or $domain eq '127.0.0.1'  
		  or $domain !~ /(.*)\.(.+)\.(.+)/ )
		{ # dots need more than 2 at least.
			$domain = ''; # to avoid not saving cookie when domain is set
		}
		$self->{'cookie'} .= 'Set-Cookie: ' . $name . '=' . $cookie . ';';
		$self->{'cookie'} .= ' expires=' . $date . ';';
		$self->{'cookie'} .= ' domain=' . $domain . ';' if ($domain);
		$self->{'cookie'} .= ' path=' . $path . ';' if ($path);
		$self->{'cookie'} .= ' secure;' if ($param{'secure'});
		$self->{'cookie'} .= LF;
	}
	return $self->{'cookie'};
}
# --------------------------------------
# // public functions - globals
sub head
{ # output http head
	my $self = shift;
	my %param = ( # input parameters
		'location' => '',                            # [optional][URI.] address to forward
		'charset'  => Spica::Language->get->charset, # [optional][CHAR] character set
		'type'     => 'text/html',                   # [optional][CHAR] Content-Type
		'cache'    => 0,                             # [optional][NUM.] chache expires day
		'length'   => undef,                         # [optional][NUM.] Content-Length
		@_,
	);
	my @output = ();
	push(@output,$self->{'cookie'}) if ($self->{'cookie'} ne '');
	push(@output,'Content-length: ' . $param{'length'} . LF) if ( defined($param{'length'}) );
	if ($param{'location'} ne '')
	{
		my ($hash,$query) = ();
		($param{'location'},$hash)  = split('\#',$param{'location'},2) if (index($param{'location'},'#') > -1);
		($param{'location'},$query) = split('\?',$param{'location'},2) if (index($param{'location'},'?') > -1);
		$param{'location'} .= '?' . $query;
		$param{'location'} .= '#' . $hash if ($hash ne '');
		push(@output,'Status: 302 Moved',LF);
		push(@output,'Location: ',$param{'location'},LF);
		push(@output,'Content-Type: ',$param{'type'},'; charset=',$param{'charset'},';',LF);
		push(@output,LF); # boundary between head and body
		push(@output,'<html><head><meta http-equiv="Refresh" content="0;URL=',$param{'location'},'"></head></html>',LF);
	}
	else
	{
		push(@output,'Content-Type: ',$param{'type'},'; charset=',$param{'charset'},';',LF);
		if ($param{'cache'} > 0)
		{
			my $expire = Spica::Time->new($param{'cache'})->format('form'=>TIME_FORMAT);
			push(@output,'Pragma: no-cache',LF);
			push(@output,'Cache-Control: no-store, no-cache, must-revalidate, post-check=0, pre-check=0',LF);
			push(@output,'Expires: ',$expire,LF);
		}
		push(@output,LF); # boundary between head and body
	}
	return( join('',@output) );
}
sub path_info
{ # from CGI.pm 3.05 / Copyright (C) 1995-1998 Lincoln D. Stein.
	my $self = shift;
	return if ( !defined($ENV{'PATH_INFO'}) );
	my $iis = ( defined($ENV{'SERVER_SOFTWARE'}) and $ENV{'SERVER_SOFTWARE'} =~ /IIS/ );
	my $path = $ENV{'PATH_INFO'};
	$path =~ s/^\Q$ENV{'SCRIPT_NAME'}\E// if $iis;
	return $path;
}
sub script_name
{ # from CGI.pm 3.05 / Copyright (C) 1995-1998 Lincoln D. Stein.
	return $ENV{'SCRIPT_NAME'} if ( defined($ENV{'SCRIPT_NAME'}) );
	return "/$0" unless $0=~/^\//;
	return $0;
}
sub host
{
	return $ENV{'HTTP_HOST'} if ( defined($ENV{'HTTP_HOST'}) );
	return 'localhost';
}
sub current_path
{
	my $path = &host() . &script_name();
	my $protocol = 'http';
	$protocol = 'https' if ( defined($ENV{'HTTPS'}) and $ENV{'HTTPS'} =~ /ON/i );
	$protocol = 'https' if ( defined($ENV{'SERVER_PORT'}) and $ENV{'SERVER_PORT'} == 443 );
	# $path =~ s!(.*/)(.*)!$1!; # removed temporary
	return $protocol . '://' . $path;
}
sub content_type
{
	return $ENV{'CONTENT_TYPE'};
}
sub content_length
{
	return $ENV{'CONTENT_LENGTH'};
}
sub request_method
{
	return $ENV{'REQUEST_METHOD'}
}
sub referer
{
	return $ENV{'HTTP_REFERER'};
}
sub user_agent
{
	return $ENV{'HTTP_USER_AGENT'};
}
sub remote_host
{
	my %env = (
		'host' => $ENV{'REMOTE_HOST'},
		'addr' => $ENV{'REMOTE_ADDR'},
	);
	$env{'host'} = $env{'addr'} if ($env{'host'} eq '');
	if ($env{'host'} eq $env{'addr'})
	{
		$env{'host'} = gethostbyaddr(pack('C4',split(/\./,$env{'host'})),2) || $env{'addr'};
	}
	return \%env;
}
# --------------------------------------
# // private functions - utilities
sub _read
{ # from CGI.pm 3.05 / Copyright (C) 1995-1998 Lincoln D. Stein.
	my $self = shift;
	my ($buf,$len,$offset) = @_;
	local $^W=0;
	return ($Spica::MODPERL) 
		? $self->req->read($$buf, $len, $offset) 
		: read(STDIN,$$buf,$len, $offset);
}
sub _read_std
{
	my $self = shift;
	my ($buf,$len,$offset) = @_;
	local $^W=0;
	return read(STDIN,$$buf,$len, $offset);
}
sub _error
{
	my $self  = shift;
	my $title = shift;
	my @msg   = @_;
	my $body;
	$title =~ tr/\x0D\x0A//d;
	$body .= sprintf("Error [%s] in Spica::Interface\n---\n",$title);
	$body .= join("\n",@msg) . "\n===\n";
	($Spica::MODPERL)
		? $self->req->send_cgi_header('Content-Type: text/plain' . LF . LF)
		: CORE::print('Content-Type: text/plain' . LF . LF);
	($Spica::MODPERL)
		? $self->req->print($body) 
		: CORE::print($body);
}
sub _linefeed
{ # unify linefeed code
	$_[0] =~ s/\x0D\x0A/\n/g;
	$_[0] =~ tr/\x0D\x0A/\n\n/;
	return($_[0]);
}
sub _decode_uri
{ # decode uri
	$_[0] =~ s/%([0-9A-Fa-f][0-9A-Fa-f])/pack('H2',$1)/eg;
	return($_[0]);
}
sub _encode_uri
{ # encode uri
	$_[0] =~ s/(\W)/'%' . unpack('H2', $1)/eg;
	return($_[0]);
}
# --------------------------------------
# // private functions - parser
sub _parse_data
{ # from cgi-lib.pl 2.18 / Copyright (C) 1993-1999 Steven E. Brenner
	my $self = shift;
	my %param = (
		'max_data'    => 102400,   # maximum bytes to accept via POST - 2^17
		'write_file'  => 0,        # directory to which to write files, or 0 if files should not be written
		'file_pre'    => 'clform', # Prefix of file names, in directory above
		'buf_size'    => 8129,     # default buffer size when reading multipart
		'max_bound'   => 100,      # maximum boundary length to be encounterd
		'decode_flag' => '',       # decode flag parameter
		'use_cmdline' => undef,    # read command-line options if necessary
		%{$self->{'parser'}}
	);
	binmode(STDIN);  # we need these for DOS-based systems
	binmode(STDOUT); # and they shouldn't hurt anything else 
	binmode(STDERR);
	# Get several env variables
	my $type = $self->content_type;
	my $len  = $self->content_length;
	my $meth = $self->request_method;
	$meth = 'GET' if ($meth eq '');
	$self->_error("Request to receive too much data: $len bytes\n") if ($len > $param{'max_data'});
	if ($type =~ /^multipart\/form-data/)
	{ # === multi-part ===
		$self->_parse_multipart(%param);
	}
	elsif ($type =~ /\/xml/)
	{ # === xml ===
		$self->_parse_posted_xml(%param);
	}
	elsif ($meth =~ /GET|HEAD|POST/ or $type =~ /^application\/x-www-form-urlencoded(;|$)/)
	{ # === non multi-part ===
		$self->_parse_non_multipart(%param);
	}
	else
	{
		$self->_error("Unknown Content-type: $type or Method: $meth\n");
	}
	return $self; 
}
sub _parse_non_multipart
{
	my $self = shift;
	my %param = @_;
	# --- local parameters ---
	my ($in, $got, $name) = ();
	my $type = $self->content_type;
	my $len  = $self->content_length;
	my $meth = $self->request_method;
	my $cmdflag = undef;
	# --- parsing ---
	if (!defined $meth or $meth eq '')
	{
		$in = ($Spica::MODPERL) ? $self->req->args : $ENV{'QUERY_STRING'};
		$cmdflag = $param{'use_cmdline'};  # also use command-line options
	}
	elsif($meth eq 'GET' or $meth eq 'HEAD')
	{
		$in = ($Spica::MODPERL) ? $self->req->args : $ENV{'QUERY_STRING'};
	}
	elsif ($meth eq 'POST')
	{
		$self->_error("[Short Read] wanted $len, got $got\n") if (($got = $self->_read(\$in, $len) != $len));
	}
	else
	{
		$self->_error("Unknown request method: $meth\n");
	}
	$in = &_linefeed($in);
	my $check_dec = ($param{'decode_flag'} and index($in,$param{'decode_flag'}) > -1) ? 0 : 1;
	@{$self->{'keys'}} = split(/[&;]/,$in); 
	push(@{$self->{'keys'}}, @ARGV) if ( $cmdflag ); # add command-line parameters
	for (my $i=0;$i<@{$self->{'keys'}};$i++)
	{
		$self->{'keys'}->[$i] =~ s/\+/ /g;
		my ($key, $val) = split(/=/,$self->{'keys'}->[$i],2);
		$self->{'params'}->{$key} .= "\0" if ( defined($self->{'params'}->{$key}) );
		$key = &_decode_uri($key) if ($check_dec);
		$val = &_decode_uri($val) if ($check_dec);
		$self->{'params'}->{$key} .= $val;
	}
	$self->{'params'}->{'__body'} = $in;
}
sub _parse_multipart
{
	my $self = shift;
	my %param = @_;
	# --- local parameters ---
	my ($in, $got, $name) = ();
	my $type = $self->content_type;
	my $len  = $self->content_length;
	my $meth = $self->request_method;
	my %incfn; # Client's filename (may not be provided)
	my %inct;  # Client's content-type (may not be provided)
	my %insfn; # Server's filename (for spooled files)
	# --- parsing ---
	eval
	{
		die("Invalid request method for  multipart/form-data: $meth\n") if ($meth ne 'POST');
		my ($head, @heads, $cd, $ct, $fname, $ctype, $bpos, $lpos, $amt, $fn);
		my $serial = 0;
		my $bufsize = $param{'buf_size'};
		my $maxbound = $param{'max_bound'};
		my $writefiles = $param{'write_file'};
		my $buf = '';
		my ($boundary) = $type =~ /boundary="([^"]+)"/; #" # find boundary
		($boundary) = $type =~ /boundary=(\S+)/ unless $boundary;
		die("Boundary not provided: probably a bug in your server\n") unless $boundary;
		$boundary =  "--" . $boundary;
		my $blen = length($boundary);
		if ($writefiles)
		{
			stat($writefiles);
			$writefiles = "/tmp" unless  -d _ && -w _;
			$writefiles .= "/$param{'file_pre'}"; 
		}
		my $left = $len;
		PART: while (1)
		{ # find each part of the multi-part while reading data
			$amt = ($left > $bufsize + $maxbound - length($buf)) 
			     ? $bufsize + $maxbound - length($buf)
			     : $left;
			$got = $self->_read_std(\$buf, $amt, length($buf));
			die("[Short Read] wanted $amt, got $got\n") if ($got != $amt);
			$left -= $amt;
			$self->{'params'}->{$name} .= "\0" if ( defined($self->{'params'}->{$name}) ); 
			$self->{'params'}->{$name} .= $fn if $fn;
			$name =~ /([-\w]+)/;  # This allows $insfn{$name} to be untainted
			if ( defined $1 )
			{
				$insfn{$1} .= "\0" if defined $insfn{$1}; 
				$insfn{$1} .= $fn if $fn;
			}
			BODY: while (($bpos = index($buf, $boundary)) == -1)
			{
				if ($left == 0 && $buf eq '')
				{
					foreach my $value (values %insfn)
					{
						unlink(split("\0",$value));
					}
					die("reached end of input while seeking boundary of multipart. Format of CGI input is wrong.\n");
				}
				if ($name)
				{ # if no $name, then it's the prologue -- discard
					if ($fn)
					{
						print CGI_FILE substr($buf, 0, $bufsize);
					}
					else
					{
						$self->{'params'}->{$name} .= substr($buf, 0, $bufsize);
					}
				}
				$buf = substr($buf, $bufsize);
				$amt = ($left > $bufsize) ? $bufsize : $left; # $maxbound == length($buf);
				$got = $self->_read_std(\$buf, $amt, length($buf));
				die("[Short Read] wanted $amt, got $got\n") if ($got != $amt);
				$left -= $amt;
			} # end of BODY
			if (defined $name)
			{ # if no $name, then it's the prologue -- discard
				if ($fn)
				{
					print CGI_FILE substr($buf, 0, $bpos - 2);
				}
				else
				{
					$self->{'params'}->{$name} .= substr($buf, 0, $bpos - 2); # kill last \r\n
				}
			}
			close (CGI_FILE);
			last PART if (substr($buf, $bpos + $blen, 2) eq "--");
			substr($buf, 0, $bpos + $blen + 2) = '';
			$amt = ($left > $bufsize + $maxbound - length($buf)) 
			     ? $bufsize + $maxbound - length($buf) 
			     : $left;
			$got = $self->_read_std(\$buf, $amt, length($buf));
			die("[Short Read] wanted $amt, got $got\n") if ($got != $amt);
			$left -= $amt;
			undef $head;
			undef $fn;
			HEAD: while (($lpos = index($buf, "\r\n\r\n")) == -1)
			{
				if ($left == 0  && $buf eq '')
				{
					foreach my $value (values %insfn)
					{
						unlink(split("\0",$value));
					}
					die("reached end of input while seeking end of headers. Format of CGI input is wrong.\n$buf");
				}
				$head .= substr($buf, 0, $bufsize);
				$buf = substr($buf, $bufsize);
				$amt = ($left > $bufsize) ? $bufsize : $left; # $maxbound == length($buf);
				$got = $self->_read_std(\$buf, $amt, length($buf));
				die("[Short Read] wanted $amt, got $got\n") if ($got != $amt);
				$left -= $amt;
			} # end of HEAD
			$head .= substr($buf, 0, $lpos + 2);
			push(@{$self->{'keys'}}, $head);
			@heads = split("\r\n", $head);
			($cd) = grep (/^\s*Content-Disposition:/i, @heads);
			($ct) = grep (/^\s*Content-Type:/i, @heads);
			($name) = $cd =~ /\bname="([^"]+)"/i; #"; 
			($name) = $cd =~ /\bname=([^\s:;]+)/i unless defined $name;  
			($fname) = $cd =~ /\bfilename="([^"]*)"/i; #"; # filename can be null-str
			($fname) = $cd =~ /\bfilename=([^\s:;]+)/i unless defined $fname;
			$incfn{$name} .= (defined($self->{'params'}->{$name}) ? "\0" : "") . (defined($fname) ? $fname : "");
			($ctype) = $ct =~ /^\s*Content-type:\s*"([^"]+)"/i;  #";
			($ctype) = $ct =~ /^\s*Content-Type:\s*([^\s:;]+)/i unless defined $ctype;
			$inct{$name} .= (defined($self->{'params'}->{$name})) ? "\0" . $ctype : "" . $ctype;
			if ($writefiles && defined $fname)
			{
				$serial++;
				$fn = $writefiles . ".$$.$serial";
				open(CG_FILE, ">$fn") or die("Couldn't open $fn\n");
				binmode(CG_FILE); # write files accurately
			}
			substr($buf, 0, $lpos + 4) = '';
			undef $fname;
			undef $ctype;
		} # end of PART
	}; # end of eval
	if ($@)
	{
		foreach my $value (values %insfn)
		{
			unlink(split("\0",$value));
		}
		$self->_error($@);
	}
}
sub _parse_posted_xml
{
	my $self = shift;
	my %param = @_;
	# --- local parameters ---
	my ($in, $got) = ();
	my $type = $self->content_type;
	my $len  = $self->content_length;
	# --- parsing ---
	$got = $self->_read(\$in, $len);
	$self->_error("[Short Read] wanted $len, got $got\n") if ($got != $len);
	$in = &_linefeed($in);
	$self->{'params'}->{'charset'} = $2 if ($in =~ /^<\?xml(.*?)encoding=\"(.*?)\"(.*?)>/s);
	if ($in =~ /<methodCall>/is)
	{ # xml-rpc
		$self->{'params'}->{'methodName'} = $1 if ($in =~ /<methodName>(.*?)<\/methodName>/is);
		$self->{'params'}->{'params'}     = $1 if ($in =~ /<params>(.*?)<\/params>/is);
		$self->{'xmlrpc'} = 1;
	}
	else
	{ # other xml request
		$self->{'params'}->{'__body'} = $in;
	}
	$self->{'keys'} = [];
}
1;
__END__
