# Spica::Language
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==
# == Copyright (C) 2005 Spica Project Kei/Fuco/Takkyun ==

# ======================================
package Spica::Language;
use strict;
use Spica::Object;
use base qw( Spica::RequestObject );
use vars qw( $VERSION );
$VERSION = '0.06';
# 0.06 [2006/04/06] changed load to use Spica::InitLoader
# 0.05 [2006/04/06] changed string to handle null string
# 0.04 [2006/03/17] changed bracket rule
# 0.03 [2005/11/15] added construct/setup to construct correctly
# 0.02 [2005/11/08] added %aCharsets
# 0.01 [2005/10/25] refactored
# 0.00 [2005/01/17] generated
# --------------------------------------
# // defines / static variables
my %aCharsets = (
	'iso-8859-1'  => 'ascii',
	'euc-jp'      => 'euc',
	'shift_jis'   => 'sjis',
	'iso-2022-jp' => 'jis',
	'utf-8'       => 'utf8',
	'utf-16'      => 'utf16',
);
# --------------------------------------
# // override base class
sub initialize
{ # Spica::Language works as a factory of language modules.
	my $self = shift;
	my $lang = shift;
	my $class = ref($self);
	if ($lang)
	{
		my $language = $class->module_name($lang);
		return $language->construct('lang'=>$lang) if ($class->load_module($lang));
	}
	return $class->construct;
}
# --------------------------------------
# // public functions - initializer
sub construct
{
	my $class = shift;
	my $self = {
		'lang'     => 'en',
		'charset'  => 'ASCII',
		'charcode' => 'ascii',
		'strings'  => {},
		@_
	};
	bless($self,$class);
	return $self->setup;
}
sub setup
{
	my $self = shift;
	my %param = (
		@_,
		'strings' => {
			'week_en'     => ['Sun','Mon','Tue','Wed','Thu','Fri','Sat'],
			'week_enlong' => ['Sunday','Monday','Tuesday','Wednesday','Thursday','Friday','Saturday'],
			'week_fr'     => ['dim','lun','mar','mer','jeu','ven','sam'],
			'week_frlong' => ['dimanche','lundi','mardi','mercredi','jeudi','vendredi','samedi'],
			'week_ja'     => ['&#x65E5;','&#x6708;','&#x706B;','&#x6C34;','&#x6728;','&#x91D1;','&#x571F;'],
			'week_jalong' => [
				'&#x65E5;&#x66DC;&#x65E5;','&#x6708;&#x66DC;&#x65E5;','&#x706B;&#x66DC;&#x65E5;','&#x6C34;&#x66DC;&#x65E5;',
				'&#x6728;&#x66DC;&#x65E5;','&#x91D1;&#x66DC;&#x65E5;','&#x571F;&#x66DC;&#x65E5;'],
			'month_en'     => ['Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'],
			'month_enlong' => [
				'January','February','March','April','May','June',
				'July','August','September','October','November','December'],
			'month_fr'     => [
				'jan','f&eacute;v','mar','avr','mai','juin',
				'juil','ao&ucirc;','sep','oct','nov','d&eacute;c'],
			'month_frlong' => [
				'janvier','f&eacute;vrier','mars','avril','mai','juin',
				'juillet','ao&ucirc;t','septembre','octobre','novembre','d&eacute;cembre'],
		}
	);
	return $self->SUPER::initialize(%param);
}
# --------------------------------------
# // public functions - accessors
sub language
{
	my $self = shift;
	return $self->{'lang'};
}
sub charset
{
	my $self = shift;
	$self->{'cahrset'} = shift if (@_);
	return $self->{'cahrset'};
}
sub charcode
{
	my $self = shift;
	$self->{'charcode'} = shift if (@_);
	return $self->{'charcode'};
}
sub string
{
	my $self = shift;
	my $key  = shift;
	return undef if ( !defined($key) or $key eq '');
	$self->{'strings'}->{$key} = shift if (@_);
	return $key if ( !defined($self->{'strings'}->{$key}) or $self->{'strings'}->{$key} eq '');
	return $self->{'strings'}->{$key};
}
sub load
{
	my $self = shift;
	my $file = shift;
	my $loader = Spica::InitLoader->new('file'=>$file);
	$loader->load($self->{'strings'});
}
# --------------------------------------
# // public functions - common
sub convert
{
	my $self = shift;
	my ($text,$outcode,$in_code) = @_;
	return($text);
}
sub checkcode
{
	my $self = shift;
	my $text = shift;
	return $self->charcode;
}
sub charcode_of
{
	my $self = shift;
	my $charset = lc( shift() );
	return ($aCharsets{$charset}) ? $aCharsets{$charset} : $charset;
}
sub charset_of
{
	my $self = shift;
	my $charcode = lc( shift() );
	my %aCharcode = reverse(%aCharsets);
	return ($aCharcode{$charcode}) ? $aCharcode{$charcode} : $charcode;
}
1;
__END__
