# Spica::Net::Aws
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package Spica::Net::Aws;
use strict;
use XML::Parser::Lite;
use Spica::Language;
use Spica::Net;
use base qw( Spica::Net );
use vars qw( $VERSION );
$VERSION = '0.06';
# 0.06 [2006/07/04] got rid of "return( undef )"
# 0.05 [2006/03/17] changed bracket rule
# 0.04 [2005/11/07] refactered
# 0.03 [2005/07/25] added mode_table as class method
# 0.02 [2005/07/24] changed _connect to connect amazon.com
# 0.01 [2005/07/22] changed aws developer token.
# 0.00 [2005/04/12] port form sbaws.pl
# --------------------------------------
# // defines / static variables
sub AWS_LOCALE_US     (){ 'us' };
sub AWS_LOCALE_UK     (){ 'uk' };
sub AWS_LOCALE_DE     (){ 'de' };
sub AWS_LOCALE_JP     (){ 'jp' };
sub AWS_LOCALE_FR     (){ 'fr' };
sub AWS_LOCALE_CA     (){ 'ca' };
sub AWS_SERVER_US     (){ 'http://xml-us.amznxslt.com/onca/xml3?' };
sub AWS_SERVER_JP     (){ 'http://xml-jp.amznxslt.com/onca/xml3?' };
sub AWS_ASSOCIATE     (){ 'simpleboxes-22' };
sub AWS_DEVELOPER     (){ '0ZZ51W51PSHKTDFA9002' };
sub AWS_TYPE          (){ 'lite' };
sub AWS_CODETYPE      (){ 'xml' };
sub AWS_ERROR_INITIAL (){ 'initialization error' };
sub AWS_ERROR_CONNECT (){ 'failed to connect' };
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	my %param = (
		'count'   => 0,
		'matched' => 0,
		'page'    => 0,
		'locale'  => AWS_LOCALE_JP,
		'error'   => undef,
		@_
	);
	$self->generate_accessor('count','matched','page','locale','error');
	return $self->SUPER::initialize(%param);
}
# --------------------------------------
# // public functions
sub get_genre
{ # it can be class method as well.
	my $self = shift;
	my $locale = shift if (@_);
	$locale = $self->locale if (!$locale and ref($self));
	if ($locale eq AWS_LOCALE_US)
	{
		return (
			'books'         => 'Books',
			'music'         => 'Popular Music',
			'classical'     => 'Classical Music',
			'dvd'           => 'DVD',
			'vhs'           => 'Video',
			'electronics'   => 'Electronics',
			'kitchen'       => 'Kitchen &amp; Housewares',
			'software'      => 'Software',
			'videogames'    => 'Computer &amp; Video Games',
			'magazines'     => 'Magazines',
			'toys'          => 'Toys &amp; Games',
			'photo'         => 'Camera &amp; Photo',
			'baby'          => 'Baby',
			'garden'        => 'Outdoor Living',
			'pc-hardware'   => 'Computers',
			'tools'         => 'Tools &amp; Hardware',
			'asin'          => 'ASIN',
		);
	}
	elsif ($locale eq AWS_LOCALE_UK)
	{
		return (
			'books-uk'       => 'Books',
			'music'          => 'Popular Music',
			'classical'      => 'Classical Music',
			'dvd-uk'         => 'DVD',
			'vhs-uk'         => 'Video',
			'electronics-uk' => 'Electronics',
			'kitchen-uk'     => 'Kitchen &amp; Housewares',
			'sw-vg-uk'       => 'Software',
			'video-games-uk' => 'Computer &amp; Video Games',
			'toys-uk'        => 'Toys &amp; Games',
			'asin'           => 'ASIN',
		);
	}
	elsif ($locale eq AWS_LOCALE_DE)
	{
		return (
			'books-de'         => 'Books',
			'pop-music-de'     => 'Popular Music',
			'classical-de'     => 'Classical Music',
			'dvd-de'           => 'DVD',
			'vhs-de'           => 'Video',
			'ce-de'            => 'Electronics',
			'kitchen-de'       => 'Kitchen &amp; Housewares',
			'software-de'      => 'Software',
			'video-games-de'   => 'Computer &amp; Video Games',
			'magazines-de'     => 'Magazines',
			'books-de-intl-us' => 'English/Foreign Books',
			'asin'             => 'ASIN',
		);
	}
	elsif ($locale eq AWS_LOCALE_JP)
	{
		return (
			'books-jp'       => 'Books',
			'music-jp'       => 'Popular Music',
			'classical-jp'   => 'Classical Music',
			'dvd-jp'         => 'DVD',
			'vhs-jp'         => 'Video',
			'electronics-jp' => 'Electronics',
			'kitchen-jp'     => 'Kitchen &amp; Housewares',
			'software-jp'    => 'Software',
			'videogames-jp'  => 'Computer &amp; Video Games',
			'toys-jp'        => 'Toys &amp; Games',
			'books-us'       => 'English/Foreign Books',
			'asin'           => 'ASIN',
		);
	}
	elsif ($locale eq AWS_LOCALE_FR)
	{
		return (
			'books-fr'         => 'Books',
			'music-fr'         => 'Popular Music',
			'classical-fr'     => 'Classical Music',
			'dvd-fr'           => 'DVD',
			'vhs-fr'           => 'Video',
			'software-fr'      => 'Software',
			'videogames-fr'    => 'Computer &amp; Video Games',
			'books-fr-intl-us' => 'English/Foreign Books',
			'asin'             => 'ASIN',
		);
	}
	elsif ($locale eq AWS_LOCALE_CA)
	{
		return (
			'books-ca'      => 'Books',
			'music-fr'      => 'Popular Music',
			'classical-ca'  => 'Classical Music',
			'dvd-fr'        => 'DVD',
			'vhs-ca'        => 'Video',
			'software-ca'   => 'Software',
			'videogames-ca' => 'Computer &amp; Video Games',
			'books-ca-en'   => 'English/Foreign Books',
			'books-ca-fr'   => 'French/Foreign Books',
			'asin'          => 'ASIN',
		);
	}
	else
	{
		return ('asin'=>'ASIN');
	}
}
sub get_data
{
	my $self = shift;
	my %param = &_convert_param(
		'genre'    => undef,
		'keyword'  => undef,
		'asin'     => undef,
		'page'     => 0,
		'locale'   => $self->locale,
		'id'       => AWS_ASSOCIATE,
		'charcode' => 'utf8',
		@_
	);
	# --- connect server and get content ---
	my $content = $self->_get_content(%param);
	return( [] ) if ($self->error);
	# --- initialize local variables ---
	my $count = 0;
	my $page = 0;
	my $matched = 0;
	my $buf = undef;
	my @data = ();
	# --- construct parser ---
	my $parser = XML::Parser::Lite->new;
	$parser->setHandlers(
		Init  => sub {},
		Final => sub {},
		Start => \&_xml_start,
		Char  => \&_xml_char,
		End   => \&_xml_end,
	);
	# --- parse content ---
	$parser->parse($content) if ($content);
	$self->count($count);
	$self->matched($matched);
	$self->page($page);
	return \@data;
	# --- local functions ---
	sub _xml_start
	{
		my $self = shift;
		my $tag = shift;
		$data[$count]->{'url'} = $_[1] if ($tag eq 'Details');
		$buf = $tag;
	};
	sub _xml_char
	{
		my $self = shift;
		my $elem = shift;
		if ($buf eq 'TotalResults')
		{ # matched number of search
			$matched = int($elem);
		}
		elsif ($buf eq 'TotalPages')
		{ # total pages
			$page = int($elem);
		}
		elsif ($buf eq 'Artists' or $buf eq 'Authors')
		{ # creators
			$data[$count]->{'creator'} = []; # initialize 'cre' field
		}
		elsif ($elem ne '')
		{
			if ($buf eq 'ProductName')
			{ # product name
				$data[$count]->{'name'} = $elem;
			}
			elsif ($buf eq 'Asin')
			{ # asin
				$data[$count]->{'asin'} = $elem;
			}
			elsif ($buf eq 'Catalog')
			{ # catalog
				$data[$count]->{'catalog'} = $elem;
			}
			elsif ($buf eq 'Artist' or $buf eq 'Author')
			{ # each creator
				push(@{$data[$count]->{'creator'}},$elem);
			}
			elsif ($buf eq 'ReleaseDate')
			{ # release date
				$data[$count]->{'date'} = $elem;
			}
			elsif ($buf eq 'Manufacturer')
			{ # manufacturer
				$data[$count]->{'maker'} = $elem;
			}
			elsif ($buf eq 'ImageUrlSmall')
			{ # small image
				$data[$count]->{'image_small'} = $elem;
			}
			elsif ($buf eq 'ImageUrlMedium')
			{ # medium image
				$data[$count]->{'image_medium'} = $elem;
			}
			elsif ($buf eq 'ImageUrlLarge')
			{ # large image
				$data[$count]->{'image_large'} = $elem;
			}
			elsif ($buf eq 'Availability')
			{ # availability
				$data[$count]->{'avaiable'} = $elem;
			}
			elsif ($buf eq 'ListPrice')
			{ # list price
				$data[$count]->{'list_price'} = $elem;
			}
			elsif ($buf eq 'OurPrice')
			{ # their price
				$data[$count]->{'sale_price'} = $elem;
			}
			elsif ($buf eq 'UsedPrice')
			{ # used price
				$data[$count]->{'used_price'} = $elem;
			}
		}
	};
	sub _xml_end
	{
		my $self = shift;
		my $tag = shift;
		$count++ if ($tag eq 'Details');
		$buf = undef;
		return();
	};
}
# --------------------------------------
# // private functions
sub _get_content
{
	my $self = shift;
	my %param = @_;
	my $lang = Spica::Language->get;
	my $content = undef;
	$param{'KeywordSearch'} = $lang->convert($param{'KeywordSearch'},'utf8',$param{'charcode'});
	$content = $self->_connect(%param);
	$content = $lang->convert($content,$param{'charcode'},'utf8') if ($text ne '');
	if ($content =~ /<ErrorMsg>(.*?)<\/ErrorMsg>/)
	{
		$self->error($1);
		$content = undef;
	}
	return $content;
}
sub _connect
{
	my $self  = shift;
	my %param = @_;
	my $ua = $self->agent;
	$self->{'connected'} = 1;
	if (!$ua)
	{
		$self->error(AWS_ERROR_INITIAL);
		return;
	}
	my $url = &_get_server_address($param{'locale'});
	$param{'t'} = AWS_ASSOCIATE if ($param{'t'} eq '');
	foreach my $query ( keys(%param) )
	{
		next if ($query eq 'charcode');
		next unless ( defined($param{$query}) );
		$url .= $query . '=' . &_encode_uri($param{$query}) . '&';
	}
	chop($url) if ($url =~ /\&$/);
	my $response = $ua->get($url);
	if ( index($response->{'_rc'},'2') == 0 )
	{
		return $response->{'_content'};
	}
	else
	{
		$self->error(AWS_ERROR_CONNECT);
		return;
	}
}
sub _convert_param
{
	my %in = @_;
	my %out = ();
	my %aTable = (
		'id'       => 't',
		'keyword'  => 'KeywordSearch',
		'asin'     => 'AsinSearch',
		'page'     => 'page',
		'genre'    => 'mode',
		'locale'   => 'locale',
		'charcode' => 'charcode',
	);
	while ( my ($from,$dest) = each(%aTable) )
	{
		$out{$dest} = $in{$from};
	}
	$out{'dev-t'} = AWS_DEVELOPER;
	$out{'type'}  = AWS_TYPE;
	$out{'f'}     = AWS_CODETYPE;
	return %out;
}
sub _get_server_address
{
	my $locale = shift;
	return AWS_SERVER_JP if ($locale eq AWS_LOCALE_JP);
	return AWS_SERVER_US;
}
sub _encode_uri
{
	$_[0] =~ s/(\W)/'%' . unpack('H2', $1)/eg;
	return($_[0]);
}
1;
__END__
=head1 NAME

Spica::Net::Aws - Amazon Web Services Handler

=head1 SYNOPSIS

	require Spica::Net::Aws;
	my $aws = Spica::Net::Aws->new;
	my $result = $aws->get_data(
		'id'        => 'webservices-20', # Amazon Associate ID
		'genre'     => 'category',       # Search Catgeory e.g. books-jp, books-us ....
		'keyword'   => 'keyword',        # Search Keyword
		'asin'      => 'asin',           # Search ASID
		'page'      => 0,                # Display Page
		'charcode'  => 'utf8',           # Char Code e.g. utf8, euc, sjis ....
	);
	if (!$aws->error)
	{
		my $matched = $aws->matched; # get number of total results
		my $pages = $aws->page; # get number of the last page
		for (my $i=0;$i<$aws->count;$i++)
		{
			my $item = $result->[$i];
			# handles each item of results
		}
	}

=head1 DESCRIPTION

Spica::Net::Aws handles serching on Amazon.co.jp/Amazon.com via AWS.

Spica::Net::Aws::get_data works as xml parser and returns search results of AWS as an array reference.
Spica::Net::Aws::get_data can be used following parameters
	'id'       # Amazon Associate ID
	'genre'    # Search Catgeory e.g. books-jp, books-us ....
	'keyword'  # Search by Keyword
	'asin'     # Search by ASID
	'page'     # Display Page
	'charcode' # Output Char Code

Spica::Net::Aws::get_data returns an array reference.
Each element of array is a hash reference that has following data.
	'url'          # url for product
	'asin'         # asin of product
	'name'         # product name
	'catalog'      # catalog
	'creator'      # creators (stores as array)
	'date'         # release date
	'maker'        # manufacturer
	'image_small'  # url for small image
	'image_medium' # url for medium image
	'image_large'  # url for large image
	'avaiable'     # availability
	'list_price'   # list price
	'sale_price'   # price on amazon
	'used_price'   # used price

=head1 AUTHOR

Takuya Otani <mailto:takuya.otani@gmail.com> http://serennz.sakura.ne.jp/

=head1 LICENSE

Copyright (C) 2004-2005 Takuya Otani(SimpleBoxes) and SerendipityNZ Ltd.

=cut
