# Spica::Net::Mailer
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==

# ======================================
package Spica::Net::Mailer;
use strict;
use Spica::Language;
use Spica::Net;
use base qw( Spica::Net );
use vars qw( $VERSION );
$VERSION = '0.02';
# 0.02 [2007/08/01] changed sendmail to handle Cc and Bcc
# 0.01 [2006/03/17] changed bracket rule
# 0.00 [2005/10/09] created
# --------------------------------------
# // defines / static variables
sub ERROR_NOT_SUPPORTED    (){ "indicated type is not supported : " };
sub ERROR_SMTP_ERROR       (){ "failed to create Net::SMTP\n" };
sub ERROR_NO_DESTINATION   (){ "no destination address\n" };
sub ERROR_DISABLE_SENDMAIL (){ "cannot use sendmail, please check the path for sendmail\n" };
sub ERROR_NOTOPEN_SENDMAIL (){ "cannot open via sendmail\n" };
sub ERROR_NO_ADDRESS       (){ "need to set pop server address\n" };
sub ERROR_NO_ACCOUNT       (){ "need to set pop account\n" };
sub ERROR_NO_PASSWORD      (){ "need to set pop password\n" };
sub ERROR_POP3_ERROR       (){ "failed to create Net::POP3ss\n" };
sub ERROR_AUTHORIZATION    (){ "failed to authorize\n" };
# --------------------------------------
# // override base class
sub initialize
{
	my $self = shift;
	my %param = (
		'address'  => 'anonymous@example.com',
		'sender'   => {
			'type'    => 'sendmail',
			'address' => '/usr/sbin/sendmail',
		},
		'receiver' => {
			'address'  => undef,
			'acsount'  => undef,
			'password' => undef,
			'is_apop'  => 0,
		},
		'charcode' => 'jis',
		@_
	);
	return $self->SUPER::initialize(%param);
}
# --------------------------------------
# // public functions
sub sendmail
{ # send a mail
	my $self = shift;
	my %param = (
		'To'      => undef,
		'Cc'      => undef,
		'Bcc'     => undef,
		'From'    => $self->{'address'},
		'Subject' => 'no title',
		'Body'    => '',
		@_,
	);
	my $head = $self->head(%param);
	my $body = Spica::Language->get->convert($param{'Body'},$self->{'charcode'});
	eval
	{
		die(ERROR_NO_DESTINATION) if (!$param{'To'});
		my $server = $self->{'sender'}{'address'};
		my $from = $param{'From'};
		SWITCH_TYPE: {
			$_ = $self->{'sender'}{'type'};
			/^sendmail/ && do
			{
				die(ERROR_DISABLE_SENDMAIL) if !(-x $server);
				open(SENDMAIL,"| $server -t -i -f '$from'") or die(ERROR_NOTOPEN_SENDMAIL);
				print SENDMAIL $head;
				print SENDMAIL $body;
				close(SENDMAIL);
				last SWITCH_TYPE;
			};
			/^smtp/ && do
			{
				require Net::SMTP;
				my $smtp = Net::SMTP->new($server,'Hello'=>$server);
				$smtp->mail($from);
				$smtp->to($param{'To'});
				$smtp->cc($param{'Cc'}) if ($param{'Cc'});
				$smtp->bcc($param{'Bcc'}) if ($param{'Bcc'});
				$smtp->data();
				$smtp->datasend($head);
				$smtp->datasend($body);
				$smtp->dataend();
				$smtp->quit;
				last SWITCH_TYPE;
			};
			die(ERROR_NOT_SUPPORTED . $_ . "\n");
		}; # end of SWITCHE_TYPE
	}; # end of eval
	return ($@) ? undef : 1;
}
sub head
{ # generate header
	my $self = shift;
	my %param = @_;
	my $header;
	my $lang = Spica::Language->get;
	foreach my $key ( keys(%param) )
	{
		next if ($key eq 'Body');
		$header .= $key . ': ' . $lang->convert($param{$key},$self->{'charcode'}) . "\n";
	}
	$header .= 'MIME-Version: 1.0' . "\n";
	$header .= 'Content-type: text/plain; charset=' . $lang->charset_of($self->{'charcode'}) . "\n";
	$header .= 'Content-Transfer-Encoding: 7bit' . "\n";
	$header .= 'X-Mailer: ' . $self->agent_name . "\n";
	$header .= "\n"; # boundary between head and body
	require 'mimeutil.pl';
	return &mimeutil::mimeencode($header);
}
sub receive
{ # receive mail
	my $self = shift;
	my %param = (
		'subj'  => undef, # indicates subject to receive
		'msgid' => undef, # indicates message-id of mails which have been received
		'from'  => undef, # indicates from to receive (reference of array)
		'size'  => undef, # indicates maximum size of mail to receive (-1 means no limitation)
		'del'   => undef, # indicates flag whether remove the mail after receiving or not.  (1 means removing)
		@_,
	);
	my @mail; # output is array of mails which are received
	die(ERROR_NO_ADDRESS) if (!defined($self->{'receiver'}{'address'}) or $self->{'receiver'}{'address'} eq '');
	die(ERROR_NO_ACCOUNT) if (!defined($self->{'receiver'}{'account'}) or $self->{'receiver'}{'account'} eq '');
	die(ERROR_NO_PASSWORD) if (!defined($self->{'receiver'}{'password'}) or $self->{'receiver'}{'password'} eq '');
	eval {
		require Net::POP3;
		my $pop3 = Net::POP3->new($self->{'receiver'}{'address'}) or die(ERROR_POP3_ERROR);
		my $check_auth = undef;
		if ($self->{'receiver'}{'is_apop'})
		{
			$check_auth = $pop3->apop($self->{'receiver'}{'account'},$self->{'receiver'}{'password'});
		}
		else
		{
			$pop3->user($self->{'receiver'}{'account'});
			$check_auth = $pop3->pass($self->{'receiver'}{'password'});
		}
		die(ERROR_AUTHORIZATION) if (!$check_auth);
		my $msglist = $pop3->list();
		foreach my $msgid (keys %$msglist)
		{
			my ($tmp,$data);
			next if (${$msglist}{$msgid} > $param{'size'} and $param{'size'} > -1); # check mail size
			$tmp = $pop3->top($msgid); # get header
			$data = join('',@$tmp);
			$data = &_linefeed($data);
			$data .= "\n" if ($data !~ /\n$/); # add linefeed end of data
			$tmp = $self->extract_head($data,'Subject'); # extract Subject
			next if ($tmp eq '');
			next if ($param{'subj'} ne '' and index($tmp,$param{'subj'}) == -1);
			if ($param{'from'})
			{
				my $frm_flag;
				$tmp = $self->extract_head($data,'From'); # extract From
				foreach (@{$param{'from'}})
				{
					$frm_flag = 1 if (index($tmp,$_) > -1);
					last if ($frm_flag);
				}
				next if (!$frm_flag);
			}
			if ($param{'msgid'} ne '')
			{
				$tmp = $self->extract_head($data,'Message-ID');
				last if ($tmp eq $param{'msgid'}); # check Message-ID
			}
			$tmp = $pop3->get($msgid); # get mail
			$data = join('',@$tmp) . "\n.\n";
			$data = &_linefeed($data);
			push(@mail,$data); # store mail
			$pop3->delete($msgid) if ($param{'del'}); # remove mail from server
		}
		$pop3->quit; # finish
	};
	die($@) if ($@); # error
	return(@mail);
	# [references] ==============================================
	# Let's code POP3 client with perl module! (in Japanese)
	# http://x68000.q-e-d.net/~68user/net/module-pop3.html
	# Receiving mail with Perl (in Japanese)
	# http://homepage3.nifty.com/hippo2000/perltips/rcvmail.htm
	# =========================================================
}
sub parse_text
{ # parsing text of mail
	my $self = shift;
	my $data = shift;
	my $out = {
		'head' => '',
		'body' => '',
		'boundary' => '',
	};
	if ($data =~ /^(.*?)\n\n(.*?)\n\.\n$/s)
	{
		$out->{'head'} = $1 . "\n";
		$out->{'body'} = $2;
	}
	if ($out->{'head'} =~ /Content-Type:\s?multipart\/mixed;\n?\s?boundary="(.*?)"/si)
	{
		$out->{'boundary'} = '--' . $1;
	}
	else
	{
		$out->{'boundary'} = '';
	}
	return($out);
}
sub multipart
{ # parsing multipart
	my $self = shift;
	my $data = shift; # output of parse_text
	my @out;
	my @parts = split($data->{'boundary'}, $data->{'body'});
	shift @parts; # the last of @parts is not used.
	require 'mimeutil.pl';
	for (my $i=0;$i<@parts;$i++)
	{
		if($parts[$i] =~ /(.*?)\n\n(.*)/s){
			my ($head,$body) = ($1,$2);
			$out[$i]->{'head'} = $head;
			$out[$i]->{'type'} = &_part_type($head);
			$out[$i]->{'code'} = &_part_encode($head);
			$out[$i]->{'name'} = &_part_name($head);
			if ($out[$i]->{'code'} =~ /Base64/i)
			{
				$out[$i]->{'body'} = &mimeutil::bodydecode($body,'b64');
				$out[$i]->{'body'} .= &mimeutil::bdeflush('b64');
			}
			elsif ($out[$i]->{'code'} =~ /Quoted/i)
			{
				$out[$i]->{'body'}  = &mimeutil::bodydecode($body,'qp');
				$out[$i]->{'body'} .= &mimeutil::bdeflush('qp');
			}
			else
			{
				$out[$i]->{'body'} = $body;
			}
		}
	}
	return(\@out);
}
sub extract_head
{ # extracting header
	my $self = shift;
	my ($elem,$head) = @_;
	my $out;
	require 'mimeutil.pl';
	if ($head =~ /$elem: (.*?)\n([\w|-]*?): /s)
	{ # exists at middle of headers
		$out = $1;
	}
	elsif ($head =~ /$elem: (.*?)\n$/s)
	{ # exists at the end of headers
		$out = $1;
	}
	$out = &mimeutil::mimedecode($out, 'EUC'); # output charcode is euc
	$out =~ s/\n\s?//g;
	$out =~ s/\s$//;
	return($out);
}
# --------------------------------------
# // private functions
sub _linefeed
{
	$_[0] =~ s/\x0D\x0A/\n/g;
	$_[0] =~ tr/\x0D\x0A/\n\n/;
	return($_[0]);
}
sub _part_type
{ # get Content-Type
	my $head = shift;
	return ($head =~ /Content-Type: (.*?);/si) ? $1 : undef;
}
sub _part_name
{ # get file name
	my $head = shift;
	return ($head =~ /Content-Type:\s?.*?;\n?\s*name="(.*?)"/si) ? $1 : undef;
}
sub _part_encode
{ # get encode type / Copyright (C) 2004 Kei Sato
	my $head = shift;
	return ($head =~ /Content-Transfer-Encoding: (.*)/si) ? $1 : undef;
}
1;
__END__
