# Spica::TemplateManager
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==
# == Copyright (C) 2005 Spica Project Kei/Fuco/Takkyun ==

# ======================================
package Spica::TemplateManager;
use strict;
use integer;
use Carp;
use Spica::Object;
use vars qw( $VERSION );
$VERSION = '0.05';
# 0.05 [2006/07/04] got rid of "return( undef )"
# 0.04 [2006/03/17] changed bracket rule
# 0.03 [2005/11/15] porting from Serene Bach
# 0.02 [2005/10/19] changed block structure to store contents as an array instead of text
# 0.01 [2005/10/18] added clear
# 0.00 [2004/11/17] generated
# --------------------------------------
# // defines / static variables
sub BLOCKS (){ 0 }
sub DATA   (){ 1 }
sub NUMBER (){ 2 }
sub TAG    (){ 0 }
sub BLOCK  (){ 1 }
sub TEXT   (){ 0 }
sub TAGS   (){ 1 }
sub NAME   (){ 2 }
# --------------------------------------
# // override base class
sub new
{
	my $class = shift;
	my $self = bless [], $class;
	return $self->initialize(@_);
}
sub initialize
{
	my $self = shift;
	my $template = shift;
	$self->[BLOCKS] = []; # array for blocks
	$self->[NUMBER] = 0;  # number of proceeded block
	$self->[DATA]   = []; # hash for data
	$self->[DATA][TAG]   = {};
	$self->[DATA][BLOCK] = {};
	croak('Need to set template') unless ( $template );
	$self->_parse($template);
	return $self;
}
# --------------------------------------
# // public functions
sub output
{ # evolving template
	my $self = shift;
	my $output = '';
	for (my $i=0;$i<@{$self->[BLOCKS]};$i++)
	{
		my $rep = ($self->[BLOCKS][$i][NAME] eq '_main')
		        ? 1
		        : $self->[DATA][BLOCK]{$self->[BLOCKS][$i][NAME]};
		for (my $j=0;$j<$rep;$j++)
		{
			for (my $k=0;$k<@{$self->[BLOCKS][$i][TEXT]};$k++)
			{
				my $line = $self->[BLOCKS][$i][TEXT][$k];
				foreach my $key (@{$self->[BLOCKS][$i][TAGS]})
				{
					next if ( index($line,$key) == -1 );
					if ( defined($self->[DATA][TAG]{$key}[$j]) )
					{
						$line =~ s/$key/$self->[DATA][TAG]{$key}[$j]/g;
					}
					else
					{
						$line =~ s/$key//g;
					}
				}
				$output .= $line . "\n";
			}
		}
	}
	return($output);
}
sub deleteBlock
{ # delete block
	my $self  = shift;
	my $block = shift; # block name to delete
	croak('lack of parameter') unless( defined($block) );
	if ( $self->existed($block) )
	{
		my $check = 0;
		for (my $i=0;$i<@{$self->[BLOCKS]};$i++)
		{
			next if ($self->[BLOCKS][$i][NAME] ne $block);
			$check = $i;
			last;
		}
		# Are both blocks of neighbour the same?
		if (  $check > 0 
		  and $check < $#{$self->[BLOCKS]} 
		  and $self->[BLOCKS][$check - 1][NAME] eq $self->[BLOCKS][$check + 1][NAME])
		{
			# merge text
			push(
				@{$self->[BLOCKS][$check - 1][TEXT]},
				@{$self->[BLOCKS][$check + 1][TEXT]}
			);
			# merge tags
			push(
				@{$self->[BLOCKS][$check - 1][TAGS]},
				@{$self->[BLOCKS][$check + 1][TAGS]}
			);
			# remove block
			splice(@{$self->[BLOCKS]},$check,2);
		}
	}
	return();
}
sub clear
{
	my $self = shift;
	$self->[DATA]{'tag'} = {};
	foreach my $block ( keys(%{$self->[DATA][BLOCK]}) )
	{
		$self->[DATA][BLOCK]{$block} = 0;
	}
}
sub tag
{
	my $self = shift;
	my ($tag,$content) = @_; # tag name, content
	my $num = $self->[NUMBER];
	if ( defined($tag) )
	{
		$self->[DATA][TAG]{'{' . $tag . '}'}[$num] = $content;
	}
	else
	{
		croak('Lack of parameters');
	}
	return();
}
sub num
{
	my $self = shift;
	$self->[NUMBER] = shift;
}
sub block
{
	my $self  = shift;
	my ($block,$num) = @_; # block name, repetition number
	if ( defined($block) and defined($num) )
	{
		$self->[DATA][BLOCK]{$block} = $num;
	}
	else
	{
		croak('Lack of parameters');
	}
	return();
}
sub exists
{
	my $self  = shift;
	my $block = shift; # block name
	if ( defined($block) )
	{
		return( exists($self->[DATA][BLOCK]{$block}) );
	}
	else
	{
		return;
	}
}
sub unifyFor
{
	my $self  = shift;
	my $tag   = shift; # tag name
	my $block = shift; # block to indicate
	my $key = '{' . $tag . '}';
	my $num  = @{$self->[DATA][TAG]{$key}}; # repetition number
	$num = $self->[DATA][BLOCK]{$block} if ( defined($block) );
	my $content = $self->[DATA][TAG]{$key}[0];
	if ( defined($content) and $num > 1 )
	{
		for (my $i=1;$i<$num;$i++)
		{
			$self->[DATA][TAG]{$key}[$i] = $content;
		}
	}
	return();
}
# --------------------------------------
# // private functions
sub _parse
{
	my $self = shift;
	my $template = shift;
	my $num = 0;
	my $cur = '_main';
	my @stack = ();
	$self->_set($num,$cur);
	push(@stack,$cur);
	foreach my $line (split("\n",$template))
	{
		if ($line =~ /<\!-- BEGIN (\w+) -->/)
		{
			$num++;
			$cur = $1;
			$self->_set($num,$cur);
			$self->[DATA][BLOCK]{$cur} = 0; # initialize for block
			push(@stack,$cur);
			next;
		}
		elsif (index($line,'<!-- END ') > -1)
		{
			$num++;
			pop(@stack);
			$cur = $stack[$#stack];
			$self->_set($num,$cur);
			next;
		}
		if ($line =~ /\{\w+?\}/)
		{
			push(@{$self->[BLOCKS][$num][TAGS]},($line =~ /(\{\w+?\})/g));
		}
		push(@{$self->[BLOCKS][$num][TEXT]},$line);
	}
	return();
}
sub _set
{
	my $self = shift;
	my ($num,$name) = @_;
	$self->[BLOCKS][$num] = [];
	$self->[BLOCKS][$num][TEXT] = [];
	$self->[BLOCKS][$num][TAGS] = [];
	$self->[BLOCKS][$num][NAME] = $name;
}
1;
__END__
