# Spica::Time
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==
# == Copyright (C) 2005 Spica Project Kei/Fuco/Takkyun ==

# ======================================
package Spica::Time;
use strict;
use Spica::Language;
use Spica::Object;
use base qw( Spica::Object );
use vars qw( $VERSION );
$VERSION = '0.11';
# 0.11 [2006/03/27] added BEGIN block to load Time::Local::Extended
# 0.10 [2006/03/17] changed bracket rule
# 0.09 [2006/03/06] changed _convert to avoid an error at gmtime
# 0.08 [2006/01/08] changed _format to add %ZoneLong%
# 0.07 [2005/11/24] added data to return an entity of object
# 0.06 [2005/11/02] added diff to calculate day before/after target day
# 0.05 [2005/10/25] changed _format to handle Language object correctly
# 0.04 [2005/10/21] refactored
# 0.03 [2005/10/20] changed _format to add %Zone%
# 0.02 [2005/02/08] defined input parameter as hash
# 0.01 [2005/02/01] move into util
# 0.00 [2004/11/20] porting from sbtime.pl
# --------------------------------------
# // initializer
BEGIN
{
	eval{ require Time::Local::Extended; };
	require Time::Local if ($@);
}
# --------------------------------------
# // override base class
sub new
{
	my $class = shift;
	my $now = undef;
	if ($_[0] eq '')
	{
		$now = time();
	}
	elsif ($_[0] =~ /^\d+$/)
	{
		$now = shift;
	}
	else
	{
		$now = &_convert(@_);
	}
	return bless \$now, $class;
}
# --------------------------------------
# // public functions - class method
sub get_weekday
{ # from http://www.din.or.jp/%7Eohzaki/perl.htm / Copyright (C) 1999-2004 OHZAKI Hiroki
	my $class = shift;
	my %param = (
		'year' => 1970,
		'mon'  => 1,
		'day'  => 1,
		@_
	);
	if ($param{'mon'} == 1 or $param{'mon'} == 2)
	{
		$param{'year'}--;
		$param{'mon'} += 12;
	}
	int($param{'year'} + int($param{'year'} / 4) - int($param{'year'} / 100) + int($param{'year'} / 400)
	+ int((13 * $param{'mon'} + 8) / 5) + $param{'day'}) % 7;
}
sub get_lastday
{ # from http://www.din.or.jp/%7Eohzaki/perl.htm / Copyright (C) 1999-2004 OHZAKI Hiroki
	my $class = shift;
	my %param = (
		'year' => 1970,
		'mon'  => 1,
		@_
	);
	(31,28,31,30,31,30,31,31,30,31,30,31)[$param{'mon'} - 1] 
	+ (($param{'mon'} == 2) and &_leapcheck($param{'year'}));
}
sub diff_timezone
{
	my $class = shift;
	my ($zone1,$zone2) = @_;
	&_zone($zone1) - &_zone($zone2);
}
# --------------------------------------
# // public functions
sub format
{ # formated time, it can be called as class method too.
	my $self = shift;
	my $now  = ref($self) ? $$self : undef;
	my %param = (
		'time' => $now,    # Unix Time
		'form' => '',      # Format
		'zone' => '+0000', # default time zone
		'lang' => 'en',    # default language code
		@_
	);
	return( &_format( %param ) );
}
sub data
{
	my $self = shift;
	$$self;
}
sub diff
{
	my $self = shift;
	my %param = (
		'day'  => 0,
		'hour' => 0,
		'min'  => 0,
		'sec'  => 0,
		@_
	);
	my $diff = $param{'sec'};
	$diff += 60 * $param{'min'};
	$diff += 60 * 60 * $param{'hour'};
	$diff += 24 * 60 * 60 * $param{'day'};
	return Spica::Time->new($$self + $diff);
}
# --------------------------------------
# // private functions
sub _format
{
	my %param = @_;              # input parameter
	my $output = $param{'form'}; # output parameter
	my $zone = &_zone($param{'zone'}); # time zone
	my $lang = Spica::Language->get;    # get a language instance
	if ($param{'time'} and $param{'form'})
	{
		my ($se,$mi,$ho,$md,$mo,$ye,$wd,$yd,$is) = gmtime($param{'time'} + $zone);
		if (index($output,'%Year') > -1)
		{ # Year
			my $tmp = $ye += 1900;
			$output =~ s/%YearLong%/$tmp/g;
			$output =~ s/%Year%/$tmp/g;
			$tmp = &_pad0($ye % 100);
			$output =~ s/%YearShort%/$tmp/g;
		}
		if (index($output,'%Mon') > -1)
		{ # Month
			my $tmp = &_pad0($mo + 1);
			my $check = undef;
			$check = $lang->string('month_' . $param{'lang'}) if ($lang);
			$tmp = $check->[$mo] if ($check  and ref($check) eq 'ARRAY');
			$output =~ s/%MonShort%/$tmp/g;
			$check = $lang->string('month_' . $param{'lang'} . 'long') if ($lang);
			$tmp = $check->[$mo] if ($check  and ref($check) eq 'ARRAY');
			$output =~ s/%MonLong%/$tmp/g;
			$tmp = $mo + 1;
			$output =~ s/%MonNumShort%/$tmp/g;
			$output =~ s/%MonNum%/$tmp/g;
			$tmp = &_pad0($mo + 1);
			$output =~ s/%MonNumLong%/$tmp/g;
			$output =~ s/%MonNumPad%/$tmp/g;
			$output =~ s/%Mon%/$tmp/g;
		}
		if (index($output,'%Day') > -1)
		{ # Day
			my $tmp = $md . &_nsuf($md);
			$output =~ s/%DayOrd%/$tmp/g;
			$tmp = $md;
			$output =~ s/%DayShort%/$tmp/g;
			$tmp = &_pad0($md);
			$output =~ s/%DayLong%/$tmp/g;
			$output =~ s/%DayPad%/$tmp/g;
			$output =~ s/%Day%/$tmp/g;
		}
		if (index($output,'%Week') > -1)
		{ # Week
			my $tmp = '';
			my $check = undef;
			$check = $lang->string('week_' . $param{'lang'}) if ($lang);
			$tmp = $check->[$wd] if ($check  and ref($check) eq 'ARRAY');
			$output =~ s/%WeekShort%/$tmp/g;
			$output =~ s/%Week%/$tmp/g;
			$check = $lang->string('week_' . $param{'lang'} . 'long') if ($lang);
			$tmp = $check->[$wd] if ($check  and ref($check) eq 'ARRAY');
			$output =~ s/%WeekLong%/$tmp/g;
		}
		if (index($output,'%Hour') > -1)
		{ # Hour
			my $tmp = ($ho < 12) ? &_pad0($ho) : &_pad0($ho - 12);
			$output =~ s/%Hour11%/$tmp/g;
			$tmp = 12 if ($tmp == 0);
			$output =~ s/%Hour12%/$tmp/g;
			$tmp = &_pad0($ho);
			$output =~ s/%Hour%/$tmp/g;
			$tmp = $ho;
			$output =~ s/%Hour24%/$tmp/g;
			$tmp = ($ho < 12) ? 'AM' : 'PM';
			$output =~ s/%HourAP%/$tmp/g;
		}
		if (index($output,'%Min') > -1)
		{ # Minute
			my $tmp = &_pad0($mi);
			$output =~ s/%Min%/$tmp/g;
		}
		if (index($output,'%Sec') > -1)
		{ # Second
			my $tmp = &_pad0($se);
			$output =~ s/%Sec%/$tmp/g;
		}
		if (index($output,'%Zone') > -1)
		{ # Time Zone
			my $hour = substr($param{'zone'},0,3);
			my $min  = substr($param{'zone'},3,2);
			$output =~ s/%Zone%/$hour$min/g;
			$output =~ s/%ZoneLong%/$hour\:$min/g;
		}
	}
	return($output);
}
sub _convert
{ # convert to unix time
	my %p = (
		'year' => 1970, # must be later than 1970
		'mon'  => 1,    # must be within 1 - 12
		'day'  => 1,    # must be more than 0
		'hour' => 0,
		'min'  => 0,
		'sec'  => 0,
		'zone' => '+0000',
		@_,
	);
	my $out = undef;
	$p{'year'} = int($p{'year'}) - 1900;
	$p{'mon'}  = int($p{'mon'}) - 1;
	$p{'zone'} = &_zone($p{'zone'});
	eval{ $out = timegm($p{'sec'},$p{'min'},$p{'hour'},$p{'day'},$p{'mon'},$p{'year'}) - $p{'zone'} };
	return ($@) ? undef : $out;
}
sub _zone
{ # converting time zone
	return( ($1 eq '-') ? -3600 * $2 - 60 * $3 : 3600 * $2 + 60 * $3 ) if ($_[0] =~ /^([\+-])(\d\d)(\d\d)/);
	return(0);
}
sub _nsuf
{ # suffix for ordinal number
	return('st') if ( ($_[0] % 10) == 1 and $_[0] != 11 );
	return('nd') if ( ($_[0] % 10) == 2 and $_[0] != 12 );
	return('rd') if ( ($_[0] % 10) == 3 and $_[0] != 13 );
	return('th');
}
sub _pad0
{ # padding 0
	return( ($_[0] < 10) ? '0' . $_[0] : $_[0] );
}
sub _leapcheck
{
	return($_[0] % 4 == 0 and ($_[0] % 400 == 0 or $_[0] % 100 != 0));
}
1;
__END__

=head1 NAME

Spica::Time - time object

=head1 SYNOPSIS

 use Spica::Time;
 my $time_obj = Spica::Time->new; # set current time
 my $format = '%Week%, %DayShort%-%MonShort%-%YearLong% %Hour24%:%Min%:%Sec% GMT';
 my $formated_time = $time_obj->format('form'=>$format);

=head1 DESCRIPTION

[time format module]
 format method returns formated text.

[input paramaeters]
 'form' => Format,        ex) '%Year%/%Mon%/%Day% %Hour%:%Min%:%Sec'
 'zone' => Time Zone,     ex) +0900 [\-+](\d\d)(\d\d)
 'lang' => Language Code, ex) 'en','ja' and so on

[format symbols]
 description         : symbol                               :example
 --------------------:--------------------------------------:--------
 A.D. 4 digits       : %Year% or %YearLong%                 :2005
 A.D. 2 dgits        : %YearShort%                          :05
 Month 2 digits      : %Mon% or %MonNumLong% or %MonNumPad% :02
 Month Short Name    : %MonShort%                     [lang]:Feb.
 Month Long Name     : %MonLong%                      [lang]:February
 Month (Number)      : %MonNumShort% or %MonNum%            :2
 Day 2 digits        : %Day% or %DayLong% or %DayPad%       :01
 Day (Ordinal number): %DayOrd%                             :1st
 Day                 : %DayShort%                           :1
 Day of week         : %Week% or %WeekShort%          [lang]:Tue
 Day of week (long)  : %WeekLong%                     [lang]:Tuesday
 Hour 2 digits       : %Hour%                               :00
 Hour (24 hours)     : %Hour24%                             :0
 Hour (0-11 hours)   : %Hour11%                             :00
 Hour (1-12 hours)   : %Hour12%                             :12
 AM/PM               : %HourAP%                             :AM
 Minute 2 digits     : %Min%                                :01
 Second 2 digits     : %Sec%                                :02
 Timezone            : %Zone%                               :+0900
 Timezone with colon : %ZoneLong%                           :+09:00

[note]
 [lang] in symbol means output string depends on language.

=head1 AUTHOR

Takuya Otani <mailto:takuya.otani@gmail.com> http://serennz.sakura.ne.jp/

=head1 LICENSE

Copyright (C) 2005 Takuya Otani@SimpleBoxes / SerendipityNZ Ltd.

=cut
