# Spica::Util::Skeleton
# == Author(s) : Takuya Otani <takuya.otani@gmail.com> ==
# == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==
# == Copyright (C) 2005 Spica Project Kei/Fuco/Takkyun ==

# ======================================
package Spica::Util::Skeleton;
use strict;
use Spica::Object;
use base qw( Spica::Object );
use vars qw( $VERSION );
$VERSION = '0.03';
# 0.03 [2006/07/30] modified javascript_base to up to date
# 0.02 [2006/04/06] changed template to use base64 encoded data
# 0.01 [2006/04/06] changed string_base to removed default value
# 0.00 [2006/03/30] generated
# --------------------------------------
# // defines / static variables
# ... nothing ...
# --------------------------------------
# // override base class
# ... nothing ...
# --------------------------------------
# // public functions
sub skeleton
{
	my $class = shift;
	my $type = shift;
	my $head = $class->header;
	my $body = $class->$type();
	$body =~ s/__SPICA_HEADER__/$head/;
	$body;
}
sub output
{
	my $class = shift;
	my $data = shift;
	$data =~ s/^\s+//;
	$data =~ s/\n\s+/\n/g;
	$data =~ s/^#\s//;
	$data =~ s/\n#\s/\n/g;
	$data;
}
sub decode
{
	my $class = shift;
	my $data = $class->output(shift);
	my $output = undef;
	require 'mimeutil.pl';
	$output  = &mimeutil::bodydecode($data,'b64');
	$output .= &mimeutil::bdeflush('b64');
	$output;
}
sub header
{
	my $class = shift;
	return $class->output(<<"__EOD__");
	# # == Powered by Spica $Spica::LIBRARY
	# # == Copyright (C) 2005 SimpleBoxes/SerendipityNZ Ltd. ==
	# # == Copyright (C) 2005 Spica Project Kei/Fuco/Takkyun ==
__EOD__
}
sub script_base
{
	my $class = shift;
	return $class->decode(<<'__EOD__');
	# IyFfX1BFUkxfUEFUSF9fCiMgCiMgbWFpbiBzY3JpcHQgLSBfX1BST0pFQ1RfTkFNRV9fCl9f
	# U1BJQ0FfSEVBREVSX18KdXNlIHN0cmljdDsKdXNlIGxpYiBxdyguIC4vbGliIC4vYmFzZSAu
	# L2V4dCk7CnJlcXVpcmUgJ2FkZGxpYicgaWYgKC1yICdhZGRsaWInKTsKCkJFR0lOCnsgIyBj
	# aGFuZ2UgY3VycmVudCBkaXJlY3RvcnkgZm9yIG1vZF9wZXJsCglpZiAoJEVOVnsnTU9EX1BF
	# UkwnfSkKCXsKCQlteSAkcGF0aCA9ICRFTlZ7J1NDUklQVF9GSUxFTkFNRSd9OwoJCSRwYXRo
	# ID1+IHMvXiguKilbOlwvXFxdLiokLyQxLzsKCQljaGRpcigkcGF0aCk7Cgl9Cn0KZXZhbAp7
	# CglyZXF1aXJlIFNwaWNhOwoJU3BpY2EtPnJ1bignTWFpbicpOwp9OwppZiAoJEApCnsKCXBy
	# aW50ICJDb250ZW50LVR5cGU6IHRleHQvcGxhaW5cblxuIjsKCXByaW50ICRAOwp9Cg==
__EOD__
}
sub config_base
{
	my $class = shift;
	return $class->decode(<<'__EOD__');
	# IyBjb25maWcucGwgLSBjb25maWd1cmF0aW9uIGZvciBfX1BST0pFQ1RfTkFNRV9fCl9fU1BJ
	# Q0FfSEVBREVSX18KcGFja2FnZSBTcGljYTo6Q29uZmlnOwoKJFNwaWNhOjpQUk9EVUNUID0g
	# J19fUFJPSkVDVF9OQU1FX18nOwokU3BpY2E6OlZFUlNJT04gPSAnMC4wMCc7CiMgMC4wMCBb
	# X19HRU5FUkFURURfREFURV9fXSBnZW5lcmF0ZWQKCnN1YiBzZXR0aW5nCnsKCXJldHVybiAo
	# Cl9fQ09ORklHVVJBVElPTl9fCgkpOwp9CjE7Cl9fRU5EX18K
__EOD__
}
sub app_base
{
	my $class = shift;
	return $class->decode(<<'__EOD__');
	# IyBTcGljYTo6QXBwOjpNYWluCl9fU1BJQ0FfSEVBREVSX18KIyA9PT09PT09PT09PT09PT09
	# PT09PT09PT09PT09PT09PT09PT09PQpwYWNrYWdlIFNwaWNhOjpBcHA6Ok1haW47CnVzZSBz
	# dHJpY3Q7CnVzZSBTcGljYTo6QXBwOwp1c2UgYmFzZSBxdyggU3BpY2E6OkFwcCApOwp1c2Ug
	# dmFycyBxdyggJFZFUlNJT04gKTsKJFZFUlNJT04gPSAnMC4wMCc7CiMgMC4wMCBbX19HRU5F
	# UkFURURfREFURV9fXSBnZW5lcmF0ZWQKIyAtLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0t
	# LS0tLS0tLS0tLQojIC8vIGRlZmluZXMgLyBzdGF0aWMgdmFyaWFibGVzCnN1YiBURU1QTEFU
	# RSAoKXsgJ21haW4uaHRtbCcgfQojIC0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0t
	# LS0tLS0tCiMgLy8gb3ZlcnJpZGUgYmFzZSBjbGFzcyAtIFNwaWNhOjpPYmplY3QKc3ViIGlu
	# aXRpYWxpemUKewoJbXkgJHNlbGYgPSBzaGlmdDsKCW15ICVwYXJhbSA9ICgKCQkjIFhYWCBk
	# ZWNsYXJlIHlvdXIgbWVtYmVyIHZhcmlhYmxlcyBYWFgKCQlAXwoJKTsKCSRzZWxmLT5TVVBF
	# Ujo6aW5pdGlhbGl6ZSglcGFyYW0pOwoJcmV0dXJuICRzZWxmLT5faW5pdGlhbGl6ZTsKfQoj
	# IC0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tCiMgLy8gb3ZlcnJpZGUg
	# YmFzZSBjbGFzcyAtIFNwaWNhOjpBcHAKc3ViIHJ1bgp7CglteSAkc2VsZiA9IHNoaWZ0OwoJ
	# bXkgJGNtcyA9IFNwaWNhOjpUZW1wbGF0ZU1hbmFnZXItPm5ldygkc2VsZi0+bG9hZF90ZW1w
	# bGF0ZSgnZmlsZSc9PlRFTVBMQVRFKSk7CgkjIFhYWCBpbXBsZW1lbnQgeW91ciBjb2RlIFhY
	# WAoJJHNlbGYtPmNvbW1vbl9wYXJ0cygkY21zKTsKCSRzZWxmLT5vdXRwdXQoJGNtcy0+b3V0
	# cHV0KTsKfQpzdWIgY29tbW9uX3BhcnRzCnsKCW15ICRzZWxmID0gc2hpZnQ7CglteSAkY21z
	# ICA9IHNoaWZ0OwoJJHNlbGYtPlNVUEVSOjpjb21tb25fcGFydHMoJGNtcyk7CgkjIFhYWCBp
	# bXBsZW1lbnQgeW91ciBjb2RlIGZvciBjb21tb24gdGVtcGxhdGUgcGFydHMgWFhYCn0KIyAt
	# LS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLQojIC8vIHB1YmxpYyBmdW5j
	# dGlvbnMKIyBYWFggaW1wbGVtZW50IHlvdXIgY29kZSBYWFgKIyAtLS0tLS0tLS0tLS0tLS0t
	# LS0tLS0tLS0tLS0tLS0tLS0tLS0tLQojIC8vIHByaXZhdGUgZnVuY3Rpb25zCnN1YiBfaW5p
	# dGlhbGl6ZQp7CglteSAkc2VsZiA9IHNoaWZ0OwoJIyBYWFggaW1wbGVtZW50IHlvdXIgY29k
	# ZSBmb3IgaW5pdGlhbGl6YXRpb24gIFhYWAoJcmV0dXJuICRzZWxmOwp9CjE7Cl9fRU5EX18K
__EOD__
}
sub data_base
{
	my $class = shift;
	return $class->decode(<<'__EOD__');
	# IyBTcGljYTo6RGF0YTo6X19EQVRBX0NMQVNTX18KX19TUElDQV9IRUFERVJfXwojID09PT09
	# PT09PT09PT09PT09PT09PT09PT09PT09PT09PT09PT09CnBhY2thZ2UgU3BpY2E6OkRhdGE6
	# Ol9fREFUQV9DTEFTU19fOwp1c2Ugc3RyaWN0Owp1c2UgU3BpY2E6OkRhdGE6Ok9iamVjdDsK
	# dXNlIGJhc2UgcXcoIFNwaWNhOjpEYXRhOjpPYmplY3QgKTsKdXNlIHZhcnMgcXcoICRWRVJT
	# SU9OICk7CiRWRVJTSU9OID0gJzAuMDAnOwojIDAuMDAgW19fR0VORVJBVEVEX0RBVEVfX10g
	# Z2VuZXJhdGVkCiMgLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0KIyAv
	# LyBvdmVycmlkZSBiYXNlIGNsYXNzCnN1YiB0YWJsZQp7CglteSAkc2VsZiA9IHNoaWZ0OwoJ
	# cmV0dXJuICgKX19EQVRBX1NUUlVDVFVSRV9fCgkpOwp9CiMgLS0tLS0tLS0tLS0tLS0tLS0t
	# LS0tLS0tLS0tLS0tLS0tLS0tLS0KIyAvLyBwdWJsaWMgZnVuY3Rpb25zCiMgWFhYIGltcGxl
	# bWVudCB5b3VyIGNvZGUgWFhYCiMgLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0tLS0t
	# LS0tLS0KIyAvLyBwcml2YXRlIGZ1bmN0aW9ucwojIFhYWCBpbXBsZW1lbnQgeW91ciBjb2Rl
	# IFhYWAoxOwpfX0VORF9fCg==
__EOD__
}
sub template_base
{
	my $class = shift;
	return $class->decode(<<'__EOD__');
	# PD94bWwgdmVyc2lvbj0iMS4wIiBlbmNvZGluZz0ie3NpdGVfZW5jb2Rpbmd9Ij8+CjwhRE9D
	# VFlQRSBodG1sIFBVQkxJQyAiLS8vVzNDLy9EVEQgWEhUTUwgMS4xLy9FTiIgImh0dHA6Ly93
	# d3cudzMub3JnL1RSL3hodG1sMTEvRFREL3hodG1sMTEuZHRkIj4KPGh0bWwgeG1sbnM9Imh0
	# dHA6Ly93d3cudzMub3JnLzE5OTkveGh0bWwiIHhtbDpsYW5nPSJ7c2l0ZV9sYW5nfSI+Cjxo
	# ZWFkPgo8bWV0YSBodHRwLWVxdWl2PSJDb250ZW50LVR5cGUiIGNvbnRlbnQ9InRleHQvaHRt
	# bDsgY2hhcnNldD17c2l0ZV9lbmNvZGluZ30iIC8+CjxtZXRhIG5hbWU9ImdlbmVyYXRvciIg
	# Y29udGVudD0ie3Byb2R1Y3RfbmFtZX0iIC8+CjxsaW5rIHJlbD0ic3R5bGVzaGVldCIgdHlw
	# ZT0idGV4dC9jc3MiIGhyZWY9IntzaXRlX2Nzc30iIHRpdGxlPSJkZWZhdWx0IiBtZWRpYT0i
	# c2NyZWVuLHR2IiAvPgo8dGl0bGU+e3NpdGVfdGl0bGV9PC90aXRsZT4KPC9oZWFkPgo8Ym9k
	# eT4KPGgxPntzaXRlX3RpdGxlfTwvaDE+CjwhLS0gWFhYIHdyaXRlIHlvdXIgbWFya3VwIFhY
	# WCAtLT4KPGFkZHJlc3M+UG93ZXJlZCBieSA8YSBocmVmPSJ7cHJvZHVjdF93ZWJwYWdlfSI+
	# e3Byb2R1Y3RfbmFtZX08L2E+IHtwcm9kdWN0X3ZlcnNpb259PC9hZGRyZXNzPgo8L2JvZHk+
	# CjwvaHRtbD4KPCEtLQpfX1NQSUNBX0hFQURFUl9fLS0+Cg==
__EOD__
}
sub style_base
{
	my $class = shift;
	return $class->decode(<<'__EOD__');
	# LyogPT0gd3JpdHRlbiBieSBUYWt1eWEgT3RhbmkgPHRha3V5YS5vdGFuaUBnbWFpbC5jb20+
	# ID09PSAqLwovKiA9PSBDb3B5cmlnaHQgKEMpIDIwMDQgU2ltcGxlQm94ZXMvU2VyZW5kaXBp
	# dHlOWiBMdGQuID09ICovCgovKiBbcmVzZXQgZWxlbWVudHNdICovCiogewoJbWFyZ2luOiAw
	# cHg7CglwYWRkaW5nOiAwcHg7Cn0KYm9keSB7CgliYWNrZ3JvdW5kLXJlcGVhdDogcmVwZWF0
	# LXk7CgliYWNrZ3JvdW5kLXBvc2l0aW9uOiAwJSAwJTsKfQphIHsKCXRleHQtZGVjb3JhdGlv
	# bjogdW5kZXJsaW5lOwp9CmltZyB7Cglib3JkZXItd2lkdGg6IDBweDsKfQpoMSxoMixoMyxo
	# NCxoNSxoNiB7CgltYXJnaW4tYm90dG9tOiAxZW07Cglmb250LXdlaWdodDogYm9sZDsKCWZv
	# bnQtc3R5bGU6IG5vcm1hbDsKfQpwLGJsb2NrcXVvdGUscHJlLHVsLGRsLG9sLHRhYmxlLGhy
	# IHsKCW1hcmdpbi1ib3R0b206IDFlbTsKCWxpbmUtaGVpZ2h0OiAxLjRlbTsKfQp1bCxvbCxk
	# ZCB7CgltYXJnaW4tbGVmdDogMmVtOwp9CnN0cm9uZyB7Cglmb250LXdlaWdodDogYm9sZDsK
	# fQovKiBbY29tbW9uIHBhcnRzXSAqLwpwLHByZSx1bCxkbCxvbCx0YWJsZSB7CglsaW5lLWhl
	# aWdodDogMS40ZW07Cglmb250LXNpemU6IDEwMCU7Cn0KYmxvY2txdW90ZSB7CglwYWRkaW5n
	# OiAwcHggMC41ZW07CglsaW5lLWhlaWdodDogMS40ZW07Cglmb250LXNpemU6IDEwMCU7Cn0K
	# aDEsaDIsaDMsaDQsaDUsaDYgewoJZm9udC1zaXplOiAxMTAlOwoJbGluZS1oZWlnaHQ6IDEu
	# NGVtOwp9CmgxIHsKCWRpc3BsYXk6IG5vbmU7Cglmb250LXNpemU6IDEzMCU7Cn0KaDIgewoJ
	# Zm9udC1zaXplOiAxMjAlOwp9CmFkZHJlc3MgewoJbWFyZ2luOiAwcHggMWVtIDFlbSAxZW07
	# CglwYWRkaW5nOiAwcHggMHB4IDQ1cHggMHB4OwoJdGV4dC1hbGlnbjogcmlnaHQ7Cglmb250
	# LXNpemU6IDkwJTsKCWZvbnQtc3R5bGU6IG5vcm1hbDsKCWJhY2tncm91bmQtcG9zaXRpb246
	# IDEwMCUgMTAwJTsKCWJhY2tncm91bmQtcmVwZWF0OiBuby1yZXBlYXQ7Cn0KLyogW2luZGl2
	# aWR1YWwgcGFydHNdICovCmgyLmVycm9yIHsKCW1hcmdpbjogMWVtOwoJcGFkZGluZzogMC4y
	# ZW07Cn0KcC5tc2cgewoJbWFyZ2luOiAxZW07CglwYWRkaW5nOiAwLjVlbTsKCXdoaXRlLXNw
	# YWNlOiBwcmU7Cn0KLyogW2ZvbnRzXSAqLwpib2R5LGxpLGR0LGRkLHAsdGQsdGgsaDEsaDIs
	# aDMsaDQsaDUsaDYgewoJLyogZXhjZXB0IE1hY0lFIFwqLwoJZm9udC1mYW1pbHk6ICJMdWNp
	# ZGEgR3JhbmRlIiwgc2Fucy1zZXJpZiFpbXBvcnRhbnQ7CgkvKiBlbmQgZm9yIE1hY0lFICov
	# Cn0KaDEsaDIsaDMsaDQsaDUsaDYsc3Ryb25nLHRhYmxlIHRoIHsKCWZvbnQtZmFtaWx5OiAi
	# SGlyYWdpbm8gS2FrdSBHb3RoaWMgU3RkIixzYW5zLXNlcmlmOwp9CnAubXNnIHsKCWZvbnQt
	# ZmFtaWx5OiAiQ291cmllciIsIkNvdXJpZXIgTmV3IiwiTW9uYWNvIixtb25vc3BhY2UhaW1w
	# b3J0YW50Owp9Ci8qIFtjb2xvcnNdICovCmJvZHkgewoJY29sb3I6ICMzMzQ7CgliYWNrZ3Jv
	# dW5kLWNvbG9yOiAjZWVlOwp9CmE6bGluayB7Cgljb2xvcjogIzY2YjsKfQphOmFjdGl2ZSB7
	# Cgljb2xvcjogIzU1YzsKfQphOnZpc2l0ZWQgewoJY29sb3I6ICM5NmM7Cn0KYTpob3ZlciB7
	# Cgljb2xvcjogI2U5MDsKfQpibG9ja3F1b3RlIHsKCWNvbG9yOiAjNjY5OwoJYmFja2dyb3Vu
	# ZC1jb2xvcjogI2RkZTsKfQpoMi5lcnJvciB7CgliYWNrZ3JvdW5kLWNvbG9yOiAjZGRkOwp9
	# CnAubXNnIHsKCWNvbG9yOiAjOTk5Owp9Ci8qIFtpbWFnZXNdICovCmFkZHJlc3MgewoJYmFj
	# a2dyb3VuZC1pbWFnZTogdXJsKHNwaWNhLnBuZyk7Cn0K
__EOD__
}
sub string_base
{
	my $class = shift;
	return $class->decode(<<'__EOD__');
	# RmFpbGVkIHRvIGNvbm5lY3QgZGF0YWJhc2UuCQpGYWlsZWQgdG8gbG9jay4JCkZhaWxlZCB0
	# byBvcGVuIHRoZSBmaWxlLgkKVGhlIGZpbGUgYWxyZWFkeSBleGlzdHMuCQpUaGUgZmlsZSBk
	# b2VzIG5vdCBleGlzdC4JCkZhaWxlZCB0byBjcmVhdGUgdGhlIGZpbGUuCQpJdCBpcyBhIGRp
	# cmVjdG9yeS4JClRoZSBmaWxlIGlzIG5vdCByZWFkYWJsZS4JClRoZSBmaWxlIGlzIG5vdCB3
	# cml0YWJsZS4JClRoZSBkYXRhIGlzIHVuZGVmaW5lZC4JCg==
__EOD__
}
sub javascript_base
{
	my $class = shift;
	return $class->decode(<<'__EOD__');
	# Ly8gc3BpY2EgamF2YXNjcmlwdCBsaWJyYXJpZXMgLSBzcGljYS5qcyAtLSB2ZXJzaW9uIDAu
	# MDMKLy8gPT0gd3JpdHRlbiBieSBUYWt1eWEgT3RhbmkgPHRha3V5YS5vdGFuaUBnbWFpbC5j
	# b20+ID09PQovLyA9PSBDb3B5cmlnaHQgKEMpIDIwMDYgU2ltcGxlQm94ZXMvU2VyZW5kaXBp
	# dHlOWiBMdGQuID09CgovLyA9PT0gYXJyYXkgPT09CmlmICghQXJyYXkucHJvdG90eXBlLnBv
	# cCkKewoJQXJyYXkucHJvdG90eXBlLnBvcCA9IGZ1bmN0aW9uKCkKCXsKCQlpZiAoIXRoaXMu
	# bGVuZ3RoKSByZXR1cm4gbnVsbDsKCQl2YXIgbGFzdCA9IHRoaXNbdGhpcy5sZW5ndGggLSAx
	# XTsKCQktLXRoaXMubGVuZ3RoOwoJCXJldHVybiBsYXN0OwoJfQp9CmlmICghQXJyYXkucHJv
	# dG90eXBlLnB1c2gpCnsKCUFycmF5LnByb3RvdHlwZS5wdXNoID0gZnVuY3Rpb24oKQoJewoJ
	# CWZvciAodmFyIGk9MCxuPWFyZ3VtZW50cy5sZW5ndGg7aTxuO2krKykKCQkJdGhpc1t0aGlz
	# Lmxlbmd0aF0gPSBhcmd1bWVudHNbaV07CgkJcmV0dXJuIHRoaXMubGVuZ3RoOwoJfQp9Cmlm
	# ICghQXJyYXkucHJvdG90eXBlLmluZGV4T2YpCnsKCUFycmF5LnByb3RvdHlwZS5pbmRleE9m
	# ID0gZnVuY3Rpb24odmFsdWUsaWR4KQoJewoJCWlmICh0eXBlb2YoaWR4KSAhPSAnbnVtYmVy
	# JykgaWR4ID0gMDsKCQllbHNlIGlmIChpZHggPCAwKSBpZHggPSB0aGlzLmxlbmd0aCArIGlk
	# eDsKCQlmb3IgKHZhciBpPWlkeCxuPXRoaXMubGVuZ3RoO2k8bjtpKyspCgkJCWlmICh0aGlz
	# W2ldID09PSB2YWx1ZSkgcmV0dXJuIGk7CgkJcmV0dXJuIC0xOwoJfQp9Ci8vID09PSBicm93
	# c2VyID09PQpmdW5jdGlvbiBCcm93c2VyKCkKewoJdGhpcy5uYW1lID0gbmF2aWdhdG9yLnVz
	# ZXJBZ2VudDsKCXRoaXMuaXNXaW5JRSA9IHRoaXMuaXNNYWNJRSA9IGZhbHNlOwoJdGhpcy5p
	# c0dlY2tvICA9IHRoaXMubmFtZS5tYXRjaCgvR2Vja29cLy8pOwoJdGhpcy5pc1NhZmFyaSA9
	# IHRoaXMubmFtZS5tYXRjaCgvQXBwbGVXZWJLaXQvKTsKCXRoaXMuaXNLSFRNTCAgPSB0aGlz
	# LmlzU2FmYXJpIHx8IG5hdmlnYXRvci5hcHBWZXJzaW9uLm1hdGNoKC9Lb25xdWVyb3J8S0hU
	# TUwvKTsKCXRoaXMuaXNPcGVyYSAgPSB3aW5kb3cub3BlcmE7CglpZiAoZG9jdW1lbnQuYWxs
	# ICYmICF0aGlzLmlzR2Vja28gJiYgIXRoaXMuaXNTYWZhcmkgJiYgIXRoaXMuaXNPcGVyYSkK
	# CXsKCQl0aGlzLmlzV2luSUUgPSB0aGlzLm5hbWUubWF0Y2goL1dpbi8pOwoJCXRoaXMuaXNN
	# YWNJRSA9IHRoaXMubmFtZS5tYXRjaCgvTWFjLyk7Cgl9Cn0KdmFyIEJyb3dzZXIgPSBuZXcg
	# QnJvd3NlcigpOwovLyA9PT0gZXZlbnQgPT09CmlmICghd2luZG93LkV2ZW50KSB2YXIgRXZl
	# bnQgPSBuZXcgT2JqZWN0OwpFdmVudCA9IHsKCWNhY2hlIDogZmFsc2UsCglnZXRFdmVudCA6
	# IGZ1bmN0aW9uKGV2bnQpCgl7CgkJcmV0dXJuIChldm50KSA/IGV2bnQgOiAoKHdpbmRvdy5l
	# dmVudCkgPyB3aW5kb3cuZXZlbnQgOiBudWxsKTsKCX0sCglnZXRLZXkgOiBmdW5jdGlvbihl
	# dm50KQoJewoJCWV2bnQgPSB0aGlzLmdldEV2ZW50KGV2bnQpOwoJCXJldHVybiAoZXZudC53
	# aGljaCkgPyBldm50LndoaWNoIDogZXZudC5rZXlDb2RlOwoJfSwKCXJlZ2lzdGVyIDogZnVu
	# Y3Rpb24ob2JqZWN0LCB0eXBlLCBoYW5kbGVyKQoJewoJCWlmICh0eXBlID09ICdrZXlwcmVz
	# cycgJiYgKEJyb3dzZXIuaXNLSFRNTCB8fCBvYmplY3QuYXR0YWNoRXZlbnQpKSB0eXBlID0g
	# J2tleWRvd24nOwoJCWlmICghdGhpcy5jYWNoZSkgdGhpcy5jYWNoZSA9IFtdOwoJCWlmIChv
	# YmplY3QuYWRkRXZlbnRMaXN0ZW5lcikKCQl7CgkJCXRoaXMuY2FjaGUucHVzaChbb2JqZWN0
	# LHR5cGUsaGFuZGxlcl0pOwoJCQlvYmplY3QuYWRkRXZlbnRMaXN0ZW5lcih0eXBlLCBoYW5k
	# bGVyLCBmYWxzZSk7CgkJfQoJCWVsc2UgaWYgKG9iamVjdC5hdHRhY2hFdmVudCkKCQl7CgkJ
	# CXRoaXMuY2FjaGUucHVzaChbb2JqZWN0LHR5cGUsaGFuZGxlcl0pOwoJCQlvYmplY3QuYXR0
	# YWNoRXZlbnQoWydvbicsdHlwZV0uam9pbignJyksaGFuZGxlcik7CgkJfQoJCWVsc2UKCQl7
	# CgkJCW9iamVjdFtbJ29uJyx0eXBlXS5qb2luKCcnKV0gPSBoYW5kbGVyOwoJCX0KCX0sCglk
	# ZXJlZ2lzdGVyIDogZnVuY3Rpb24ob2JqZWN0LCB0eXBlLCBoYW5kbGVyKQoJewoJCWlmICh0
	# eXBlID09ICdrZXlwcmVzcycgJiYgKEJyb3dzZXIuaXNLSFRNTCB8fCBvYmplY3QuYXR0YWNo
	# RXZlbnQpKSB0eXBlID0gJ2tleWRvd24nOwoJCWlmIChvYmplY3QucmVtb3ZlRXZlbnRMaXN0
	# ZW5lcikKCQkJb2JqZWN0LnJlbW92ZUV2ZW50TGlzdGVuZXIodHlwZSwgaGFuZGxlciwgZmFs
	# c2UpOwoJCWVsc2UgaWYgKG9iamVjdC5kZXRhY2hFdmVudCkKCQkJb2JqZWN0LmRldGFjaEV2
	# ZW50KFsnb24nLHR5cGVdLmpvaW4oJycpLCBoYW5kbGVyKTsKCQllbHNlCgkJCW9iamVjdFtb
	# J29uJyx0eXBlXS5qb2luKCcnKV0gPSBudWxsOwoJfSwKCWRlcmVnaXN0ZXJBbGwgOiBmdW5j
	# dGlvbigpCgl7CgkJaWYgKCFFdmVudC5jYWNoZSkgcmV0dXJuCgkJZm9yICh2YXIgaT0wLG49
	# RXZlbnQuY2FjaGUubGVuZ3RoO2k8bjtpKyspCgkJewoJCQlFdmVudC5kZXJlZ2lzdGVyKEV2
	# ZW50LmNhY2hlW2ldKTsKCQkJRXZlbnQuY2FjaGVbaV1bMF0gPSBudWxsOwoJCX0KCQlFdmVu
	# dC5jYWNoZSA9IGZhbHNlOwoJfQp9OwpFdmVudC5yZWdpc3Rlcih3aW5kb3csICd1bmxvYWQn
	# LCBFdmVudC5kZXJlZ2lzdGVyQWxsKTsKLy8gPT09IGRvbSA9PT0KZG9jdW1lbnQuZ2V0RWxl
	# bWV0c0J5Q2xhc3NOYW1lID0gZnVuY3Rpb24obmFtZSx0YXJnZXQpCnsKCXZhciByZXN1bHQg
	# PSBbXTsKCXZhciBvYmplY3QgID0gbnVsbDsKCXZhciBzZWFyY2ggPSBuZXcgUmVnRXhwKFsn
	# KF58XFxzKScsbmFtZSwnKFxcc3wkKSddLmpvaW4oJycpKTsKCWlmICh0YXJnZXQgJiYgdGFy
	# Z2V0LmdldEVsZW1lbnRzQnlUYWdOYW1lKQoJCW9iamVjdCA9IHRhcmdldC5nZXRFbGVtZW50
	# c0J5VGFnTmFtZSgnKicpOwoJaWYgKCFvYmplY3QpCgkJb2JqZWN0ID0gZG9jdW1lbnQuZ2V0
	# RWxlbWVudHNCeVRhZ05hbWUgPyBkb2N1bWVudC5nZXRFbGVtZW50c0J5VGFnTmFtZSgnKicp
	# IDogZG9jdW1lbnQuYWxsOwoJZm9yICh2YXIgaT0wLG49b2JqZWN0Lmxlbmd0aDtpPG47aSsr
	# KQoJewoJCXZhciBjaGVjayA9IG9iamVjdFtpXS5nZXRBdHRyaWJ1dGUoJ2NsYXNzJykgfHwg
	# b2JqZWN0W2ldLmNsYXNzTmFtZTsKCQlpZiAoY2hlY2subWF0Y2goc2VhcmNoKSkgcmVzdWx0
	# LnB1c2gob2JqZWN0W2ldKTsKCX0KCXJldHVybiByZXN1bHQ7Cn0KLy8gPT09IGxpYnJhcnkg
	# PT09CmZ1bmN0aW9uIExpYnJhcnkoKQp7Cgl0aGlzLl9wYXRoID0gJyc7Cgl0aGlzLl9jYWNo
	# ZSA9IFtdOwoJdGhpcy5sYW5nID0gJyc7Cgl0aGlzLmJhc2UgPSAnJzsKCXJldHVybiB0aGlz
	# Ll9pbml0KCk7Cn0KTGlicmFyeS5wcm90b3R5cGUgPSB7CglfaW5pdCA6IGZ1bmN0aW9uKCkK
	# CXsKCQl2YXIgY2hlY2tlciA9IGRvY3VtZW50LmdldEVsZW1lbnRzQnlOYW1lKCdYLVJlc291
	# cmNlLURpcicpWzBdOwoJCWlmIChjaGVja2VyKQoJCXsKCQkJdGhpcy5iYXNlID0gY2hlY2tl
	# ci5nZXRBdHRyaWJ1dGUoJ2NvbnRlbnQnKTsKCQkJdGhpcy5fcGF0aCA9IFt0aGlzLmJhc2Us
	# J2pzLyddLmpvaW4oJycpOwoJCX0KCQlyZXR1cm4gdGhpczsKCX0sCglfaW5pdExhbmcgOiBm
	# dW5jdGlvbigpCgl7CgkJdmFyIGh0bWwgPSBkb2N1bWVudC5nZXRFbGVtZW50c0J5VGFnTmFt
	# ZSgnaHRtbCcpWzBdOwoJCWlmICghaHRtbCkgcmV0dXJuOwoJCXRoaXMubGFuZyA9IGh0bWwu
	# Z2V0QXR0cmlidXRlKCd4bWw6bGFuZycpIHx8IGh0bWwuZ2V0QXR0cmlidXRlKCdsYW5nJyk7
	# Cgl9LAoJcmVxdWlyZSA6IGZ1bmN0aW9uKGxpYnMpCgl7CgkJdmFyIHByZSAgPSAnXG48c2Ny
	# aXB0IHR5cGU9InRleHQvamF2YXNjcmlwdCIgc3JjPSInOwoJCXZhciBwb3N0ID0gJy5qcyI+
	# PC9zY3JpcHQ+JzsKCQlmb3IgKHZhciBpPTAsbj1saWJzLmxlbmd0aDtpPG47aSsrKQoJCXsK
	# CQkJaWYgKHRoaXMuX2NhY2hlLmluZGV4T2YobGlic1tpXSkgPiAtMSkgY29udGludWU7CgkJ
	# CWRvY3VtZW50LndyaXRlKFtwcmUsdGhpcy5fcGF0aCxsaWJzW2ldLHBvc3RdLmpvaW4oJycp
	# KTsKCQkJdGhpcy5fY2FjaGUucHVzaChsaWJzW2ldKTsKCQl9Cgl9LAoJcGF0aCA6IGZ1bmN0
	# aW9uKHBhdGgpCgl7CgkJaWYgKCFwYXRoKSB0aGlzLl9wYXRoID0gJyc7CgkJZWxzZQoJCXsK
	# CQkJaWYgKCFwYXRoLm1hdGNoKC9cLyQvKSkgcGF0aCA9IFtwYXRoLCcvJ10uam9pbignJyk7
	# CgkJCXRoaXMuX3BhdGggPSBwYXRoOwoJCX0KCX0KfQp2YXIgTGlicmFyeSA9IG5ldyBMaWJy
	# YXJ5KCk7CkV2ZW50LnJlZ2lzdGVyKHdpbmRvdywnbG9hZCcsZnVuY3Rpb24oKSB7IExpYnJh
	# cnkuX2luaXRMYW5nKCkgfSk7Cg==
__EOD__
}
sub logo_base
{
	my $class = shift;
	my $data = $class->output(<<'__EOD__');
	# iVBORw0KGgoAAAANSUhEUgAAAMgAAAAoCAIAAAA0fiJLAAAAA3NCSVQICAjb4U/gAAAACXBI
	# WXMAAAsSAAALEgHS3X78AAAAIHRFWHRTb2Z0d2FyZQBNYWNyb21lZGlhIEZpcmV3b3JrcyBN
	# WLuRKiQAAAAUdEVYdENyZWF0aW9uIFRpbWUAMDYuMS41ufsxJgAAL8tJREFUeJyMvPuzpddx
	# HbZW997fedw7M3dmgBkAJN4vkRQfelCKKZXkslNO/oGUk6r8cS6lnErZpViJUpScUkxTlChK
	# piXaFE08+ABAEMRgAMx77j3ft7tXfujvDkAlqcopFObMmXO+1+7dvXr16ubDBw8lCYJQbwgK
	# AED8/3oJAshf/qQOwv+Pb/6//hNBAP/PU0uqj+oLkEDWh+T6k18++yevf3gLj766HnQ9Qn2w
	# vpfw6A3+wb2xfkn+w8v/5Q/Pb3S9cn7qhL900vPT/cPjr+8+9QNIn/7s0TsSAH/p6L98FtUX
	# 9cnPzq+Lv/yoPv1D/NLhPvX38zX61MU8+rMuhaw3DdAPvvf3Z4dDjJjHnKlUZiYENyOlzKzv
	# g5HpZgAy0sisOyKMRkCZBAmmgUAmzCxz0CwzzCwkCU7jal2ow8tIGjLNLDIfXa3RJQVkZpSU
	# amYQEkrJYCQCCcCMEUGYex2BwHocAhnp7vVQcv2+KbOWw8wzU5Kbi4BEI1JjjGZGX404QuZW
	# lwUgxkwjRdIFZaq5ZQqAlBJaa2VPFDITbpIEullGmgEpGkBkZu3CEMyMYGbSrSycYEAEjJ4j
	# zGBmqUzJyMwsw1QmSbcmSJBCq9VamRLMLM9NIDNNSCUAd689GiESUrpZJIwkkmKZkRkBICEg
	# IUlGZKY1j0wjjezTZrvZTL1/6Te/3FI6jOX04cPDYR6HeVlGoI4Qbk4gI8xFWIxUytxpFspP
	# th0JyYSMFGQkSaPRMEaIEHwgynRaayPC3ADkEk46LXIxswW1/AaJJFMBlf8gqUwDBRpYVwgz
	# SGXbQkhy0mhDmZndHADcxhi5jGmaMkMSwNZaIGOE093cDJkJQKKYZX3NbB6LmVNBI2GZghGE
	# CagnAIBcDou3RpobUymJNGXWQhKUkJGteUYODApm3txTY707cIkw94wo70IzGVMZy/DWYow+
	# TUyV5/B1i4oAaWnIEcrsrUuKCHdflsXczMzdU4oxzEyQUiDdLKB6qjTLTAIpmVlK7j7PBzeb
	# mhssMiMlwgAzQwIQYVTtUS0Zbq2573fSEHaS1CCMeZkP8+H07PTsbCzLMkQCVEQ4IMgJ0M+d
	# KSIjR5rRWwMQBkWSzDHQvJlb1l4BgAAiQjBAbjSzsoXDsnR3khkBwsxiidYaMgEYmJlpFOTm
	# EQOk0VLZYIeI3icDxhgwNW8RIaXTCMoI4GHEWIZ1b61n5tk8g6EhAI1NhLTei3K05gYr7wuB
	# NIoZi7mBSbhISSOivF1mJtDMjJmLrLfyeWaGWuzy0BBS5R3NXMplGTgPGb25lLUzRybdkOk0
	# CZJgDKh8bUT42UEjaETC3URkpkJT7wmNZUjZW4PZGANkxGitKSUplU43YzlUmJGMTJKZImEQ
	# SBGrL1cqluOj42UsD+4/hDnNYE0RyuHuSJGMCMCXXNxsu9vPQCxDx0e9EVKTFMtYluVwONx/
	# 8GCMMZaRmU4zI0lJyuy9jcjygA4yhRBaGxnWmiRzjGXAQBIpCmiQRFhGAmxmi0RBzWqRsjdU
	# PCJohsSyLFoCQjleaz6UjyCEkUodEodcujWzRkkZByCdBgCoPyUhmZlUHs4ONJNEk+YEaY9A
	# jfkYs5stMylExIqWzJCEhpmRIjydBEMikSPMXNIMuIOSFotIAQQzw8xBuhuMOYKACWks10pi
	# RJA2GyUYISCVMkZEo0EgKCKVggNYxtLdjTCaJDOmYRlDI9ydZDPP1NnDU3MHILd5mSebBNUW
	# NYaywhvLjw6lm1lhPKNSQKaS5Hw4e+KJ69upf/zRrYcPTq054RUfYizurfZD0NxiLAd3j2V4
	# czef5+WwLAIagIgYYxwOh2WeI1IjpAxmBgtTL/O8zL6gorgHQEGZUEYExshMMAHIrJmNWJwW
	# S0ZEMwdgoswjguXKaUFFhDIq6sHWEEayw3JJCuwtMhaNyTuABkqIZcCYjohhZljRlwozNVrB
	# 8FiS5DhbSBa+6a2NGGuIBDKz0BXcxhJupiRNAGJZrCJHSobMMWY92qmQZJkZEdGdAEQnGZHu
	# bmYRY4yRmTRzdyPnefapn68xQY4xmzWSUkREmMyMKbgvy4KEt1bRjiRhy7IYVCGbpJoVDhs5
	# YozmTRIifZgkax5jHBrH4eCtGUhhWZY1MhokRUVVcvVSka2ZMg/L4s6rV67c+ujjD27eNO89
	# esSZmZlZhMiYI81dCKNFLFNr8J6Zh+XMZ9vFFlJbIV5KI1Oal5nxS8lCwXYIVkmSch5DRndH
	# jhHD4WUQAKjMlCLTAMlEyzVNOMyzsy1j+MYDOuQMYGotUxRyGWlEytxCMhBEREhy7xEpKUZ+
	# 4t4DBmbAGscYaaTgjSscOA8iRstMWuG/ZGLOkNRW2CtJkRjLyNYrPSkTASiZJPM1e8lIKM1M
	# 0lhGxbh5GYDRQVhEjBG9tcxkysExgkJIY0RWDB3RWyunLghEQpGJhFiJkxTrSi/L0loTEBgA
	# ksxc9x4HM1PNyiyWMVT3C4wIT41lcXgIlso12AlU5BhDAKy1zIxC5DGQyhhQHg4PP/+5l08u
	# bN7+yQc5ovdNRsYImWTKSuopGnLR0EBqRNiGG9+YYOeZo5XHigiQrTWSquhOLBkkMxPGAkb1
	# Ilmf1F0hkoUkJBMQSTeVmQkBDeUhRzqzGTd9ZCRUPrxinJLG1kAKsYx5ngMKaJ7nzBxjhJSZ
	# C3IoBwVrkmitktvMSkOhoXketTXrlZle0UGa5/lRVh6FhKCICCWMy3KIWJbIkSI9c/Vq5STq
	# UCNiRIQGTNacXuntip1JlxiZY4x6dHUEAdM0ASsSJTnGMLOIZVkOS664zUCKFOlWv1pxiFQ/
	# Cch6I9lacxqF3bT93IuvlEOS9Nknnzq5dImkaKKNMVprIqNwvjGhyupIOUWFIigBaY10W2J4
	# i1/9/Mudp4f7H+22dAIG7w3GJUbEAiSRGYsBTrbW6jGuZALWdN9QqWS5RIGplYqosEzImMQc
	# oxY7eZ6WzIOpyVszX9dVSiIg0jLPTRAiATNrbVmWZVkoIGWAATmCFdwjONITjTZ5o6+5cdlN
	# XZK7wwgjXMlM5jwOq7kwI5fM0btba0uEiIIVZ/NQcurbwowqgmMoE05rrdXnZYjN3Gl1dhEi
	# JBTmjQgkDDaSsJ4FE1FbDpKcZiCMLCMxCojMzJznOce6vrWHU1Kzcjllc5AAI73+L5Hw2upM
	# WeHa8wdSBtd7v3B8jEhEOrjf7adpIpnMtmnunpnLskTEUM4x6kka6LQVB6PiS2ammS3L4Ynr
	# jz3zmWvL6T3mbJljjMpV8xN2xuqvZOWgUb9dkxuy7LxC4foUFXC2gahcopiYOkprbU3zFI0G
	# IySYFWggWclqJCJkBsocMjBiCGjNldmcCDZ3Cocxt9YSKam1FsugeWYYXWQIkPXeTWhmOUb3
	# hvN1qr3ojcsQGx2rT5KbjFl0Gc3IuoV6AqQDdGRvHanzUIgYY1nCQFs3GoaGKHp3wAxjWZCk
	# zGnO5kLMAWTvPbMIApFMjctXTo4vXoAwTdOHH354++4dE05OTq5evbrM84cf3ITbk0888aMf
	# /3iapuefefrNn/z45Pji9WvX3njjRxcuXnr8+uOZ+d4v3n94/8EzzzwNQcob73/w9DNPtT7d
	# f3D/xo0bdAfw9Gefnno/nQ8AAjLSzAjsd/sXnnte0i9uvH96WF558cV333vv/oMHT11/IqWP
	# PvwwllHQRJmQKh/iylFJkc8///zx8fGyLJvN5sEZCMLN4EL2PkmiLIZIt84c4eZWnNRm03sH
	# WEerIM2pt8LBUDRxoq//pzEVSrqlBpEWQmStcUBwWx1sZKN1qWW6cuPGYA4172ZOGoXBRMcc
	# M0yEx1BmMDNzZA6jpu6ZI3IxQdIyEtYyMPXtnFoEurfWCkePOAAKU12Au/cyZsD6VAk5mIYU
	# ucQn+xIA3WA8O8wJmdl2O7Wpe290C6XJTBaIRESEmYlBV1oGo7tteqP3EAMIwJrLKON2t7ty
	# +crDBw9u3f7oueeeOzra74+Pnn322dt37kbmCy+/tMyH3X5/+fLlxx9//MLR8dWLJ1cuX87M
	# k5OLzz73zM2bH9y/f++Vl15052633R/tbt36+PkXnwPw/vu/ODk5uX79OoWXXnhxs5nev/nB
	# brMFMHlzWqXDu93u9u3b8zK/8tLLQoC69vjV7rz22OPj7JAjeM6bROUWdk5WuyV0mE+fffJx
	# 18FSCHozmia0jkYZkgSIZIxNZxK26WrwjXtrRksUQS5UVngeyNesPiPMrPLeoiWbiJSh8q+k
	# t4gAwcixjCTc3YRYhpkZaWAzXzydlgRByiLCrQHFB6rQj5lbcStOFU/d3CQZrRxfxLZ1Cp3I
	# DPNGs5XLFosiVWHquo9czBuVUsFvovJVwMwiouirCtm26TKbI4xWt1/op77JoWY+YhBo3mm1
	# BQLNISUTQIxReVaOIJHK+/fvf/zxx2x2cnLvwvEFgPfv3//w5gfN7bHHHtvudrdu3To5OTk+
	# Pj47O7t8+fLR0dHbb7+93+/d/YXnX6gQcXThGMD9+/fvPXxwfHwM4OTKFQBHR0ehPD4+fu1H
	# bz48fZg38tXjl8v1xrJI+uijj27fvn3r1q1rj1+7cHx844MPnn/m2fHEEhm3bt8u8kmimXtv
	# Y14kOtsSM5m9T9s+Xbt2bSyjNWvdxsNBn8JyLGdolpISEDabDYUco7IQuMMshfnsMLZbCizD
	# SimQAWSds8jJWqbMQtkmOExSFLtFVlgtPCYJvtY+aAZgibEsS+99QCRNZcXMSLixoIRZ1XQe
	# GXcZuCQqSMikSACZarShT8J8ZjLR6AEkQCGLDyMjFjNrZgApmJlMzQyB1hppK8YEDDSQrUUE
	# zcqyzy3ejFaOsBA86WZYliVFSQk1c5kVMDUzZSi12+3Me598v9vfvnMH0m732Gaajo72AE4f
	# ni7LePXVV+d5fuudd37llVci4vatW8WC/pc3XyO5n7Z379558voTxePPy3z79p13f/7u5ZPL
	# d+7ecdo8z9euXP3p2/dOLp2s4K+KV+TxhQs3bn5wcnIZwNnpPC9LRFy/dv2nb791Op8Zzd2L
	# m1gxUz1tOmXeLHPkyNbaNLU+WTvTKFw5WYEpZwulYqVJrbi6TOXS0dwbsCLBBoDGKnHBaLRE
	# ABDhrZVBcMlYBgF3dzCXwap86dxopFlLWZs3X5alw1qbzJwxw5nMyhiBpJF1SxFARkRrzehA
	# 1aeqBE5BGkFyKMsoC8mhWLxUa0W4CxIcJJNIqbWm8xJ1eZQxFgAOLyuRxJAJhLzy+CjukN56
	# RGiEMrXxKo9UZV6ZETlN27AVzDrgVYdJNXdrvRjtV159aerThx/dvHnzg2ZtM20/9/nPR8Tb
	# b799GMuSmuf5zr27yzIenp4ezg609vHtW/ujoy/+yhci4qOPP9rutpvNpsD+66+9/sqrr157
	# /PH79+8fTk/PDoe33nrrueee+41f+/X7D+4D8KlL8oJD7l/8wq8CeOudny2H0diXZWw2m698
	# +St/973vvfXu29vt1t0riPTumZlR8TFyJOkPHpw+9vS1ixeOjjebD8dtb/0whsA1v3Y6mMPN
	# bDJba6CZAFJpza15sc+NQKc5YMZGq8Q+zqvQlBRZwLC4mWKQWcHi/BURJMgiSbMqUu6rH7Ki
	# uFKpLNioyMJ41jzP8+1yfaunjKg00N3LsGKsp6tVXpNaoJsX3cfzUsman9Zx6osyM+utduoo
	# WzGQthLuzV0CJAPN2xiDQi5LQZBF2TJ775UP1XZqtKr71qaMMeQu4DAffvT2T6BgquLrL977
	# +Y0b7ycySU4Nad9/7QcOc/c33vxRjgCgzLffefud9959+YUXv/D5L9QpJD3/wvM/+vGP/sN/
	# /I/dW3Mj4d7u3bv3gx/8gOSClDRNU2Yuyh+8/tqAqlBmQSPHcrhw4Xi73fbWv/KlL/3wzdeM
	# NllrrQ2lCW7mZilFRAbM2s/fe//ZL7509erlo/20b3Y6FjcE6+RaxkiBgpGqPJc0sIiFyovL
	# SZkAlfsCSKSCZta8KBMZi/wFwOYFTdQ9PxW/AloyinSwTEVsWrdmS4zQMKNJTXSaw5zOrESN
	# osGt6gqRY2QsGYOCW0TwvCDFVIE8pzXRQNIqokkqOsPBbl7ReU3gjUnAbGSawxwri+s2lEmJ
	# EjAyEikqlCMjdG437gYzGFPbPpEaY05oiVFp6RpHADb31gQkcp4Ph8OZotasm7uMMKRiYGRG
	# jpEajhXtOU2K3mzy5uRzTz/zwnPPr3WfdZPwpRdf+t3f+Z0HZw9PD2efMNdGVrgmcxkOmlnv
	# fTtNxZjkWCgc7bdmtt/ta63OHj4cMQ5jScJkY+QYOULzCNJDSfjbb/8MH987vnjpypUrFy/s
	# dq6N2643TyDVaL33oqhk9KnneVYEYBSAWokTQEogM8cnO76KypkOroDDfYyR57H5k/WTii9w
	# EOXbyLEKaFg1BEgrK2EG0mnNPIEl4hFYbq1Zb9ZXVunRq0onj6isFWMJWGvbWLOaurCkca0H
	# 1612ZzM4wNSyHB7RMOuFnYP6uvG603pDcplnSGOM8s0AwgA30pjIkRSXzCEVxWpmNz+8+d7P
	# f+ZKyiAtmXBbKxEhRjLSEzKyOal65udUkL/68iskIbzxozf+/NvfunXndl3hY1evnly6OI/l
	# UWIr41A6aJ/S50jiSM3DE1PvzXB6ePj6G6/Py3w4HL7xzW+kAFmKMTQyf4nAaw3ANE1vvfPz
	# n7z1s921z372mWdPTi71CRGnioM8rPkYI3PljR+xdO7eWgsoI0QUqdkAmKGqDFQ0lixCJo4Q
	# mAQSIjmW3PqUSkvQnOQoORL83GZNpPeuCBOXTIPDARqkZYxEGunWjDSEGUKZAsRu7gmAIxNY
	# XVSYAqJRXJc/IGLVgdFo7nlu4plZdftc6/Zr/YeCEM16GiOWoXR3RIqsoNx6BxBKMzuMZdM6
	# gFii9Y1gMsFtyViRKIRgYz235pEhLWM4zW3aNAmytkomHbBMNzchzUalUYCWhWazRGObOlIE
	# dvt9/euNmx/8qz/819N28x/+9rv/7X/937zw3PP/6n/914fTs8snV2IMW3lJkUzQ3MaynB7O
	# rDVzM5qQGKP3HoEQ3v75e99/7YdnDx8qdXzhQm995WuUTJwdTqW05jFmANupn57O3//7N154
	# 6fmnn33x5ocf33348HDnbMlIeY4lRiLH0MCAm6/08RhJdbDt3WAFURoBGzJhUksLkmkLuxJD
	# DPNOMpZw9033Zmi2ATDHAWRLZOZpHHrvFIxigoYxkm2aJi7LYmlOK8Vhs2Zmg0rCrGVmc1Nk
	# YfADZrdeXrpNpkwTuveAxkiZujsOoFGWtcNySIS7J7J4lsw0YNM6ig/RAiLT4M5MM/MQUwDN
	# mzAA4LxuXQXQJaPbSnnP84xmS4yq7RtpIp0GhlJMuGkMM2u9C4hEmlmaYilzH5FOU6mDAO8t
	# R7g1ZTZzpaQoxnKeDwAEXbp48eTCRWv9wtHxd7/3ve/+7d/NY+yPjuCN3l988ZUrVy5n5us/
	# enNelldffvmpJ5+U9PD04Y9/+pO3fvp2ROx3e5lJ4/ErV7/67G9m5nu/eO/73//7/fF+O00l
	# 9Lt04eizn/nMlctXjo+PJT188ODGBzfefffdj+/d+Zu/+/4zT13/ta987plnnvnwww8P9z88
	# lT/30quPXXtiv9/P8/Lzn//8hz/84cVLF19++WUhP/zow7d++tPmLUYABAxYs0Ij0g2EMhZK
	# GSKzyTHS3WXMZWmt5QhRZujmmUmje2dfi/bWfIklxpBBJibcz6s9NOtNMTKz4H2ppnRYzGxj
	# jeQ84FbqU1tyaWu1JEljY4kfWzNJMg1ljvDWHMwclJwtSzdGMRWp1prBSYYtQxmGKn1V5jGU
	# dHdznVdsijdhnpcYMqfeVek0GRlYgu6PNNdjDBlLBxHLgLOZp2XJN4vjkcRGVP5YMRoyZ4I2
	# WQ4VLR0aD+/df/DgwX6/3213//y/++d/8m//tLmVgyH2AHa7o1SeXLl09eoVAR/d+ujVl18l
	# VxX08f7oy1/44tNPfub/+nffeIiHO+yMuHjx4mOPXQXws5//jE6Yi2zOr3zxV5+4fn1VJksA
	# jo6OXnjhxeeff+GNN9/4D3/zV//+r/766NL+lV994VdzuXj1Vju6BnrVJTab6YUXnvvMZ556
	# 6+23rz52FcC7775778F9b323gtJS1EFWdHUGIzfWGthh9impd6V2mREKOAISInKJiIhRRU2u
	# 4ocws26ey6DO9XpZAC4fYZH6jheNJACY57mbI6LUZI9A1aMaMIwREXUtcEUmlBmAjHCaYmCE
	# IUsihcqAi1woQQRMydCadgAYGQ/OTkvistJ1ZsViVNiSNA5zzEvVMccY9aslxhp8K+lpbs1H
	# LkvMsQwKrbnZCjGXZZnnOVdlYSVJLFxVB0xIKbp97z//p4qGjz/2+G9/9bcqEZx6203Thd1R
	# awZgv92VXb/68qvguYh/ZYd15cqV3/naP7p3/97h7DAv82a7rT3w8a2PRyxCeOM/+q3ffvL6
	# Ezwv756TO+vfXn3l1S9/5Td+9oubf/Jn3/rh91/bnnz2My988dLFi70ZlVAog+Bms3nl5Zfr
	# au/ev3d6ejrGkpFKFQZuKB4hs7GY9RXCZ/GOpkAk1Fob86Fga5b4jvRmpUeMkEbQ5NaQUsKw
	# ajRba5mJzNJiAmLKaaWAE1C0U2sNGiTNCSik0DC5JKSchFmcSwZMdFjrTaVKjijcXQnskouZ
	# iwoRpZA0A9ca5QrCUqRab07jSHcfGQXjVg2CrRrL1awltzULK3XXquYAIzOZZYXeVsnGEoMp
	# 92bgdrOp0vtKfmSxbpjownBIUNCN+uDGjb/4y7/43d/5XQCvvvorVx977Bvf+IaP8G2XG4EY
	# y36/LzsC+Nff/Zv/+Ld/e3p2enLx0le+9OXf+LVfh/TUU09dvXxy9vBUOe23++of+Oj2x/NY
	# IuLLv/rFk4uXypz+8q/+8odvvPHxrY8JvPTiS1/7R1+7/vi1H/zwv3z/+3+f7O+9f+tP/uw7
	# /+yfPXa0P9psdvfu3fl3f/Ynr735JuCf+/yXfv/3/umjJov33v/FiFiFoYCTK8bqRoJm1gwz
	# s6qSnU5yHgMySgbuNpvMXCKcbFYfAkikMUuoA+eq5jMzB5ZlQWWn5BijmHF3o+DuJcQeY3Bd
	# tIbMJVJS6bWRK8U/xqBN55VSZSYFUyUaMBhoNELMzFAWt5JKbyXGJEgqlJroSCVNkSK6ObrF
	# MlLJRWxOYIyxnTYjR+UHkkbE5N7P72V1hCRLN8K1GBWSpUiWHrraTxQCCaOZr2UxsrUuJKFS
	# sruXomF6//33//wv/vz3fvf3BD125epvffWr3/nr7/i0gdIbd7vdeRThN//im3/zN3+z2+6u
	# Xr5C2vf+8/cuX778wnPPU3rqqad+8Pc/3G39aH9U8pI7d+72Np1cuvTUk0+WYuqPv/5/vP7G
	# 69Nme/nyFQAf377zv//p17v7g3v3dpvtpQsnu93u8tXPnp4t83z3/r073/jmv+2OX//1X1Pq
	# o9u3vvefvvtrv/bVIkbu3r29mbZrzl/5TRWhS41e3LKbQQEFxoKxTMZt8w3RIriEh1xooCea
	# mDkyM3LBOcslyd1b84gRhgUJJaFSaBUlQTKUmdl6t3M+U8QyFHC6qdkan0XCFuSgkhEYUrIZ
	# m1lbKSsrOQ1WVWBFn09oUl/FPwAspBElEnGz3lqjlbBktWyzJiqymUuCWxIirHmd4pFG7VFl
	# SVIx9cUeI1KfylKXjKFMZ9gqOwllaCWfE2GGrPrQKixqu+32xgcf/PtvfbOko889+9yLL770
	# 4P79VBKrYUm6fef2X33n28fHR5cvn1y5dHL15OTy1atvv/N2BdrHrj52/+H9w3w4OtoX8Xv3
	# 7l03+8wTn4Uo6Pt///3X3nx9tz+6dHL56mOPP37t8StXTi4eHe375trVK5cuHG2n5tTTTz8N
	# MCK//n/+2zd/+rOP7pzeuXP/MA/Jb3zwAc7bx+7evddojSyZn1A9NYAhu6kbJmISd61vvU/m
	# LjjECCqNotKgZjDk2k+y7s5GsuBOwY4VfJDb7ZbmpSlt5o+KLfrUC+c9CKtUl66sjDprUUuh
	# 1cSmKlmu8rdlPrhVW9LKryNDMZroCQtVXTqJRjOh+LNHfNjK7kZ8ImxcFgCl8FyWxUTmahAi
	# 5rGU7IlZeeVqPWXQJElvbVpB+ggHrXkpJkgWOozKLYsVg0pe9uiT3vp2u710fPH9X7z/nb/+
	# Tj2or/7Gb87LfHZ6Osa4eOliucyfvfuz7X5/tD+6cHxht91ups1u2tC9IGtKI+P4+Khw1O27
	# d0qKdu3xqyVl+OHrr203u0sXL126dGm/22932912d+no+OKF46Pd7tKFi95sGfOlixcrZLz1
	# zjt92oHTg1M9eChim2klLr5z527zVkXY1k0lgSiP5QAimeGQQ5ZSSW/BzEiN1s1sJeJW4tLM
	# ehNTpkfI3c5RiGKpIuCiXGKYe8GUCJEOoHtrrdmjFkeBAkxLLIzBGLSi1FkiIaY0otHcLCOw
	# JMWpdQDLcpDCHNWb2FtzyEHTKscgmVxxdwVKM1PldGUQbrX2xegjsrBCjjCsVviIOy2naJ8i
	# zFprbB7QyAwpidJCBrSU0UmSKlMxwQSL1Bh2royuaFgMb7VzXjg6+vk777z//i/K4J568onD
	# 2cOI0UoNC5zNZ83de2+ttapRSicXL5a537l3l2Zt0wCAuHPndm8N5z3JJEaM1leZkIxjGQmJ
	# zdtm2l6gbXrfA10giJSGtD862m53m77prfepP/744+Wwbt2+5e7em3evjkicUw5oTGb4eb2g
	# MSfn1DQ127Rp0zZYRoMwFhMw4Gw0WcbO+w7ECIMawjXIHGMZIK07bIK73OAGz4Cluehg6VIB
	# mXvrG1CpUBWXE+69oZtoI11ozKnR3dxNWa1izayVwrM8BEYiozdII5S0Mi064ICFmnk65U30
	# kcpqIhQo9PQW1mGbPs3IgymJtpnQDd3Ky+aI7q081hptVe06a9uMlM1ARZNvbMISHbZB8wHN
	# o9EyA1aCon7eIE13JgJukeOZZ595+tln7z24v8TS3HdH25s3PyindbTfn83LHMuFixcLxrzy
	# 4isS18zTjMaJ/ZUXX6ng9PY777jZlctX6zzv37wJb2Cb57mqFC+/+NLQECGFZThtkrUUlmEL
	# MhSx9E17ePqgmmZfefnlEoG2qZvZiPHiiy9WPnr33j2jJzkyh4bO+4BtJXbcik/q5pSqgKrI
	# +s/piqSlsIBzxAExE0M5YhnupORwJpHZaN2bkfW4zZEaleiawd2lPBeiFKEwMmR00tzbmppJ
	# koayDHBkhoYIc7q7mIlwaxBDwKMAl6tdZlYYXeHXKmiOpOTudC+n26ZuzUvynwXRiqYorxPS
	# yPKyCS6RFTFzRCxDYknjzRqW9GRhrByhHFBC6SavLFZ0byRHquqnYAIZQu/dmv/6r//myy+/
	# 8spLL7/wwgv3Htw/OxwEPP/8C4AI3LpzpwjCaZoIEDi5dOmf/P4/npf57HB27/49kr/9X/3W
	# 0dGexJ07d374xg/71HbbXWVty7xMZp187xfv1UP/6m989Yuvfn6ZD2NZzg5nZ4ezx69f+/1/
	# 8vufefbpu6d3x5iTWsby/o0bgAj+zm9/bZr62XI6L4eR43Of/8LJyeVapdu3b7Xmza2U+NXA
	# QhbdIDhkEjOaRGuHDMuyuPL/BVZIqhhzkxk5zGCVLg+ffIwwOLmmQcsyu3sZUIHr1toYowAV
	# UFLxyMiUSM+118DMbJnnkjZkZtXWcB4ujFQGzdYeWlprDg2c01cle7Bz3VVmwlEiM6QoL4G6
	# knOMiHC6mSkyIzg1q2Q3UkqlokJ8UQy9KbUsc3NH9Yug2ghGM5esmyUlqXcn3bSqvBGJZhFD
	# SsCJVGTAIsJ7M+OFixcqXHz5S1/6lVd/5eNbH1++fHkzTQDO5rMf//RHx0fHTrt86aR2Hckv
	# fO7zL7/40u07t6c+nZyccCUE8fU//fp2s9lsto9dvVqx++YHN3rr0zTduHHj3v17x0fHAP7x
	# 7/7ePM8f3f4YwNWTK9NmA+nzn/vclStX/vJbf7k/2jnszdffePG5F0Bcf+zx//G//x/ee/8X
	# Trt+7fqnKc6bH96sFrfCClhLz9VMQRjMQga5KpNPKAxDWTDpLMbZSldGuuB0xUo8KrJk+dVl
	# MnIZYxazQH3v3cykqJoruWKUtZCpMGNzL3l/s+rTW+H8KiJIUXDvJA1mpNO795XAzFrbT7LL
	# w6FaX0SiNLsmsJoI3KlERqN3a2uS4UimmoVhLevSkJq6kwIUpTpUdlij4VNa5LpOd18FasZV
	# ZAmvlrUCY2b1L6M5jKkAZVPf9raBeHZ6+ItvfWs+HCpVn6b+5PUnNtOmYs2f/bs/g3K33Uho
	# vWAu/u4/fY/ktJmuX7t++fJlAAAPZ4ev/+nXb9788Hh3oVv35hVw5/lQLhvGv/yrb8/zXEax
	# 2WyeuPbEk9efmKapZPv37t//5je/+eDhg8O8CBxzfPvb3z4nUfHUE09ev34dwNnZ2e07t8tf
	# HA4zxN42RgNsjGpYLCITIsIsG5E5mEtzTl69A2vjSjOvjKxBjdYAnWdDVdRcKfLmBYdRoxB0
	# ToJ/SlbwSGVh68JkZgjRy7jGMOkRZeDuIYyUEhmrUqWyucMyF8Z6lOhlCalbKzGonxf4KgOo
	# wRUVSoofN6Gbb1pHpMMbmwJjjrUUQ6/xCAZDyAI5kglvPVLF+VfHjrfNCICt5l4UiTXGSEIm
	# bzQzCpRt3DqLKKFVIwhA48OHD//oj/+3115/rVBRPbHbd27/4R/94U9+8uPj4wubzeb6eRHm
	# xgcf/PGf/PF3//a7/NScmu//l//8L/7lv/jRj988uXTx8qWT7Xb75PUn6uHfuXPbe4Nbb376
	# 8OG/+aN/89rrrx0OB6GcLkHevnvnz//iW3/wL/+n23dvbXaTTV1ubP0XNz789ne+fe/+PZyX
	# K95592d/8C//4NI5y3r3zt3NtHEyxkhltTgA4OHe3Tf//Z89vH3r9N6D5XA4fXCWtZBuy5zp
	# lDggAO6MiNXyhXkJuVe4HGMMUDSYLefjBgi3R8MnViFe8Yr+6CpLNwLY+hNy7YZ7lF+QS4xP
	# MrI6ZizuPo9lpfWxShLXTmWWvr46DcPd4V7ZXFUqSR7qLGs3sEWEWRtj9GbLslgZa1NmosoF
	# ieaeGSagtYiAOYBDzlPrmYlAq75qRaknMpPNgFximLXqgiezZPdGLydh7hExxjLP8/0HDx6e
	# Pji5dCLp9r279+/f3202+/3+wtF+s9ldunT5a1/7GoB3fvb2//yv/pfjo+Oj/f7k0klkvP3O
	# 26333Xa722wv7I52x8cRcTYfPr718RhLb+34+MJ+u5s2E0Lzsjx4+ODWvTuXLl5q3lJ58+bN
	# eZ679/1+v9vtjvZHbbuFRBnBh6d37927u9vvSN6/f//+/fu9urN6z4z9bn/h6MKlk0v7/f7C
	# hf2FiydHly783j/9/QahRr4QMqA5ImKJtSeo06PGp2CdMkAhc9D71D1pBTGam1Xvl4JSpzs9
	# DBlpphiRsJIIu3eujXgBGGDuLWLO1Dwv2+3WbEVjJVgHMLXzqBcjMSQKI0unqixjUslISlnv
	# q6RLkjpphrUNRCPCXMZWetfiI6CsgU1FLqyaVaEnnVYjFQSJVtRui7BcJ0rUDmAs7h0Id5sP
	# IzMa3d2rBuBsq5aSFM0aILpZZjgog9Mgx9Qv8Hi72RzmQ2ZuNpvtZtt7202b3a4DduXKlcrR
	# Prh5c7/dXTy+sNtuz07PRoxrVx83t91+v+2Tm1fx6mi3N9q8HMzseH/kzR2WyI2Zu7tPkXE4
	# O4uISxcuFtWy3+1ba9M01bahQOMRdt394enp2XLYTtvd1e2me7n0iJj6Zpqm5u7mZR1u5+Dd
	# 2My8SrY5UpBTRlfTIjVnKpdUBEoHX4o/AEQalQiglAv05kgwF66tnGs4a6RyJboK9JUTWhaN
	# MaRs3dwJZHdXUpFI1W4Ws6TlBEi4Wwpuzu6EDVSvCM0cNiRBBmkeA8jWfYmRI2jepg5jIrG2
	# OddYGUbEPM/eemWsrTnFMcaY08ys9yK5Ygxx7UxViKF1qMZINs8c7ixOoeAgyJZOcuFCMhQA
	# qJIriWQ3XzLGfNamToeB3Xqf+rTZRKSoqXVaolIhVFukABwOZ9tNP9rvdts9hMgsma6bozkE
	# QwuIyu00te6bPpU7t2aZcnfILl2YYlnOllaFVHeXtNlsQumth6gIAS72Prk369N2Gc0NzN4d
	# igwsh3l3dEywm6/zXcTIlMqwIAe7T2g5esY8t96DANiTdBspN4vK5mzVr5GUaL1XN1koZTzE
	# kKWZkzTahjYrACTNm5ksUm4emc4uaZoiMqQJYJ+amwPp3ZiMDJHWXNXoUPR0pLvPSvNV7rJr
	# W0lCNveRGRk0xYhp2yNlxuY9+tzYU+m9i11Sqxo5aYK15tNa5jIwMow5TXROywgn6Q1Ms75y
	# tkBrLTIjAk4JzbsY8CkxNhtr5vNyRjItzIwm9+bRSU8bSGWie49YttPmbGGzLkRvUygzdLTv
	# SuYYvfuCKMHWYT5cufIYANJu3b7d+tR63+637r6OfzIs80B3BFrrrJ5qIHIUdzNNZV4JttYd
	# HNN2M8WG5w1XMDezw+EhqG1rKLck0ZBh02abSiMio7cOWObQvrpM5NOG5t4mb627E2jr9mmt
	# 9R4Zux1L6m0qZG0jo9XQmVzbvNxbiu5WQpBRTXws4VRbIVQyCbrhbFky9tsdgDlmB6a2GctS
	# Y8vMpuIyjE0o8UJEJKnuHW4As/hxyN2rb6cUrxZR8wHNbB0HFdHoCrTNRLcYq+hg6xszi1BE
	# +tQiwsxzRJ9ajiDUzeiIyGZYFtZgD6NPG8uUkt64LMMMEtxNEiM3m2kdaZFo1tm8DVqiube+
	# LyxIUpxIy5GQiY2NGdncWtsYKGNrnuoATKmUmyll3qfWHHULCfDBwwdFz8zzfHR8vD/ab3f7
	# HOEbJyhou9uCZPUz0esuIt3XzhcT0fdbusVQdek1Oy+rV59+c7c9zWogW3VLe/MMmDET5joc
	# DpUt9TYVbJXgzfu0MbNHtdIGkt7o7pveDZyXbW/0FiECovcMmEEY0PnMOyZEeioj1L1VU2hk
	# dGuQaiqgYLTWJ6QUFISJvcopfXJxNdyiHcwI1Wi/DSkqAMuEmy9Z3W/ZWn0ZboTYWwF8A1AT
	# Bru8JPaEA8gEnRLsk+Ik2WxZhnvLMczIiaFwb5FL73DjZgtnqxNVggORps3a8YPWvMaUmDmM
	# MYYg95aERnBt8asEpZnZqAaAjZUImKRiNLMIGdiZpJckSzBBzZkRNSazV8oXw1t799137ty7
	# ezibvfVLJ5eOjo4207bgAYQadFLTBZqhOjUyahhkCwjVoWUOsfw6qj3fPDIQMJOXQE4aubg5
	# tBbcJJV+xZtPm41ZtRrU8KYFYEKbzbTdbXe73Xa7LWkm2nYjhbVmh4OPYWPYyvFQlelQkemi
	# 2dpVLNW0FmSKZnAbI7M+po2Rbg2mMQLNJAUCZLU1jAzIapJnjQcSCq8wIlbWsyaLRJJeI8Iy
	# 093AjCG6KwGFoGpIrClZhddrNiZhKfF8GlZ1LEaEbO2DM19F7JHR3BNJchW3wjJl7hlpZpnK
	# zBXvu0GIrB5Rmtva01GDQCIeDXutkT/n/EZ1QFXfEFNhoARFwl0C3XQ+38bbOrsxItzbOpKU
	# nMdhf7yXkJm996lPbK1Xl7zW0TSJHGNp5gXLMlbMQFhkzZ+wCFV3Z5HH5/MQS+OV9IZqe0pl
	# qCB8Sa/MjG7lNaU0WGbQJqKoyLbZbqbN5M0BNpA2TZ5Ja77dRoya1iRJSZ23OwPMlJklszKs
	# or+gGv3IKmrnObNQwxEjUzRJYmJdaSshx2qgq/5xGNzdYiRs1U4VE2bGjBo3sjbERiRs1Wec
	# j6KrJZE1lutX7QqtbdlV2aCVbKpm26yzTMysxvmaUdDaC8BzfJ/Zequj5aMZYOtMjZW5qEY0
	# M6sxtTURgzVidt07zCzRvIwoNRkJJlKVoKiwM4l6yGWbmXGuayoWhYfTh6XQ99a4AtliQ0rT
	# UH3b1dUtaN1gQHFyRd8Aa20eoDJylY7RMoOEwJTgQFTibDW2FKjJr7W9pBSkXMdXwZqbWZ+m
	# 1nza/N+1nL0OwCAIhPmmvv/DtkMXOsgR0KZxKZM/eF6isKB3ANzXGQ9U3UOU23ZNniiLta5F
	# 8CL15m+Q2cOjNoPEvZte92/mb1xWH3qjzaae+cYW1TnQRp1YwMJ36WizQ29ih81DlSABXhaM
	# e0CWbuIw80u+mdJY5ZOBB/YAN29iC4BdiCQAAAAASUVORK5CYII=
__EOD__
}
1;
__END__
